\\ Copyright 2020, 2021, 2022, 2023 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ This is various tests which are either a bit slow or which depend on OEIS
\\ data available, or both.

default(recover,0)
default(strictargs,1)
default(parisizemax,500*10^6);
read("../interv.gp");
read("../interv-wip.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");
option_verbose=1;

\\ A224820  Harmonic r(n,m) thing
\\ r(n,1) = n
\\ r(n,2) = k with  H(k) - H(n) > 1/n
\\ r(n,m) = k with  H(k) - H(r(n,m-1)) > H(r(n,m-1)) - H(r(n,m-2))
\\ r(n,m) = k with  H(k) > 2*H(r(n,m-1)) - H(r(n,m-2))
\\ b224820.txt

\\ row n=4
\\ 1/4 < 1/5 + 1/6 < 1/7 + 1/8 + 1/9 < 1/10 + ... + 1/14 < ...

bits = 96;

add_frac(t,k) =
{
  interv_widen_bits(interv_add_constant(t, 1/k),
                    bits);
}

\\ mathworld, bounds
\\ 1/(2(n+1)) < H(n) - log(n) - gamma < 1/(2n)
\\ 1/(24(n+1)^2) < H_n - log(n+1/2) - gamma < 1/(24n^2)

\\ FIXME:
\\ Calculate H by some faster method and bisect the estimates.

Harmonic_t_list = List([ [1,1] ]);
Harmonic_t(n) =
{
  my(i);
  while((i=#Harmonic_t_list) < n,
        listput(Harmonic_t_list,
                interv_widen_bits
                (interv_add_constant(Harmonic_t_list[i], 1/(i+1)),
                 bits)));
  Harmonic_t_list[n];
}
Harmonic(n) = sum(i=1,n,1/i);
{        
  for(n=1,500,
     my(h=Harmonic(n),
        t=Harmonic_t(n));
     interv_contains(t,[h,h]) || error());
}   

A224820_target = List([]);
A224820_k      = List([]);
A224820_cache  = List([]);
A224820_next(n) =
{
  while(#A224820_target < n,
        listput(A224820_target, [0,0]);
        listput(A224820_k, #A224820_k));

  my(k=A224820_k[n]);
  my(t=[0,0]);
  \\ print("n="n" at k="k" target ",A224820_target[n]*1.0);
  while(interv_lex(t,A224820_target[n]) <= 0,
        t = add_frac(t,k++));
  A224820_target[n] = t;
  A224820_k[n] = k;
  \\ print("  result k="k);
  k;
}
A224820(n,m) =
{
  while(#A224820_cache < n,
        listput(A224820_cache,List([])));
  while(#A224820_cache[n] < m,
        listput(A224820_cache[n], A224820_next(n)));
  A224820_cache[n][m];
}
{
  check_equal(vector(10,n, A224820(n,1)),
              vector(10,n, n),
              "column m=1");
}
for(n=1,4, for(m=1,4, printf(" %5d",A224820(n,m))); print());
{
  my(H=Harmonic);
  check_equal(H(9841) - H(1093) > H(1093) - H(364), 1, "9841");
  check_equal(H(3280) - H(1093) > H(1093) - H(364), 1, "3280");
}
matrix(8,9, n,m,A224820(n,m))
{
  my(want=[    1,  4,  13, 40 /*20*/,  121, 364, 1093, 3280 /*9841*/ ;
               2,  4,  8,  16,  32,  64,  128,  256;
               3,  5,  9,  16,  29,  53,  97,   178;
               4,  6,  9,  14,  22,  35,  56,   90;
               5,  7,  10, 15,  23,  36,  57,   91;
               6,  8,  11, 16,  24,  36,  54,   81;
               7,  9,  12, 16,  22,  31,  44,   63;
               8,  10, 13, 17 , 23,  32,  45 ,  64]);
  my(got=matrix(matsize(want)[1],matsize(want)[2], n,m,
                     A224820(n,m)));
  check_equal(got, want, "example table");
check_equal(want,[
   1,   4,  13,  40, 121, 364, 1093, 3280;
   2,   4,   8,  16,  32,  64,  128,  256;
   3,   5,   9,  16,  29,  53,   97,  178;
   4,   6,   9,  14,  22,  35,   56,   90;
   5,   7,  10,  15,  23,  36,   57,   91;
   6,   8,  11,  16,  24,  36,   54,   81;
   7,   9,  12,  16,  22,  31,   44,   63;
   8,  10,  13,  17,  23,  32,   45,   64]);







  my(row(n,len) =
     my(h=Harmonic(n), target=h+1/n, k=n);
     vector(len,m, if(m==1,k,
                      my(new_h);
                      while((new_h=Harmonic(k++)) <= target, );
                      target = 2*new_h - h; h = new_h; k)));

  my(want=OEIS_data("A224820",'bfile),
     i=1);
  for(d=1,oo,
     for(n=1,d, my(m=d-n+1);
        check_equal(A224820(n,m), want[i],
                    Str("r("n","m") i="i" want "want[i]));
        i++));
  \\ check_equal(OEIS_antidiagonals_of_func(#want, A224820, 1,1,1),
  \\             want,
  \\             "vs DATA");

  my(row(n,len) = vector(len,m, A224820(n,m))); 
  check_equal(row(4,8), [4,6,9,14,22,35,56,90], "row 4");
  check_equal(row(1,8), [1,4,13,20,121,364,1093,9841], "row 1");

}
kill(Harmonic_list);
quit;



