\\ Copyright 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
default(parisizemax,300*10^6);
read("../interv.gp");
read("../interv-wip.gp");
read("../test-helpers.gp");

option_verbose=1;
option_progress = 1;



{
  my(block(k) = concat(apply(digits,[10^k .. 10^(k+1)-1])));
  \\ print(vector(2,k,fromdigits(block(k))));
  my(term(k,i) = i*10^(k*i));
  print(vector(12,i,term(1,i)));
  \\ A126431 a(n) = n * 10^n.
  print(vector(12,i,sum(i=1,i, term(1,i))));
  \\ A052245 Expansion of 10*x / ((1 - x) * (1 - 10*x)^2) in powers of x.
  quit;
}

\\ code by Davis Smith in A353962
{
  A353962_rows(n) =
  my(C=suminf(i=1,
              i/(n^(i+sum(k=1, i-1,
                          logint(k+1,n))))),
     L=List());
  for(n=1, default(realprecision)-1,
     C*=10;
     listput(L,floor(C%10)));
  Vec(L);
}

\\ second code by Davis Smith in A353962
A353962_rows(n, k) =
{
  my(K=k+1, C=vecsum(vector(K, m, m/(n^(m+sum(x=1,
               m-1, logint(x+1, n)))))));
  vector(K,m, floor(C*10^m)%10);
}

\\ third code by Davis Smith in A353962
A353962_rows(n, k)=
{
  if(default(realprecision) <20080,
     default(realprecision, 20080));
  my(K=k+1,
     C=suminf(m=1,
         m/(n^(m+sum(x=1, m-1, logint(x+1, n))))));
  vector(K, m, floor(C*10^m)%10);
}

\\ fourth code by Davis Smith in A353962
A353962_rows(n, k) =
{
  if((k<20080),
     if((default(realprecision)<20080),
        default(realprecision,20080)),
     k>=20080, default(realprecision, k+100));
  my(K=k+1,
     C=suminf(m=1,
        m/(n^(m+sum(x=1, m-1, logint(x+1, n))))));
  vector(K, m, floor(C*10^m)%10)
}

Champernowne_interval(base,len) =
{
  my(c=[0]);
  for(n=1,oo,
     if(#c>=len,break);
     c=concat(c, digits(n,base)));
  interv_frac_fromdigits(c,base);
}

{
  \\ base 10^6
  my(base=10^5);
  print("base "base);
  my(t_got=Champernowne_interval(base,10));
  my(want=interv_frac_digits(t_got)[^1]);
  print("want ",want);
  print("want len ",#want);
  my(cod=A353962_rows(base,6));

  print("cod len ",#cod);
  if(#cod < 20, print("cod  ",cod));
  print("equal: ",cod==want[1..#cod]);
  for(i=1,#cod,
     if(cod[i]!=want[i],
        print("different starting at i="i);
        break));
  for(i=max(1,#cod-10),#cod,
     print("i="i"  ",cod[i]," want ",want[i]));
  quit;
}

{
  \\ decimal
  print("decimal");
  \\ my(t_got=Champernowne_interval(10,21000));
  \\ my(want=interv_frac_digits(t_got)[^1]);
  my(want=concat(vector(8000,i,digits(i))));
  print("want len ",#want);
  my(cod=A353962_rows(10,20083));
  print("cod len ",#cod);
  print("equal: ",cod==want[1..#cod]);
  for(i=1,#cod,
     if(cod[i]!=want[i],
        print("different starting at i="i);
        break));
  for(i=#cod-10,#cod,
     print("i="i"  ",cod[i]," want ",want[i]));
  quit;
}
{
  \\ A066716 binary
  print("binary");
  my(want=OEIS_data("A066716"),
     t_want=interv_frac_fromdigits(concat(0,want)));
  print("t_want ",t_want*1.0);
  
  my(t_got=Champernowne_interval(2,100000));
  check_equal(interv_contains(t_want,t_got), 1);
  my(new_want=interv_frac_digits(t_got)[^1]);
  want==new_want[1..#want]  || error();
  want = new_want;
  print("new want len ",#want);

  my(cod=A353962_rows(2,19999));
  if(#cod<50,print("cod ",cod));
  print("cod len ",#cod);
  print("equal: ",cod==want[1..#cod]);
  for(i=1,#cod,
     if(cod[i]!=want[i],
        print("different starting at i="i);
        break));
  quit;

  \\ forstep(k=10,10,-1,
  \\    print("target k=",k);
  \\ my(cod=A353962_rows(2,k));
  \\ print("cod  ",#cod, " len  ",cod);
  \\ print("want ",#want," len  ",,want[1..#cod]);
  \\ if(cod!=want[1..#cod],
  \\    print("WRONG");
  \\    quit);
  \\ print();
  \\ );
}
quit
{
  \\ A077771 ternary
  print("ternary");
  my(want=OEIS_data("A077771"),
     t_want=interv_frac_fromdigits(concat(0,want)));
  print("t_want ",t_want*1.0);
  
  my(t_got=Champernowne_interval(3,1000));
  check_equal(interv_contains(t_want,t_got), 1);

  my(cod=A353962_rows(3,20));
  print(cod);
  print(want[1..#cod]);
  print();
}

{
  my(n=4,
     t=Champernowne_interval(n,1000),
     v=interv_frac_digits(t,,20));
  v[1]==0 || error();
  v=v[^1];
  print("base "n" is ",v);
  print();
}

{
  my(bad=0);
  for(n=2,25,
     for(len=10,50,
     my(cod=A353962_rows(n,len),
        t=Champernowne_interval(n,1000),
        t_vec=interv_frac_digits(t,,#cod+1)[^1]);

     if(t_vec != cod,
        print("wrong n="n" len="len);
        print(cod);
        print(t_vec);
        if(bad++>4,quit);
       )));
}

{
  my(n=23,
     lo=A353962_rows(n));
  default(realprecision,1000);
  my(hi=A353962_rows(n));
  hi=hi[1..#lo];
  check_equal(lo,hi);
}


\\-----------------------------------------------------------------------------

{
  \\ suminf calls
  default(realprecision,200);
  my(n=10,
     prev=0,
     s=suminf(m=1,
         print("m="m,if(m==prev+1,"","    ******"));
         prev=m;
         m/(n^(m+sum(x=1, m-1, logint(x+1, n))))));
  print(s);
  quit;
}

\\-----------------------------------------------------------------------------
