\\ Copyright 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
read("../interv.gp");
read("../interv-wip.gp");
read("../test-helpers.gp");
option_verbose = 1;

phi=quadgen(5);

phi_frac_fromdigits(v) = sum(i=1,#v, v[i]*phi^-i);
phi_frac_digits(x) =
{
  x<1 || error();
  my(v=vector(20,k,
              if(x >= phi^-k, x-=phi^-k;1, 0)));
  x==0 || error("phi_frac_digits() still ",x" ",x*1.0);
  while(#v&&v[#v]==0, v=v[^#v]);
  v;
}


\\-----------------------------------------------------------------------------
\\ Rationals Expansions Base Phi

read("vecper.gp");
{
  forstep(den=3,17,1,
     my(f=1/den,
        t=[f,f+1/phi^40]);
     while(t[1]<1, t*=phi);
     my(v=interv_frac_digits(t,phi),
        s=concat(apply(d->Str(d),v)));
     printf("%4s  %s\n", f,s);
     my(v=vecper_frac_digits(f,phi));
     print("   ",v);
     vecper_frac_fromdigits(v,phi) == f || error();

  );

  \\ A001175 Pisano periods, Fibonacci mod n
  print(vector(20,n,n+=4; vecper_periodic_length(vecper_frac_digits(1/n,phi))));;
  print(concat(vector(5,n,n+=2; vecper_periodic(vecper_frac_digits(1/n,phi)))));;
  quit;
}

\\-----------------------------------------------------------------------------

\\ code by Chittaranjan Pardeshi in A354239
alist(len) =
{
    my(phi = quadgen(5), pi=4/phi, v=List(), n=0, z=0, o=0, t=0);
    while ( \\ t==0 || error("t="t);
           n <= len || t <= len+3,

        n += 1;
        if (n%12 == 1,
            pi += 8*phi^-3  / (n+2)
                + 4*phi^-5  / (n+4)
                - 4*phi^-7  / (n+6)
                - 8*phi^-9  / (n+8)
                - 4*phi^-11 / (n+10)
                + 4*phi^-13 / (n+12));
        pi *= phi;
        if ( phi^2 <= pi,
            pi -= phi^2; o = o + 1; listput(v, 1),
            z = z + 1; listput(v, 0); if(v[n-1] == 0 && v[n-2] == 1, t = n));
        if (t > 0 && z > o,
            v[t] = 1; v[t-1] = 1; v[t-2] = 0;
            z = z - 1; o = o + 1; t = 0; ));
    return(Vec(v[1..len]))};
alist(14)
alist(20)

print();
v=alist(40);
v=[1, 1, 0, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1]
d=apply(t->if(t==1,1,-1),v);
for(i=2,#d, d[i]+=d[i-1]);
d
Set(d)
quit

1000100101010010001010101000001010010000100101000
 110100101010
             010001010101000001010010000100101000


10000010101010101000000100001010000100001010001001001000]

g=OEIS_data("A102243")
quit


is_lengths(v) =
{
  my(o=vector(#v+1),
     z=vector(#v+1));
  for(i=2,#v+1,
     o[i]=o[i-1] + (v[i-1]==1);
     z[i]=z[i-1] + (v[i-1]==0));

  for(i=1,#v,
     for(j=i,#v,
        my(sub_o = o[j+1]-o[i],
           sub_z = z[j+1]-z[i],
           len=j-i+1);
        \\ sub_o == vecsum(v[i..j]) || error();
        \\ sub_z == sum(p=i,j,v[p]==0) || error();

        delta = abs(sub_o - sub_z);
        if(delta > if(len%2==0,2,3),
           return(0));
        ));
  1;
}

{
  my(v=alist(1000));
  print("is_lengths ...");
  print(is_lengths(v));

  for(i=1,#v-20,
     if(v[i]==1 && v[i+1]==0 && v[i+2]==0,
        my(u=v);
        u[i]=0;    u[i+1]=1;    u[i+2]=1;
        \\ print("try ",i"  "#u);
        if(is_lengths(u),
           print("100 -> 011 at i="i);
           quit));
     if(v[i]==0 && v[i+1]==1 && v[i+2]==1,
        my(u=v);
        u[i]=1;    u[i+1]=0;    u[i+2]=0;
        \\ print("try ",i"  "#u);
        if(is_lengths(u),
           print("011 -> 100 at i="i);
           quit)));
 }
vector_end_diff_length(u,v) =
{
  my(len=min(#u,#v));
  for(i=0,len-1,
     if(u[#u-i]==v[#v-i], return(i)));
  len;
}
vector_end_diff_length([1,2],[3,4])
vector_end_diff_length([1,2],[1,4])
vector_end_diff_length([1,2],[1,2])
{
  my(m=0);
  for(len=0,3000,
     my(got=alist(len),
        longer=alist(len+10)[1..len],
        diff=vector_end_diff_length(got,longer));
     if(diff>m,
        print("wrong len="len"  diff="diff);
        print(got);
        print(longer);
        \\ m=diff;
        ));
  quit;
}

\\-----------------------------------------------------------------------------

default(realprecision,1000);

\\ code by Chittaranjan Pardeshi in A102243
alist(len) = {
    my(phi=quadgen(5), n=-1, pi=4/phi, gap=phi^3, hi=pi+gap, t=0, w=phi^3);
    vector(len, i,
        pi < Pi || error("lo");
        hi > Pi || error("hi");
        w = w/phi;
        while(t+w < hi && t+w > pi,
            n   = n + 1;
            pi += phi^(-12*n) * (
                    8 * phi^-3  / (12*n+3)
                  + 4 * phi^-5  / (12*n+5)
                  - 4 * phi^-7  / (12*n+7)
                  - 8 * phi^-9  / (12*n+9)
                  - 4 * phi^-11 / (12*n+11)
                  + 4 * phi^-13 / (12*n+13));
            gap /= phi^12;
            hi   = pi + gap);
        if( t+w <= pi, t += w; 1, 0))};

{
  my(want=OEIS_data("A102243"),
     got=alist(#want));
  check_equal(got,want);
}
{
  \\ think initial gap as low as phi^0

  my(v=alist(1000));
  default(realprecision,30000);
  my(g=phi_frac_fromdigits(v)*phi^3);
  printf("%.10f\n", Pi);
  printf("%.10f\n", g);
  printf("%.10g\n", Pi-g);
  printf("%.10g\n", (phi+.0)^-(#v));

  \\ 4./phi
  \\ Pi - 4/phi
}

{
  my(n=0);
  s=(               8 * phi^-3  / (12*n+3)
                  + 4 * phi^-5  / (12*n+5)
                  \\ - 4 * phi^-7  / (12*n+7)
                  \\ - 8 * phi^-9  / (12*n+9)
                  \\ - 4 * phi^-11 / (12*n+11)
                  + 4 * phi^-13 / (12*n+13));
  print("s = ",s);
  print(phi^-1);
  print(phi^-2);

  print("sum factor ",sum(i=0,10,phi^(-12*i)));
  print("summed ",s*sum(i=0,10,phi^(-12*i)));

  quit;

  \\ n=0
  \\ 8 * phi^-3  / (12*n+3)
  \\ 4 * phi^-5  / (12*n+5)
  \\ 4 * phi^-13 / (12*n+13)
  \\ 8/3.
  \\ phi^3
}
quit;

\\-----------------------------------------------------------------------------

v=[1,0,0,0,1]
x = phi_frac_fromdigits(v)
print("x    = ",phi_frac_digits(x));
print("rem  = "phi_frac_digits(x % phi^-4));
print("diff = ",phi_frac_digits(x - x % phi^-4));

print();
n = phi^-1 + phi^-12;
n = phi^-1
phi_frac_digits(n)
r = n % phi^-6
phi_frac_digits(r)
phi_frac_digits(n-r)
print();

n = phi^-12;
r = n % phi^-6
r == phi^-12 || error();
print();

n = phi^-1 + phi^-12;
r = n % phi^-6
r == phi^-12

(phi^-1 + phi^-12) % phi^-6 != phi^-12

quit


\\-----------------------------------------------------------------------------
precise(low, hi, base, prc) =
{
  if(low - low % base^prc == hi - hi % base^prc,
     1, 0);
}

phi_digits(x) =
{
  my(k=0);
  while(phi^k <= x, k++);
  my(v=vector(k+1,i, k--;
              if(x >= phi^k, x -= phi^k; 1, 0)));
  x==0 || error(x);
  v;
}
phi_fromdigits(v) = subst(Pol(v),'x,phi);
x = phi_fromdigits([1,0,0,0,1,0])
phi_digits(x)
phi_digits(x % phi^2)
phi_digits(x - x % phi^2)
print();


quit


\\-----------------------------------------------------------------------------
\\ || pi % phi^(-12*n-9) >= phi^(-12*n-12),

whole = sum(i=1,10, phi^-(2*i))
hi = sum(i=1,3, phi^-(2*i))
r = whole % phi^-6
d=whole - hi
interv_frac_digits([whole,whole], phi, 18)
interv_frac_digits([hi,hi], phi, 18)
interv_frac_digits([d,d], phi, 18)
interv_frac_digits([r,r], phi, 18)

hi / phi^-6
quit

pi = 22/7 / phi^3;
pi = sum(i=1,10, phi^-(2*i))
pi*1.0
interv_frac_digits([pi,pi], phi, 18)

p= pi % phi^-3
interv_frac_digits([p,p], phi, 18)

quit


\\-----------------------------------------------------------------------------

alist(len) =
{
  my(phi=quadgen(5), pi=4/phi, n=-1);

  while(n <= ceil(len/12)+1
        || pi % phi^(-12*n-9) >= phi^(-12*n-12),
        n = n + 1;
        pi += phi^(-12*n)
             * (8 * phi^-3  / (12*n+3)
                + 4 * phi^-5  / (12*n+5)
                - 4 * phi^-7  / (12*n+7)
                - 8 * phi^-9  / (12*n+9)
                - 4 * phi^-11 / (12*n+11)
                + 4 * phi^-13 / (12*n+13)));

  vector(len, i,
         pi *= phi;
         if(phi^3 <= pi,
            pi -= phi^3; 1, 0));
}



\\-----------------------------------------------------------------------------
\\ 0101010101

s = sum(i=1,10, phi^(-2*i));
s + 0.

interv_frac_digits([s,s],phi, 20)

quit

\\-----------------------------------------------------------------------------
\\     8 * phi^-2  / (n+2)
\\   + 4 * phi^-4  / (n+4)
\\   - 4 * phi^-6  / (n+6)
\\   - 8 * phi^-8  / (n+8)
\\   - 4 * phi^-10 / (n+10)
\\   + 4 * phi^-12 / (n+12)));

\\ my(n=0); 8/( (12*n+3)  * phi^2 ) + 0.0
\\ 1.01857
\\ phi^12
\\ 321.9968

bbp_term(n) =
{
  1/phi^(12*n)
  * (  8/( (12*n+3)  * phi^2 )
     + 4/( (12*n+5)  * phi^4 )
     - 4/( (12*n+7)  * phi^6 )
     - 8/( (12*n+9)  * phi^8 )
     - 4/( (12*n+11) * phi^10)
     + 4/( (12*n+13) * phi^12)
    );
}

bbp_term(0)*1.0
bbp_term(1)*1.0
suminf(n=0, bbp_term(n) +0.0)
4+suminf(n=0, bbp_term(n) +0.0)
Pi*phi

fdig(x) = interv_frac_digits([x,x],phi, 3*12+3);

p12(v,bar='none) =
{
  for(i=1,#v,
     my(q,r); [q,r]=divrem(i-2,12);
     if(r==0,print1(if(q==bar+1,"-"," ")));
     print1(if(v[i],v[i],".")));
}

{
  if(1,
     print1("shifted\n ");
     p12(fdig(bbp_term(0)*phi^2));
     print("terms");
     for(n=0,4,
        p12(fdig(bbp_term(n)));
        print());
    );
}

{
  if(1,
     print("summed");
     for(n=0,5,
        p12(fdig(sum(i=0,n,bbp_term(i))),
            n);
        print());
    );
}

{
  my(s=bbp_term(0));
  for(n=0,10000,
     my(to = s + bbp_term(n+1),
        fexp=12*(n+1) + 1,
        f=phi^fexp);
     my(one=floor(s*f),
        two=floor(to*f));
     if(n==0,
        print("f = ",f,"  fexp = ",fexp);
        print(one);
        print(two);

        my(three=floor(s*f*phi^12),
           four=floor(to*f*phi^12));
        print("cf ",three);
        print("   ",four);
       );
     if(one!=two,
        print(n);
        );
  s=to);
}

\\
\\ 0
\\ 713

\\ 12n+3
\\ 0
\\ 1
\\ 3
\\ 4
\\ 8
\\ 10
\\ 18
\\ 19
\\ 68
\\ 78
\\ 126
\\ 614
\\ 1340
\\ 1620
\\ 2877
\\ 3615
\\ 4964
\\ 8179
