\\ Copyright 2020, 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
default(parisizemax,500*10^6);
read("../interv.gp");
read("../interv-wip.gp");
read("../test-helpers.gp");

print("===============================================================\n\n");

option_verbose=1;
option_progress = 1;

contfrac_to_rational(v) = {
  \\ contfracpnqn() gives matrix [ p[n], p[n-1];
  \\                               q[n], q[n-1] ]
  \\ where p[n]/q[n] is the last partial quotient,
  \\ and p[n-1],q[n-1] is the second last (unused here)
  my(m=contfracpnqn(v));
  m[1,1]/m[2,1];
}
fromdigits_any(v,base=10) = subst(Pol(v),'x,base);


\\-----------------------------------------------------------------------------
\\ A358197 high decimal digit of 2^n, 5^n, 8^n same

{
  my(high(n) = digits(n)[1]);
  my(thigh(t) =
     my(up=logint(t[1],10), vp=logint(t[1],10), c);
     if(up!=vp || (c=t[1]\10^up) != t[2]\10^vp,
        error("not enough precision"));
     c);
  my(pvec=[2,5,8],
     tvec=[ [1,1], [1,1], [1,1] ],
     limit=10^20);
  my(tmul(t,p) =
     t*=p;
     if(t[2] >= limit, t=interv_widen_integer(t/10));
     t);
  for(n=0,1000000,
     my(c=thigh(tvec[1]), good=1);
     for(i=2,#pvec,
        if(thigh(tvec[i])!=c, good=0;break));
     if(good,
        if(n<500, print1(n", "));
        if(n!=0 && c!=3, error(n" "c)));7
     for(i=1,#pvec,
        tvec[i] = tmul(tvec[i],pvec[i])));
  print();
  quit;
}     


\\-----------------------------------------------------------------------------
\\ sqrt(Pi) in Base Pi

interv_frac_digits(t,base=10,limit='none) =
{
  if(type(base)!="t_VEC",base=[base,base]);
  my(l=List([]), f);
  while(1,
        if(limit!='none && #l>=limit,break);
        my(f=floor(t));
        if(f[1]!=f[2],break);
        listput(l,f[1]);
        t = interv_sub(t,f);
        t = interv_mul(t,base));
  Vec(l);
}
t_pi = interv_frac_fromdigits(OEIS_data("A000796"));
t_pi*1.0
t_sqrt_pi = interv_frac_fromdigits(OEIS_data("A002161"));
t_sqrt_pi*1.0
v = interv_frac_digits(t_sqrt_pi,t_pi);

want=[1, 2, 1, 1, 0, 0, 2, 0, 1, 2, 0, 2, 1, 1, 3, 0, 0, 0, 0, 0, 2, 2, 1, 1, 1, 1, 0, 1, 0, 1, 3, 0, 0, 2, 0, 2, 0, 0, 2, 2, 1, 2, 1, 1, 1, 1, 0, 0, 2, 2, 2, 0, 2, 1, 1, 1, 0, 0, 2, 0, 1, 2, 2, 0, 0, 1, 2, 2, 0, 3, 0, 1, 1, 0, 1, 1, 2, 1, 0, 2, 2, 2, 0, 2, 0, 1, 0, 1, 0, 3, 0, 1, 1, 0, 1, 0, 0, 2, 2, 2, 0, 2, 1, 0, 0];

print("v    len "#v);
print("want len "#want);
v[1..#want]==want
print()

sum(i=1,#want, want[i]*Pi^(1-i))
\\ 1.7724538509
fromdigits_any(v,base=10) = subst(Pol(v),'x,base);
fromdigits_any(want,Pi) / Pi^(#want-1)
sqrt(Pi)

print()
print("pi base pi");
t = interv_div(t_pi,t_pi)
t*1.0
v = interv_frac_digits(t, t_pi, 10)

quit

\\-----------------------------------------------------------------------------
\\ base phi

phi=quadgen(5);
phi_digits(x,len) =
{
  my(p=0);
  while(x>1,x/=phi; p++);
  [p,
  vector(len,i,
         x<=1 || error();
         x*=phi;
         my(d=x>1);
         x-=d; d)
  ];
}
\\ v=phi_digits(1/2,20)
\\ fromdigits_any(v,phi)/ phi^#v

v=[1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0];
v=v[1..20];
t=interv_frac_fromdigits(v,2,2);

print("phi_digits ",phi_digits(t[1],10))


bt=interv_add_constant(t,-(2^1+2^0));
pt=interv_add_constant(t,-(phi^1+phi^0));
b=interv_frac_digits(t,2,#v);
u=interv_frac_digits(t/phi^3, phi, #v);
print("b  "b);
print("u  "u);

3.8/phi^3
print("diff "u-v);
quit


\\-----------------------------------------------------------------------------
\\ 1/2 in base phi

phi=quadgen(5);
interv_frac_digits([1/2,1/2],phi,100)
quit


\\-----------------------------------------------------------------------------
\\     1. set x = y = 0;
\\        high = ceiling(log(R, phi)),
\\        low = -p
\\     2.   for exponent e from high to low
\\     3.        if ((x+F(e-1))+(y+F(e))*phi)^2 
\\                  <= (X+Y*phi)/Z
\\     4.            output([e, 1])
\\     5.            x, y = x+F(e-1), y+F(e)
\\     6.        else
\\     7.            output([e, 0])

default(realprecision)
default(realprecision,50)
phi=quadgen(5);
make(X,Y,Z,p) =
{
  my(x=0,y=0, F=fibonacci);
  my(R=sqrt( (X+Y*phi)/Z) );
  my(high = ceil(log(R)/log(phi)));
  my(low = -p);
  my(ret=List([]));
  forstep(e=high,low,-1,
     printf("e=%3d gap %f\n", e,  R - (x+y*phi));
     printf("    phi   %f\n", phi^e);

     if( ((x+F(e-1))+(y+F(e))*phi)^2 
        <= (X+Y*phi)/Z,
        listput(ret,1);
        [x, y] = [x+F(e-1), y+F(e)],
        listput(ret,0)));
  print("final x="x" y="y);
  while(ret[1]==0,listpop(ret,1));

  my(d=R - phi^-p);
  print("final xy ",x+y*phi,"  ",x+y*phi*1.0);
  print("phi pow  ", phi^-p);
  print("phi pow  ", phi^-p*1.0);
  print("gap      ", R - (x+y*phi));

  R - phi^-p <= (x+y*phi)   || error("low");
  (x+y*phi) <= R + phi^-p  || error("high");
  (x+y*phi) <= R           || error("high");

  Vec(ret);
}
make(0,1,1, 15)

want=[1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0];
make(0,1,1, #want-1) == want
vector(20,n, 1/phi^n)


{
  my(R=sqrt(phi));
  my(p=10,
     m=Map());
  forvec(v=vector(p+0,i,[0,1]),
     my(t=sum(i=1,#v, v[i]*phi^(0-(i-1))));
     \\ print(v" "t);
     if(R - phi^-p <= t && t <= R,
        print(v);
        \\ print(" "t*1.0);
        mapput(m,t,1);
       ));
  print(#m);
  quit;
}

phi=quadgen(5)
phi^-1 + phi^-2 == 1

quit



\\-----------------------------------------------------------------------------
A352077  floor sum cube roots

a(n) = floor(sum(k=0, n, k^(1/3)));
avec(n) =
{
  my(t=0);
  vector(len,k, floor(t +=  k^(1/3)));
}
s = 2^64;
s3 = s^3;
bvec(len,root) =
{
  my(ret=vector(len));
  my(total=[0,0]);
  for(i=1,len,
     \\ print(total/s*1.);
     my(is3=i*s3);
     total = interv_add(total, interv_sqrtnint([is3,is3],root));
     my(t = total/s);
     my(f=floor(t[1]));
     if(f != floor(t[2]),
        error("too small interv range"));
     ret[i] = f);
  ret;
}
bvec(10,3)

want=[0, 1, 2, 3, 5, 6, 8, 10, 12, 14, 16, 19, 21, 23, 26, 28, 31, 33, 36, 39, 41, 44, 47, 50, 53, 56, 58, 61, 65, 68, 71, 74, 77, 80, 83, 87, 90, 93, 97, 100, 104, 107, 110, 114, 117, 121, 125, 128, 132, 136, 139, 143, 147, 150, 154, 158, 162, 166, 170, 173, 177, 181, 185, 189, 193, 197];

bvec(#want-1,3) == want[^1]  || error()

len=100000;
\\ vector(len,n,a(n)) == bvec(len,3)
avec(len) == bvec(len,3)
quit
  
  

\\-----------------------------------------------------------------------------
\\ A347139 contrfrac 0, 3, 3, 3, 4, 9, 3, 1, 2

\\ decimal A003673   . 0, 0, 7, 2, 9, 7, 3, 5, 2, 5, 6
\\ contfrac A082726  0, 137, 27, 1, 3, 1, 1
t=interv_frac_fromdigits([0,  0,0,7,2,9,7,3,5,2,5,6])
interv_contfrac(t)
\\ t=interv_recip(t)
\\ interv_contfrac(t)
t=sqrt(4*Pi*t)
interv_contfrac(t)
want=[0, 3, 3, 3, 4, 9, 3, 1, 2]
quit

sqrt(4*Pi * 0.00729735256)
sqrt(4*Pi * 0.00729735257)
0.30282212
0.3028221208866128882390981884
0.3028221206791252287862147723
0.30282212227549

interv_validate(interv_from_contfrac([1,2,3]));
interv_validate(interv_from_contfrac([1,2,3,4]));
interv_from_contfrac([1,2,3,1])*1.0
interv_from_contfrac([1,2,3,2])*1.0
interv_from_contfrac([1,2,3,999999999999999])*1.0
interv_from_contfrac([1,2,3])*1.0

t=interv_from_contfrac([0, 3, 3, 3, 4, 9, 3, 1, 2])
interv_validate(t);
interv_frac_digits(t)
\\ 0, 3, 0, 2, 8, 2, 2, 1
\\ 0.30282212

contfrac_to_rational([0, 3, 3, 3, 4, 9, 3, 1, 2])*1.0
contfrac_to_rational([0, 3, 3, 3, 4, 9, 3, 1, 2,1])*1.0
quit


\\-----------------------------------------------------------------------------

t = interv_exp([1,1],50)
c = interv_contfrac(t)
quit


\\-----------------------------------------------------------------------------
\\ interv_frac_digits()

v=Vecrev(OEIS_bfile_gf("A073821"))
lo=fromdigits(v)/10^#v;
hi=(fromdigits(v)+1)/10^#v;
lo*1.0
t=[lo,hi];
c=interv_contfrac(t)
check_equal(c,vector(#c,n,2*n-2));

clo=vector(2*#c,n,2*n-2)
chi=clo; chi[#chi]++;
ct=[contfrac_to_rational(clo),contfrac_to_rational(chi)];
ct*1.0
check_equal(interv_contains(t,ct), 1);
quit

\\------------------------------------------------------------------------------
