\\ Copyright 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Usage: gp <bisection.gp
\\
\\ Demonstrate a simple use of interv_solve_bisection().

default(recover,0);
default(strictargs,1);
read("../interv.gp");


\\ Find an interval in which a solution exists for
\\
\\     x^2 - 5 == 0
\\
\\ The starting interval is [0,10] and then asking interv_solve_bisection()
\\ for "bits" many bisections tightens that interval to somethign narrower
\\ surrounding the exact answer.
\\
\\ The answer of course is sqrt(5) in this case, but often it's not so easy
\\ and there's little better than to probe away with bisections to get
\\ closer to the answer.

bits=16;
t=interv_solve_bisection([0,10], (x)->x^2-5, bits);
print("sqrt5 by ",bits,"-bit bisection: ",t);
print("  which as floats ",t*1.0);

\\ PARI 2.13 allows comparisons between quads and rationals.
\\ The check here is that the quad sqrt5 is within the t interval.
sqrt5_quad = quadgen(20);
sqrt5_quad^2 == 5 || error();
interv_contains_constant(t, sqrt5_quad)  || error();


\\-----------------------------------------------------------------------------
print("end");