\\ Copyright 2019, 2020, 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Whether to print test progress and names.
if(option_verbose=='option_verbose, option_verbose=0);

\\ Whether to setrand(), default 1 = yes.
if(option_randomize=='option_randomize, option_randomize=1);


\\------------------------------------------------------------------------------

\\ test_is_gnil() from test_is_gnil.c compiled to ./test_is_gnil.so, or ../
\\ if running from devel subdir.
\\ If install() not possible or compile not possible, then fall-back to
\\ nothing is gnil.
test_is_gnil;
kill(test_is_gnil);
{
  if(! if(!default(secure),
          iferr(install(test_is_gnil,"G",,"./test_is_gnil.so"); 1,
                e, 0)
          || iferr(install(test_is_gnil,"G",,"../test_is_gnil.so"); 1,
                   e, 0)),
     test_is_gnil(x) = 0);
}

\\ Check that got===want.
\\ This is the strict type "===" operator so as to ensure say polynomial
\\ returns really are always polynomials, etc.
\\
\\ ENHANCE-ME: Would like to report caller's source file name and line
\\ number.  For now some identifying "name" parameter only.
\\
check_equal(got,want,name="") =
{
  my(bad=0);
  check_count++;

  if(test_is_gnil(got),
     print("oops, got==gnil at "check_count": "name);
     bad=1);
  if(test_is_gnil(want),
     print("oops, want==gnil at "check_count": "name);
     bad=1);

  if(type(got)=="t_VEC" || type(type)=="t_COL",
     for(i=1,#got,
        if(test_is_gnil(got[i]),
           print("oops, element got["i"]==gnil at "check_count": "name);
           bad=1;
           break)));
  if(type(want)=="t_VEC" || type(type)=="t_COL",
     for(i=1,#want,
        if(test_is_gnil(want[i]),
           print("oops, element want["i"]==gnil at "check_count": "name);
           bad=1;
           break)));

  if(type(got)=="t_MAT",
     for(i=1,matsize(got)[1],
        for(j=1,matsize(got)[2],
           if(test_is_gnil(got[i,j]),
              print("oops, element got["i","j"]==gnil at "check_count": "name);
              bad=1;
              break(2)))));
  if(type(want)=="t_MAT",
     for(i=1,matsize(want)[1],
        for(j=1,matsize(want)[2],
           if(test_is_gnil(want[i,j]),
              print("oops, element want["i","j"]==gnil at "check_count": "name);
              bad=1;
              break(2)))));

  if(! (got === want),
     print("wrong at "check_count": "name);
     print("got  "got);
     print("want "want);
     if(1||got==want,
        print("got type  "type(got));
        print("want type "type(want));
        if(type(got)=="t_POL" || type(got)=="t_RFRAC",
           print("got variable is  "variable(got)));
        if(type(want)=="t_POL" || type(want)=="t_RFRAC",
           print("want variable is "variable(want))));
     bad=1);

  if(bad,
     quit(1),
     option_verbose,
     print("ok "check_count" "name));
}

have_lex_of_quad=iferr(lex(quadgen(12),quadgen(12));1,e,0);


\\------------------------------------------------------------------------------
progress_total_time = 0;
progress(str) =
{
  if(option_progress,
     my(t=gettime());
     progress_total_time += t;
     printf("  %.1f seconds\n%s\n", t/1000, str));
}

\\------------------------------------------------------------------------------
nearly_equal_epsilon = 1e-10;
nearly_equal(x,y, epsilon=nearly_equal_epsilon) =
{
  my(d=abs(x*1.0 - y*1.0));
  if(type(d)=="t_MAT", d=vecsum(Vec(d)));
  if(type(d)=="t_VEC" || type(d)=="t_COL", d=vecsum(d));
  abs(d) <= epsilon;
}

\\------------------------------------------------------------------------------

test_contfrac_to_rational(v) =
{
  \\ contfracpnqn() gives matrix [ p[n], p[n-1];
  \\                               q[n], q[n-1] ]
  \\ where p[n]/q[n] is the last partial quotient,
  \\ and p[n-1],q[n-1] is the second last (unused here)
  my(m=contfracpnqn(v));
  if(m[2,1], m[1,1]/m[2,1], 0);
}

\\------------------------------------------------------------------------------
