/* Copyright 2019 Kevin Ryde

   This file is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This file is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along
   with this file.  See the file COPYING.  If not, see
   <http://www.gnu.org/licenses/>.
*/

#include <pari/pari.h>

/*
GP;install("test_is_gnil","G",,"./test_is_gnil.so");
*/

/* In GP, return 1 if x is the special gnil or 0 if not.
   gnil is 0 the same as gen_0 for all arithmetic purposes.
   But the GP interactive read-eval-print suppresses the print of gnil.
   Builtin functions and syntax which have no return value give gnil
   so as to print nothing interactively.

   User-defined functions which have a defined return value should usually
   avoid returning gnil.  For example count_some_thing(x) should return the
   ordinary 0 for its count so that it prints, not the gnil which doesn't
   print.  An absent print might make a user wonder if the usage was right,
   or why there was no output.

   gnil makes no difference at all to other program code.  gnil is a
   perfectly good 0 for all other purposes, it just doesn't print to users
   calling interactively.

   The most common occurrence of gnil from user code are one-leg "if"
   statements such as

       mul_x_and_y(x,y) = if(x, x*y);

   The omitted false leg gives gnil, whereas an ordinary 0 could usually be
   expected from something arithmetic like a multiply.  The desired 0 can be
   written explicity

       mul_x_and_y(x,y) = if(x, x*y, 0);

   This tends to be uncommon.  It usually arises when some n==0 or empty
   vector etc is a special case which has to be tested with an "if".
*/
GEN
test_is_gnil (GEN x)
{
  return x==gnil ? gen_1 : gen_0;
}
