\\ Copyright 2020, 2021, 2022, 2023, 2024 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ This is various tests which are either a bit slow or which depend on OEIS
\\ data available, or both.

default(recover,0)
default(strictargs,1)
default(parisizemax,500*10^6);
read("../interv.gp");
read("../interv-wip.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");
option_verbose=0;

phi=quadgen(5);
log2(x) = log(x)/log(2);



\\-----------------------------------------------------------------------------
\\ A328726  contfrac composites

{
  my(composites(limit) = select(c->!isprime(c),[4..limit]));
  my(a_vector(len) =
     forstep(limit=len,oo,len,
        my(digs=interv_frac_digits(interv_from_contfrac(composites(limit)), , len));
        if(#digs==len,return(digs))));
  OEIS_check_vecfunc("A328726",a_vector);

  if(0,
     my(cfrac=composites(60),
        f=test_contfrac_to_rational(cfrac),
        t = interv_from_contfrac(cfrac),
        digs=interv_frac_digits(t));
     print(digits(floor(f*10^122)));
     my(cfrac=composites(63),
        f=test_contfrac_to_rational(cfrac),
        t = interv_from_contfrac(cfrac),
        digs=interv_frac_digits(t));
     print(#digs);
     print(digs);
     quit);
}

\\-----------------------------------------------------------------------------
\\ phi in middle half between integers

A190248(n) =
{
  my(f=frac(n*quadgen(5)));
  if(f<1/4, 0, f<3/4, 1, 2);
}
OEIS_check_func("A190248",,['offset,1]);

\\ A190250 Positions of 1 in A190248.
\\ numbers n for which 1/4 < {n*phi} < 3/4

\\ A295085 Numbers k such that {k*phi} < 0.25 or {k*phi} > 0.75
\\ print(select(n->A190248(n)!=1, [1..100])); quit

\\ A005653   0, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0,
my(phi=quadgen(5)); \
A078588(n) = floor(2*phi*n) - 2*floor(phi*n);
OEIS_check_func("A078588");

\\ A005653
\\ < 1/2 = 2, 4, 5, 7, 10, 12, 13, 15, 18, 20, 23,
\\ A188469
\\ > 1/2 = 1, 3, 6, 8, 9, 11, 14, 16, 17, 19, 21, 22, 24, 27, 29
\\ for(i=1,#v, print(v[i] - 2*i));
\\ not 0 when A188257 a(n)=[nr+2r]-[nr]-[2r], where r=sqrt(5)
\\ abs(vector(#v,i, v[i] - 2*i == 0))
\\ phi=quadgen(5)
\\ vector(50,n, round(2*phi*n) - 2*round(phi*n))

\\ ever larger departures from mean 2*i
\\ sqrt2=quadgen(8)
\\ sqrt2=quadgen(5)
\\ delta=0; hi=0;lo=0; \
\\ for(n=1,10000000, delta+=(-1)^(frac(n*sqrt2)<1/2); \
\\   hi=max(hi,delta); lo=min(lo,delta)); [lo,hi,delta]
\\ v=select(n->frac(n*sqrt2)<1/2, [1..50000000]); #v
\\ select(n->frac(n*sqrt2)>1/2, [1..100])
\\ v[#v]
\\ u=vector(#v,i, v[i] - 2*i); #u
\\ vecmin(u)
\\ vecmax(u)

\\ below
\\ 1, 3, 5, 6, 8, 10, 13, 15, 17, 18, 20, 22, 25, 27, 29
\\ above
\\ 2, 4, 7, 9, 11, 12, 14, 16, 19, 21, 23, 24, 26, 28
\\ quit
\\ A190248


\\-----------------------------------------------------------------------------
\\ A371886  music ratios
\\ log2(3) = 1.58496250
\\ abs(x/y - log2(3)) minimum for given y

{
  \\ A020857 Decimal expansion of log_2(3)
  my(Lplus1_digits=OEIS_data("A020857"),
     Lplus1_interv=interv_frac_fromdigits(Lplus1_digits),
     L_interv=interv_sub_constant(Lplus1_interv,1));
  print(L_interv*1.0);

  \\ A028507 contrfrac log_2(3)
  check_equal(interv_contfrac(Lplus1_interv,20),
              [1, 1, 1, 2, 2, 3, 1, 5, 2, 23, 2, 2, 1, 1, 55, 1, 4, 3, 1, 1]);
  OEIS_check_vecfunc("A028507",
    len->interv_contfrac(Lplus1_interv, len));

  \\ A005663/A005664 convergents
  \\    1, 2, 3, 8, 19, 65, 84, 485, 1054, 24727,
  \\    1, 1, 2, 5, 12, 41, 53, 306,  665, 15601,
  OEIS_check_vecfunc("A005663",
    len->contfracpnqn(interv_contfrac(Lplus1_interv, len),
                     len)[1,]);
  OEIS_check_vecfunc("A005664",
    len->contfracpnqn(interv_contfrac(Lplus1_interv, len),
                     len)[2,]);

  \\ x/y - L = 0
  \\ x = y*L
  my(best_of_y(y) =
     my(x=interv_floor_to_constant(interv_mul([y,y], L_interv)),
        dlo = interv_sub_constant(L_interv, x/y),
        dhi = interv_add_constant(-L_interv, (x+1)/y));
     if(interv_lex(dlo,dhi) < 0, x, x+1));
  \\ print(vector(200,y,y+=5; best_of_y(y))); quit;
  \\ A366701 a(n) is the nearest integer to n*log_2(3/2).
  \\ A098294 a(n) = ceiling(n*log_2(3/2))
  my(A366701(n) = if(n==0,0,best_of_y(n)));
  OEIS_check_func("A366701",A366701,['offset,0]);

  \\ floor(abs(1 / (2 * (n*log_2(3/2) - round(n*log_2(3/2))))))
  my(a(n) =
     my(N = n*L_interv,
        R = interv_round_to_constant(N),
        den = 2*interv_sub_constant(N, R));
     interv_floor(interv_abs(interv_recip(den))));

  my(want=[1, 2, 2, 1, 6, 1, 5, 1, 1, 3, 1, 25, 1, 2, 2, 1, 8, 1, 4, 1, 1, 3, 1, 12, 1, 2, 2, 1, 13, 1, 3, 1, 1, 4, 1, 8, 1, 2, 2, 1, 30, 1, 3, 1, 1, 5, 1, 6, 1, 2, 2, 1, 165, 1, 2, 2, 1, 6, 1, 5, 1, 1, 3, 1, 22, 1, 2, 2, 1, 9, 1, 4, 1, 1, 3, 1, 11, 1, 2, 2, 1, 15, 1, 3]);
  check_equal(vector(#want,n, a(n)), want);
  if(0, OEIS_check_func("A371886",a,'bfile));

  \\ round(n*L)/n = round(k*n*L)/(k*n)
  my(by_round(n) =
     my(f=A366701(n)/n);
     for(k=2,oo, if(A366701(k*n)/(k*n) != f, return(k-1))));
  check_equal(vector(1000,n,a(n)),
              vector(1000,n,by_round(n)));

  \\ d = abs(r/n - L)
  \\ L-d ... L ... L+d
  \\      ^^^^^^       some fraction x/(k*n) is closer
  \\ r/n ... L ... r/n+2*d
  \\ (k*r)/(k*n) ... (k*r+1)/(k*n)
  \\ seek 1/(k*n) > 2*d  maximum k
  \\ k*n < 1/(2*d)
  \\ k   < 1/(2*n*d)
  \\ d*n = abs(r - n*L)

  if(0,
     system("rm -f         /tmp/b.txt");
     for(n=1,10000, write("/tmp/b.txt",n," ",a(n)));
     system("ls -l         /tmp/b.txt");
     quit);

  \\ 1/(2*n*d) < 2
  \\ 4*n*abs(r/n - L) > 1
  \\ 4*abs(r - n*L) > 1
  \\ abs(round(n*L) - n*L) > 1/4
  \\ abs(round(n*L) - n*L) >= 1/(2*n^2)
  \\ L=log2(3/2)
  \\ m = 5
  \\ select(n->abs(round(n*L) - L*n) < 1/(2*m), [1..100])
  \\ select(n->abs(round(n*L) - L*n) > 1/4, [1..100])
  \\ select(n->my(r=round(n*L),d=abs(r/n - L)); 1/(2*n*d) < 2, [1..100])

  check_equal(select(n->a(n)>=5, [1..100]),
              [5, 7, 12, 17, 24, 29, 36, 41, 46, 48, 53, 58, 60, 65, 70, 77, 82, 89, 94, 99]);

  \\ when a(n)=1
  \\ n*L > +/- 1/4 away from integer
  \\ 1/4 <= frac(n*L) < 3/4
  \\ (n*2*L +1/2) mod 2 < 1
  \\ print(select(n->a(n)==1, [1..100]));
  \\ for(f=1,35,print(f"  "frac(log2(3/2)*f)))
  \\ 2*log2(3/2) = 1.169925001   \\ log_2(9) - 2
  \\ log2(9/4)

  \\ where occur
  if(0,
     \\ print(vector(10,n, floor(n*log2(3))));
     \\ print(vector(10,n, round(n*log2(3))));
     my(vector_differences(v) = if(#v==0,[], vector(#v-1,i,v[i+1]-v[i])));
     for(target=1,1,
        my(l=List([]),n=1);
        while(#l < 100, if(a(n)==target, listput(l,n)); n++);
        print(target" "Vec(l));
        print(l[#l]/100.);
        \\ print("  "vector_differences(Vec(l)));
     );
     quit);

  \\ L = num/den, num = den*L
  my(closest(den) =
     my(num=den*L_interv,
        lo=interv_floor_to_constant(num),
        hi=interv_ceil_to_constant(num),
        lo_dist = interv_sub_constant(L_interv,lo/den),
        hi_dist = interv_add_constant(-L_interv,hi/den));
     if(interv_lex(lo_dist,hi_dist) < 0,
        [lo, lo_dist],
        [hi, hi_dist]));
  my(m=[100,100], dens=List([]), nums=List([]));
  for(den=4,20000,
     my(num,dist); [num,dist]=closest(den);
     if(interv_lex(dist,m) < 0,
        listput(dens,den);
        listput(nums,num);
        m=dist;
        printf("%d/%d  %.10f\n", num,den,dist[1])));
  print(Vec(nums));
  print(Vec(dens));

  \\ A254351 numerators for log2(3)
  \\ A060528 A list of equal temperaments (equal divisions of the octave) whose nearest scale steps are closer and closer approximations to the ratios of two tones of musical harmony: the perfect 4th, 4/3 and its complement the perfect 5th, 3/2.
  quit;
}



\\-----------------------------------------------------------------------------
\\ A064648  sum reciprocals of primorials
\\ A368497  prime sum fixed point

{
  my(A002110_primorial(n) =
     n>=0 || error();
     vecprod(primes(n)));
  OEIS_check_func("A002110_primorial", A002110_primorial);
  check_equal(A002110_primorial(0), 1);
  check_equal(A002110_primorial(1), 2);

  my(A064648_recips_sum(maxi) = sum(i=1,maxi, 1/A002110_primorial(i)));
  my(A064648_recips_interv(maxi) =
     my(lo = A064648_recips_sum(maxi));
     [lo, lo + 2/A002110_primorial(maxi)]);
  OEIS_check_vecfunc("A064648",
    len->interv_frac_digits(A064648_recips_interv(len)*10,,len));

  my(A340469_sub2_sum(maxi) =
     sum(i=0,maxi, (prime(i+1)-2)/A002110_primorial(i)));
  my(A340469_sub2_interv(maxi) =
     my(lo = A340469_sub2_sum(maxi));
     [lo, lo + 2/A002110_primorial(maxi)]);
  OEIS_check_vecfunc("A340469",
    len->interv_frac_digits(A340469_sub2_interv(len),,len));

  my(A249270_sub1_sum(maxi) =
     sum(i=0,maxi, (prime(i+1)-1)/A002110_primorial(i)));
  my(A249270_sub1_interv(maxi) =
     my(lo = A249270_sub1_sum(maxi));
     [lo, lo + 2/A002110_primorial(maxi)]);
  OEIS_check_vecfunc("A249270",
    len->interv_frac_digits(A249270_sub1_interv(len),,len));

  my(S_sum(x,maxi) =
     sum(i=0,maxi, (prime(i+1)-x)/A002110_primorial(i)));
  my(maxi=2);
  check_equal(A064648_recips_sum(maxi), 1/2 + 1/6, "recips");
  check_equal(A340469_sub2_sum(maxi), (2-2)/1 + (3-2)/2 + (5-2)/6, "p-2");
  check_equal(A249270_sub1_sum(maxi), (2-1)/1 + (3-1)/2 + (5-1)/6, "p-1");
  check_equal(S_sum(x,maxi),
              (2-x)/1 + (3-x)/2 + (5-x)/6, "p-x");
  my(maxi=10);
  check_equal(S_sum(x,maxi),
     (2-x)*(1 + A064648_recips_sum(maxi)) + A340469_sub2_sum(maxi),
             "S by sub2");

  \\ S(x) = A249270 - (x-1)*(A064648 + 1)
  check_equal(S_sum(x,maxi),
    A249270_sub1_sum(maxi)
    - (x-1)*(A064648_recips_sum(maxi) + 1),
             "S by sub1");

  \\ S(x) = (1-x)*(1+A064648) + A249270
  check_equal(S_sum(x,maxi),
    (1-x)*(1+A064648_recips_sum(maxi))
    + A249270_sub1_sum(maxi),
             "S by sub1");
  \\ x = (1-x)*(1+A064648) + A249270
  \\ x*(2+A064648) = A064648 + A249270

  \\ towards
  \\ x = (A249270 + A064648 + 1)/(A064648 + 2)
}

\\-----------------------------------------------------------------------------
\\ A006752  Catalan constant Sum (-1)^n / (2n+1)^2
\\   sumalt(n=0, (-1)^n/(2*n+1)^2)
\\   sum(n=0,1000000, 1./(2*n+1)^2)
\\   plain odd squares slow


\\-----------------------------------------------------------------------------
\\ A004593  e binary
\\ A369870  cumulative

{
  my(t = interv_exp([1,1],150));  \\ e = exp(1)
  OEIS_check_vecfunc("A001113",len->interv_frac_digits(t,,len));
  OEIS_check_vecfunc("A004593",len->interv_frac_digits(t/2,2,len));

  my(want=[1, 1, 2, 2, 3, 4, 4, 5, 6, 7, 8, 9, 10, 10, 10, 10, 10, 11, 11, 12, 12, 13, 13, 13, 13, 14, 14, 15, 16, 16, 16, 16, 17, 17, 18, 18, 18, 18, 19, 19, 20, 20, 21, 22, 23, 23, 24, 25, 25, 26, 26, 26, 27, 27, 28, 28, 29, 29, 29, 30, 31, 31, 32, 32, 33, 33, 34, 34]);
  my(bits=interv_frac_digits(t/2,2,#want));
  check_equal(vector(#want,n, sum(i=1,n,bits[i])), want);
}


\\-----------------------------------------------------------------------------
\\ A369522  log(4/3) / log(3/2)

{
  my(want=[7, 0, 9, 5, 1, 1, 2, 9, 1, 3, 5, 1, 4, 5, 4, 7, 7, 6, 9, 7, 6, 1, 9, 0, 2, 6, 2, 1, 7, 4, 0, 1, 4, 1, 4, 0, 6, 1, 5, 0, 0, 3, 7, 3, 5, 2, 3, 6, 1, 0, 7, 2, 2, 3, 0, 7, 4, 4, 5, 3, 9, 0, 6, 2, 8, 7, 7, 1, 8, 5, 7, 7, 8, 9, 9, 5, 5, 4, 4, 2, 6, 6, 3, 4, 0, 2]);
  my(t= interv_div(interv_log([4/3,4/3], 200),
                   interv_log([3/2,3/2], 200)));
  my(got=interv_frac_digits(t,,#want+1));
  check_equal(got[1], 0);
  got = got[^1];
  check_equal(got, want);

  check_equal(digits(70951129135145477697619), want[1..23]);
}


\\-----------------------------------------------------------------------------
\\ interv_mul() precision

{
  my(t=[1000,1000],
     m=[1,1+1/1000]);
  check_equal(interv_mul(t,m), [1000,1001]);
}


\\-----------------------------------------------------------------------------
\\ root f x^3 - x^2 - 1 = 0
\\   (1/6)*(116+12*sqrt(93))^(1/3)
\\ + 2/(3*(116+12*sqrt(93))^(1/3)) + 1/3
{
  my(sqrt93 = quadgen(4*93));
  check_equal(simplify(sqrt93^2), 93);
\\   (1/6)* (116+12*sqrt(93))^(1/3)
\\ + 2/(3*  (116+12*sqrt(93))^(1/3))
\\  + 1/3
  my(s = (116+12*sqrt93),
     bits = 42000);
  my(t_s = interv_widen_bits([s,s],bits));
  check_equal((t_s[2]-t_s[1])*2^bits, 1, "t_s width");
  my(t_c = interv_sqrtnint(t_s*8^bits,3) / 2^bits,
     t = [1/3, 1/3]);
  t = interv_add(t, 1/6*t_c);
  t = interv_add(t, 2/3*interv_recip(t_c));

  my(p(x) = x^3 - x^2 - 1);
  check_equal(p(t[1]) < 0, 1,  "below root");
  check_equal(p(t[2]) > 0, 1,  "above root");

  \\ 50000 bits
  \\ bits     50000
  \\ decimals 15053
  \\ cfrac    14516
  if(0,
     print("bits ",bits);
     print("t_s      ",t_s*1.0);
     print("t_s bits ",#interv_frac_digits(t_s,2));
     print("t_c bits ",#interv_frac_digits(t_c,2));
     print("bits     ",#interv_frac_digits(t,2));
     print("decimals ",#interv_frac_digits(t));
     print("cfrac    ",#interv_contfrac(t));
     );

  \\ my(t = interv_solve_bisection
  \\       ([1,2], x->x^3 - x^2 - 1, 80000));
  print(t*1.0);
  OEIS_check_vecfunc("A092526",
    len -> interv_frac_digits(t,,len));

  my(want=[1, 2, 6, 1, 3, 5, 4, 22, 1, 1, 4, 1, 2, 84, 1, 3, 1, 6, 1, 3, 1, 9, 1, 1, 1, 1, 19, 3, 1, 2, 1, 5, 1, 5, 2, 2, 1, 1, 1, 1, 76, 6, 8, 1, 1, 5, 1, 5, 1, 1, 25, 1, 2, 1, 116, 2, 1, 8, 1, 1, 3, 1, 53, 5, 276, 2, 1, 1, 1, 3, 3, 2, 1, 1, 4, 13, 1, 1, 1, 4, 1, 1, 1, 9, 9, 1, 1, 9, 6, 1, 2, 32]);
  check_equal(interv_contfrac(t,#want), want,
              "contfrac");
  if(0,
     \\ b369346.txt
     OEIS_check_vecfunc("A369346",
       len->interv_contfrac(t,len), 'bfile));

  \\ default(realprecision,100);
  \\ check_equal(contfrac(solve(x=1,2, x^3 - x^2 - 1), , 80),
  \\             want[1..80]);

  check_equal(interv_frac_digits(t,2,50),
              [1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1]);
  check_equal(interv_Egyptian_terms(t,8),
              [1, 3, 8, 139, 22908, 937418727, 24128891176195996909, 964774017262641910148342320227030998843]);
  check_equal(interv_Engel_terms(t,10),
              [1, 3, 3, 6, 8, 8, 71, 152, 469, 744]);

}


\\-----------------------------------------------------------------------------
\\ A368494 = smallest k with log10(k) digits n

{
  my(want=[13, 16, 2, 3, 32, 4, 6, 7, 8, 126, 13, 132, 135, 14, 142, 145, 15, 152, 155, 16, 163, 166, 17, 174, 18, 182, 19, 191, 195, 2, 205, 21, 214, 22, 224, 23, 235, 24, 25, 252, 26, 264, 27, 28, 282, 29, 3, 302, 31, 32, 33, 332, 34, 35, 36, 37, 38, 381, 39, 4, 41]);
  my(a(n) = localprec(1000);
     my(ten=log(10), S=10^#digits(n));
     for(k=1,oo,
        my(L=frac(log(k)/ten));
        \\ while(L < n, L*=10);
        L *= S;
        if(floor(L)==n,return(k))));
  \\ check_equal(a(12),    103);
  \\ check_equal(want[12], 132);
  \\ log10(n)=log(n)/log(10)
  \\ log10(103) = 2.012837224
  \\ log10(132) = 2.120573

  check_equal(vector(#want,n, a(n)), want);
}


\\-----------------------------------------------------------------------------
\\ A188659  (1+sqrt(26))/5

{
  my(sqrt26 = quadgen(4*26),
     c = (1+sqrt26)/5);
  check_equal(simplify(sqrt26^2), 26);
  check_equal(interv_contfrac([c,c],9),
              [1, 4, 1, 1, 4, 1, 1, 4, 1]);
  \\ check_equal(interv_Egyptian_terms([c,c],10), []);
  check_equal(interv_Engel_terms([c,c],10),
              [1, 5, 11, 12, 15, 18, 19, 19, 45, 51]);
}


\\-----------------------------------------------------------------------------
\\ A253358  sum prime(n)/n^3
\\ OEIS_data("A253358")
\\   3, 3, 3, 9, 5, 3

\\ A000040
\\ Rosser and Schoenfeld
\\ n*(log n + log log n - 3/2) < prime(n)  for n>=2
\\ a(n) < n*(log n + log log n - 1/2)      for n >= 20,

{
  \\ intformal(log(n+1)/(n+1)^2)
  if(0,
     my(tail(n) = 2*intnum(x=n,oo, log(x+1)/(x+1)^2));

     my(n=0, total=0, target=1);
     forprime(p=1,oo,
        n++;
        total += p<<64 \ n^3;

        if(n >= target,
           my(t = [total, total+n]/ 2^64);
           t[2] += ceil(tail(n)*2^64 + 100) / 2^64;
           print("n="n" p="p"  "interv_frac_digits(t));
           target += 1000000);
     );
     quit);
}


\\------------------------------------------------------------------------------
\\ e contfrac = A003417

A003417_e_contfrac(n) =
{
  n>=1 || error();
  if(n==1,2, n%3, 1, 2*n\3);
}
{
  my(a=A003417_e_contfrac);
  OEIS_check_func("A003417_e_contfrac",,['offset,1]);
  check_equal(vector(100,n,n+=7; a(n)),
              vector(100,n,n+=7; 2*a(n-3) - a(n-6)));

  my(t = interv_exp([1,1],50),  \\ e = exp(1)
     got=interv_contfrac(t));
  check_equal(#got, 62);
  check_equal(got, vector(#got,n, a(n)),
              "interv_contfrac() of e");

  my(e_digits=OEIS_data("A001113"),
     t_e=interv_frac_fromdigits(e_digits),
     got=interv_contfrac(t));
  check_equal(got, vector(#got,n, a(n)),
              "interv_contfrac() of e digits");

  \\ Engel expansion
  \\   1,1, 2,3,4,...
  my(got=interv_Engel_terms(t_e));
  check_equal(got, vector(#got,n, if(n==1,1, n-1)),
              "Engel of e");
  check_equal(interv_Engel_terms(interv_sub_constant(t_e,2), 7),
              [2, 3, 4, 5, 6, 7, 8]);

  my(g(n) = if(n==1,'e, n * (self()(n-1) - (n-1))));
  check_equal(g(1),  e);
  check_equal(g(2),  2*(e-1));
  check_equal(g(3),  3*(2*(e-1) - 2));
  check_equal(g(4),  4*(3*(2*(e-1) - 2) - 3));
  check_equal(g(5),  5*(4*(3*(2*(e-1) - 2) - 3) - 4));
  check_equal(g(4),  4*3*2*e - 4*3*2*1 - 4*3*2 - 4*3);
  check_equal(g(5),  5*4*3*2*e - 5*4*3*2*1 - 5*4*3*2 - 5*4*3 - 5*4);
  check_equal(g(4)/4!,  e - 1 - 1 - 1/2!);
  check_equal(g(5)/5!,  e - 1 - 1 - 1/2! - 1/3!);
  check_equal(g(6)/6!,  e - 1 - 1 - 1/2! - 1/3! - 1/4!);

  my(fvec(t,len) =
     my(v=vector(len));
     v[1] = interv_floor(t);
     for(i=2,len,
        t = v[i-1] * (interv_add_constant(t, -v[i-1] + 1));
        v[i] = interv_floor(t));
     v);
  check_equal(fvec(t_e,7), [2, 3, 4, 5, 6, 7, 8]);
  \\ x(n) = (frac(x(n-1))+1)*floor(x(n-1)) and x(1) = Pi
  \\ x(1) = Pi
  \\ x(2) = (Pi-3+1)*3
  \\ x(3) = ((Pi-3+1)*3 - 3+1)*3
  \\ x(4) = (((Pi-3+1)*3 - 3+1)*3 - 4+1)*4
  \\ x(4) = Pi*3*3*4 - (3-1)*3*3*4 - (3-1)*3*4 - (4-1)*4
  \\ x(4)/(3*3*4) = Pi - (3-1) - (3-1)/3 - (4-1)/(3*3)
  \\ x(5) = Pi*3*3*4*5
  \\        - (3-1)*3*3*4*5 - (3-1)*3*4*5 - (4-1)*4*5 - (5-1)*5
  \\ x(5)/(3*3*4*5)=Pi - (3-1) - (3-1)/3 - (4-1)/(3*3) - (5-1)/(3*3*4)
  \\ x(4) = (((Pi-3+1)*3 - 3+1)*3 - 4+1)
  my(xvec=[7,11,19],
     x(n) = if(n==1,'p, (self()(n-1)-xvec[n-1]+1)*xvec[n-1]));
  check_equal(x(1),  p);
  check_equal(x(2),  (p - (7-1))*7, "x(2)");
  check_equal(x(3),    ((p - (7-1))*7 - (11-1))*11,  "x(3)");
  check_equal(x(4),  ( ((p - (7-1))*7 - (11-1))*11 - (19-1)) * 19,
              "x(4)");
  check_equal(x(4)/(7*11*19),
              p - (7-1) - (11-1)/7 - (19-1)/(7*11),
              "x(4)/(7*11*19)");

  my(x_to_p(xvec) =
     my(p='p); for(i=1,#xvec, p = (p-(xvec[i]-1))*xvec[i]); p);
  check_equal(x_to_p([7,11,19]), x(4));
  \\ print(subst(x_to_p(OEIS_data("A339412")),p,Pi));
  my(xvec=[7,11,19,23]);
  check_equal(x_to_p(xvec)/vecprod(xvec),
              p - (7-1) - (11-1)/7 - (19-1)/(7*11) - (23-1)/(7*11*19),
              "x_to_p 4");
  check_equal(x_to_p(xvec)/vecprod(xvec),
              p - sum(i=1,#xvec, (xvec[i]-1)/vecprod(xvec[1..i-1])),
              "x_to_p 4 prod");
  \\ Pi - (3-1) - (3-1)/3 - (4-1)/(3*3)
  \\ Pi - (3-1) - (3-1)/3 - (4-1)/(3*3) - (5-1)/(3*3*4)

  \\ print(fvec([quadgen(5),quadgen(5)],500));
  \\ quit;

  my(Pi_digits=OEIS_data("A000796",'bfile),
     t_Pi=interv_frac_fromdigits(Pi_digits));
  check_equal(fvec(t_Pi,10),
              [3, 3, 4, 5, 5, 7, 10, 10, 13, 17]);
  OEIS_check_vecfunc("A339412",len->fvec(t_Pi,len));
  my(want=OEIS_data("A339412",'bfile));
  want=want[1..400];
  check_equal(fvec(t_Pi,#want), want,
              "fvec of Pi");

  \\ apply(t->t-1,[3, 3, 4, 5, 5, 7, 10, 10, 13, 17])
  \\ print(interv_Engel_terms(t_Pi,10));
  \\ [1, 1, 1, 8, 8, 17, 19, 300, 1991, 2492]

  \\ my(e=x);
  \\ forstep(n=5,2,-1, e=n+n*e);
  \\ print(e);
  \\ quit;
}

\\-----------------------------------------------------------------------------
\\ A010499  sqrt(45)

A010499_vector(len) = digits(sqrtint(floor(45*100^(len-1))));
{
  check_equal(vector(10,len,len--; #A010499_vector(len)),
              vector(10,len,len--; len));
  OEIS_check_vecfunc("A010499_vector");
}

\\ A020802 Decimal expansion of 1/sqrt(45).
A020802_vector(len) = digits(sqrtint(floor(45*100^len))\45);
{
  check_equal(vector(10,len,len--; #A020802_vector(len)),
              vector(10,len,len--; len));
  OEIS_check_vecfunc("A020802_vector");
}

\\ A248271 Egyptian of sqrt(45)
\\ integer part 6 as single 6
my(sqrt45=quadgen(4*45), t=[sqrt45-5,sqrt45-5]); \
A248271_vector(len) =
{
  my(v=interv_Egyptian_terms(t,len));
  if(#v, v[1]=6);
  v;
}
{
  my(sqrt45=quadgen(4*45));
  check_equal(sqrt45^2 == 45, 1);
  OEIS_check_vecfunc("A248271_vector");
}

\\ A010135 Contfrac of sqrt(45)
my(sqrt45=quadgen(4*45), t=[sqrt45,sqrt45]); \
A010135_vector(len) = interv_contfrac(t,len);
OEIS_check_vecfunc("A010135_vector");

\\ A041076 numerator contfrac convergents of sqrt(45)
my(sqrt45=quadgen(4*45), t=[sqrt45,sqrt45]); \
A041076_vector(len) = \
  contfracpnqn(interv_contfrac(t,len),len)[1,];
OEIS_check_vecfunc("A041076_vector");

\\ A041077 denominator contfrac convergents of sqrt(45)
my(sqrt45=quadgen(4*45), t=[sqrt45,sqrt45]); \
A041077_vector(len) = \
  contfracpnqn(interv_contfrac(t,len),len)[2,];
OEIS_check_vecfunc("A041077_vector");


\\-----------------------------------------------------------------------------
\\ is_Fibonacci() by logs

\\ {
\\ my(is_Fibonacci(n) =
\\   my(t=5*sqr(n)); (n>0 && issquare(t-4)) || issquare(t+4));
\\
\\   \\ a(n) = n == fibonacci(round(log(sqrt(5)*n)/log(phi)))
\\   \\ vector(200,n,fibonacci(round(log(sqrt(5)*n)/log(phi))))
\\   \\ vector(200,n,round(log(sqrt(5)*n)/log(phi)))
\\   my(sqrt5 = quadgen(20),
\\      t_sqrt5 = interv_solve_bisection
\\               ([2,3], x->x^2-5, 100),
\\      t_log_phi = interv_frac_fromdigits(OEIS_data("A002390"))/10);
\\   print(t_sqrt5*1.0);
\\   print(t_log_phi*1.0);
\\   my(a(n) =
\\      my(L=interv_log(n*t_sqrt5,200));
\\      L = interv_div(L,t_log_phi);
\\      \\ print(L*1.0);
\\      my(R=interv_round_to_constant(L));
\\      fibonacci(R) == n);
\\   for(n=1,40,
\\      print(n);
\\      a(n) == is_Fibonacci(n) || error());
\\   quit;
\\ }


\\-----------------------------------------------------------------------------
\\ sqrt(666)
{
  my(sqrt666=quadgen(2664));
  check_equal(sqrt666^2 == 666, 1);
  my(want=[2, 5, 8, 0, 6, 9, 7, 5, 8, 0, 1, 1, 2, 7, 8, 8, 0, 3, 1, 5, 1, 8, 8, 4, 2, 0, 6, 0, 5, 1, 4, 9, 1, 4, 0, 8, 9, 6, 0, 8, 2, 6, 0, 6, 6, 7, 1, 8, 7, 2, 2, 0, 6, 8, 5, 8, 5, 2, 4, 1, 3, 6, 9, 2, 3, 7, 1, 2, 2, 8, 0, 8, 0, 3, 9, 8, 9, 0, 5, 1, 0, 3, 8, 3, 4, 9, 9]);
  check_equal(digits(floor(10^(#want-2)*quadgen(2664))), want);

  OEIS_check_vecfunc("A040640", len->interv_contfrac([sqrt666,sqrt666],len));
  \\ convergents num/den
  OEIS_check_vecfunc("A042280",
    len->my(v=interv_contfrac([sqrt666,sqrt666],len));
         contfracpnqn(v,len)[1,]);
  OEIS_check_vecfunc("A042281",
    len->my(v=interv_contfrac([sqrt666,sqrt666],len));
         contfracpnqn(v,len)[2,]);

  if(0,
     my(t=[sqrt666,sqrt666]);
     t=interv_add_constant(t,-24);
     print(interv_Egyptian_terms(t,10));
     print(interv_Engel_terms(t,10));
     quit);
}


\\-----------------------------------------------------------------------------
\\ A366451  most likely sum s > n when rolling n sided die

{
  my(want=[4, 6, 8, 9, 11, 13, 15, 16, 18, 20, 21, 23, 25, 27, 28, 30, 32, 34, 35, 37, 39, 40, 42, 44, 46, 47, 49, 51, 52, 54, 56, 58, 59, 61, 63, 64, 66, 68, 70, 71, 73, 75, 76, 78, 80, 82, 83, 85, 87, 88, 90, 92, 94, 95, 97, 99, 101, 102]);

  my(prob_list(n=0,len=n) =
     my(v=vector(n+len-1,i,i==n));
     for(i=n+1,#v,
        v[i] = vecsum(v[i-n..i-1]) / n);
     v=v[n-1..#v];
     for(i=1,#v, v[i]*=n^(i-2));
     v);
  print(prob_list(4,10));
  \\ n=2
  \\ A001045 Jacobsthal
  \\ n=3
  \\ A103770 A weighted Tribonacci sequence, (1,3,9).


  my(a_by_sliding_window(n,show=0) = n>=2||error();
     my(v=vector(n,i,i==n));
     for(t=0,oo,
        if(show && t==n, print(n"  "v));
        my(s=vecsum(v)); v*=n;
        if(t>n && s<v[(t-1)%n+1], if(s==v[(t-1)%n+1],error(n));
           return(t));
        v[t%n+1]=s));;
  check_equal(vector(#want,n,n++; a_by_sliding_window(n)), want);

  my(a_by_interv(n,bits=2*n) =
     n>=2||error();
     my(v=vector(n,i, if(i==n,[1,1],[0,0])));
     for(t=0,oo,
        my(s=vecsum(v)/n);
        s=interv_widen_bits(s,bits);
        if(t>n && interv_lex(s,v[(t-1)%n+1]) < 0,
           return(t));
        v[t%n+1]=s));
  check_equal(vector(#want,n,n++; a_by_sliding_window(n)), want);

  my(a_by_fixed(n,bits=64) =
     n>=2||error();
     my(v=vector(n,i, if(i==n,[1<<bits,1<<bits],[0,0])),
        s=vecsum(v));
     for(t=0,oo,
        \\ s==vecsum(v) || error();
        my(m=s/n);
        m=interv_widen_integer(m);
        if(t>n && interv_lex(m,v[(t-1)%n+1]) < 0,
           return(t));
        s += m - v[t%n+1];
        v[t%n+1]=m));
  check_equal(vector(#want,n,n++; a_by_fixed(n)), want);

  \\ [3040, 4760, 6480, 8201]
  \\ cf 3040.499
  \\    6480.499916
  \\    8200.5000263...
  print("on list ",apply(a_by_fixed, [1769, 2770, 3771, 4772]));

  my(examples = [[ 21,  08266],
                 [ 22,  08429],
                 [ 23,  08588],
                 [ 24,  08742],
                 [ 25,  08890],
                 [ 26,  09030],
                 [ 27,  09163],
                 [ 28,  09286],
                 [ 29,  09399],
                 [ 30,  09499],
                 [ 31,  09586],
                 [ 32,  09659],
                 [ 33,  09714],
                 [ 34,  09751],
                 [ 35,  09767],
                 [ 36,  09760],
                 [ 37,  09728],
                 [ 38,  09669],
                 [ 39,  09580],
                 [ 40,  09457],
                 [ 41,  09298],
                 [ 42,  09350]]);
  my(v=prob_list(20,#examples+22));
  \\ print(v);
  for(i=1,#v, v[i] /= 20^(i-2));
  \\ print(v*1.0);
  my(got_examples=vector(#examples,i, [i+20, floor(v[i+22]*10^5)]));
  check_equal(got_examples,examples, "examples");

  my(t_e = interv_frac_fromdigits(OEIS_data("A001113")));
  my(A022852(n) = interv_round_to_constant(n*t_e));
  OEIS_check_func("A022852",A022852,['offset,0]);
  \\ WRONG:
  \\ my(a_by_A022852_e_round(n) = A022852(n+1) - (n+1));
  \\ check_equal(vector(#want,n,n++; a_by_A022852_e_round(n)) -
  \\             want,
  \\             "vs a_by_A022852_e_round()");

  \\ A022852 Integer nearest n * e, where e is the natural log base.
  \\ A001113 e
  if(0,
     \\ good to n=27700
     print(t_e*1.0);
     \\ f(n) = (e-1)*(n+1/2)
     my(f_interv(n) = interv_mul(interv_add_constant(t_e,-1),
                                 n + 1/2));
     my(a_by_round(n) = interv_round_to_constant(f_interv(n)));
     for(n=2,oo,
        if(n%100==0,print(n));
        if(a_by_fixed(n)!=a_by_round(n),
           print(n)));
     quit);

  if(0,
     my(n=1000000,bits=62);
     print("n="n" bits="bits);
     gettime();
     print(a_by_fixed(n,bits));
     print("time "gettime());
     print(a_by_interv(n,bits));
     print("time "gettime());
     print(a_by_sliding_window(n));
     print("time "gettime());
     quit);
}

\\-----------------------------------------------------------------------------
\\ A366397  contfrac of pi each term +1

{
  my(want=[4, 1, 2, 4, 0, 6, 0, 1, 0, 2, 2, 8, 7, 8, 6, 5, 3, 9, 1, 6, 7, 5, 8, 5, 0, 8, 3, 2, 2, 5, 6, 8, 1, 7, 4, 9, 7, 8, 4, 2, 0, 1, 8, 3, 7, 2, 9, 7, 3, 9, 1, 3, 5, 6, 7, 7, 0, 7, 3, 4, 3, 4, 3, 5, 6, 2, 3, 1, 8, 9, 4, 5, 4, 1, 5, 8, 9, 1, 8, 0, 1, 6, 8, 3, 3, 3, 3, 1, 5, 4, 4, 2, 9, 7, 0, 6, 8, 1, 0, 3, 0, 3, 6, 0]);
  check_equal(digits(4 12406010228786539167585),
              want[1..24], "example");

  my(Pi_digits=OEIS_data("A000796",'bfile),
     t_Pi=interv_frac_fromdigits(Pi_digits));

  \\ altenrating series for pi
  my(summand(k) = (-1)^k/2^(10*k)
                 * (-2^5/(4*k+1)
                    -1/(4*k+3)
                    +2^8/(10*k+1)
                    -2^6/(10*k+3)
                    -2^2/(10*k+5)
                    -2^2/(10*k+7)
                    +1/(10*k+9)));
  my(t = [0,0]);
  for(k=0,10,
     my(s=1/2^6*summand(k));
     sign(s) == (-1)^k || error();
     t = [t[2], t[2] + s];
     if(interv_intersect(t,t_Pi) == 'none, error()));
  print(t*1.0);

  my(pi_cfrac = OEIS_data("A001203",'bfile)[1..500],
     plus_cfrac = apply(t->t+1, pi_cfrac),
     plus_t = interv_from_contfrac(plus_cfrac));

  if(0,
     my(bad=[4, 1, 2, 4, 0, 6, 0, 1, 0, 2, 2, 8, 7, 8, 6, 5, 3, 9, 1, 6, 7, 5, 8, 5, 0, 9, 0, 8, 5, 2, 4, 1, 4, 4, 9, 7, 3, 5, 7, 4, 2, 6, 3, 7, 4, 3, 1, 9, 6, 1, 7, 8, 9, 0, 7, 6, 6, 8, 9, 1, 0, 6, 9, 6, 8, 9, 8, 2, 1, 6, 6, 6, 2, 8, 3, 7, 9, 3, 9, 8, 3, 6, 7, 2, 8, 6, 7, 3, 1, 6],
        Pi_digits=OEIS_data("A000796",'bfile)[1..1000],
        N=#bad);
     print("bad len "#bad);

     my(b=fromdigits(bad)/10^(#bad-1));
     print("b     "b*1.0);
     my(b_cf=contfrac(b));
     print("b cf  "b_cf);
     print("vs    "apply(t->t+1,pi_cfrac[1..#b_cf]));
     quit;
     my(flen=20,
        f=fromdigits(Pi_digits[1..flen])/10^(flen-1));
     print("f  "f*1.0);
     my(cf=contfrac(f));
     cf=cf[1..flen];
     print("cf len "#cf"  "cf);
     my(f=test_contfrac_to_rational(apply(t->t+1,cf)));
     print("diff ",digits(floor(f*10^(#bad-1))) - bad);
     quit);

  if(0,
     default(realprecision,1000);
     my(v=contfrac(Pi));
     check_equal(v[1..#v-2], pi_cfrac[1..#v-2]);
     my(c=contfrac_to_rational(plus_cfrac));
     my(d=digits(floor(c*10^(#want-1))));
     check_equal(d,want, "vs contfrac_to_rational()");
     quit);

  print("pi_cfrac length ",#pi_cfrac);
  check_equal(interv_frac_digits(plus_t,, #want), want,
              "pi cfrac +1 vs DATA");

  if(0,
     \\ sometimes Pi rounding makes contrfrac(Pi) last term wrong
     my(prev=[]);
     for(p=50,100,
        default(realprecision,p);
        my(v=contfrac(Pi));
        if(v[1..#prev] != prev,
           print("p="p);
           print(prev);
           print(v);
           print();
           );
        prev=v);
     quit);
}


\\-----------------------------------------------------------------------------
\\ Thue-Morse Constant .01101001100101101001...

ThueMorse(n) = hammingweight(n) % 2;
ThueMorse_constant_interv(len,base=10) =
{
  interv_frac_fromdigits(vector(len,n, if(n>=2,ThueMorse(n-2))),
                         base);
}

{
  \\ base -2
  \\ 1/4 + 1/4^2 + ... = 1/3
  \\ -1/2 - 1/2^3 + ... = -2/3
  \\ 1/base + 1/base^3 + ... = 1 / b*(1-b^2)
  \\ my(b=2); 1/(b*(1-1/b^2)) == 2/3
  \\ my(b=2); 1/(b-1/b) == 2/3
  check_equal(interv_frac_digits([-1/2,-1/2],-2,1),
              [-1]);
  check_equal(interv_frac_digits([-1/2 + 1/4,-1/2 + 1/4], -2, 2),
              [-1, -2]);
  check_equal(interv_frac_digits([1/2,1/2],-2,1),
              [0]);
}
interv_frac_digits_with_negbase(t,base=10,maxlen='none) =
{
  \\ print("t = "t);
  my(l=List([]), f, add=vector(2,i, 1/(1/base-base)));
  \\ print("add ",add);
  while((maxlen=='none || #l<maxlen) && (f=floor(t+add))[1]==f[2],
        \\ print("f ",f);
        listput(l,f[1]);
        t=(t-f)*base;
        \\ print("to t="t);
       );
  Vec(l);
}
{
  check_equal(interv_frac_digits_with_negbase([-1/2,-1/2],-2,2),
              [0,1]);
}

{
  \\ arbitrary 1000 terms enough for OEIS data sections

  \\ decimal constant
  \\ A058631 contfrac
  OEIS_check_vecfunc("A058631",
    len->interv_contfrac(ThueMorse_constant_interv(1000), len));
  \\ A074071

  \\ binary constant
  \\ A014571 as decimal
  \\ A014572 contfrac
  \\ A085394/A085395 contfrac convergents
  \\ A085396 contfrac convergents num+den
  OEIS_check_vecfunc("A014571",
    len->interv_frac_digits(ThueMorse_constant_interv(1000,2)*10,,len));
  OEIS_check_vecfunc("A014572",
    len->interv_contfrac(ThueMorse_constant_interv(1000,2), len));
  OEIS_check_vecfunc("A085394",
    len->contfracpnqn(interv_contfrac(ThueMorse_constant_interv(1000,2), len),
                     len)[1,]);
  OEIS_check_vecfunc("A085395",
    len->contfracpnqn(interv_contfrac(ThueMorse_constant_interv(1000,2), len),
                     len)[2,]);
  OEIS_check_vecfunc("A085396",
    len->my(m=contfracpnqn(interv_contfrac(ThueMorse_constant_interv(1000,2), len), len));
         m[1,] + m[2,]);

  \\ binary constant
  \\ A289748 as base -2  offset 1 digit above radix point
  \\ 1 . 1 0 1 1 1 1 1 0 1 1 1 0 1 0 1 1 1 1 1 0 1 0 1 1 1 0 1 1 1
  \\       ^   ^   ^   ^   ^   ^   ^   ^   ^   ^   ^   ^   ^   ^
  \\ base -2   1.101010... = 1 - 2/3 = 1/3
  \\ base 2  1/3 = .01010101 ...
  \\ added turns even index ThueMorse 0,1 into 01 or 10
  my(A289748_vector(len) =
     interv_frac_digits_with_negbase
     (ThueMorse_constant_interv(1000,2), -2, len));
  OEIS_check_vecfunc("A289748",A289748_vector);
  \\ even positions 1, odd positions Thue-Morse
  my(A289748(n) = bitnegimply(1,n) || bitand(1,hammingweight(n-3)));
  my(v=OEIS_data("A289748"),
     odds=vector(#v\2,i, v[2*i-1]));
  check_equal(vector(#odds,n,n-=2; if(n<0,1, ThueMorse(n))),
              odds,
              "odds");
  OEIS_check_func("A289748",A289748,['offset,1]);

  if(0,
     \\ 1.74578736
     \\ 0.872893683
     \\ 0.539560350
     \\   . 0 1 1 0  1 0 0 1
     \\ 1 . 1,0,1,1, 1,1,1,0, 1,1,1,0,
     \\     1,0,1,1, 1,1,1,0, 1,0,1,1,
     \\     1,0,1,1, 1,1,1
     print(vector(20,n,ThueMorse(2*n)));  \\ ThueMorse itself
     print("base -2:  ",OEIS_data("A289748")[1..32]);
     print("evens ",vector(#v\2,i, v[2*i]));
     print("odds  ",vector(#v\2,i, v[2*i-1]));
     print(interv_frac_fromdigits(OEIS_data("A289748"),2)[1]/2*1.0 - 1/3);
     \\ 0.07912070030677426445002803493  as base 4
     \\ 0.1582414006135485289000560699
     \\ 0.872893683
     print(ThueMorse_constant_interv(1000,4)[1]*2.0);
     quit);

  \\ decimal constant
  \\ A096394 Engel expansion
  print("A096394 Engel expansion");
  OEIS_check_vecfunc("A096394",
    len->my(t=ThueMorse_constant_interv(10000,2));
         interv_Engel_terms(t,len));
  if(0,
     \\ b-file
     my(t=ThueMorse_constant_interv(300000,2),
        v=interv_Engel_terms(t,600));
     check_equal(v[32], 765130000703664);
     check_equal(v[33], 4360055342801637);
     print("length ",#v);
     system("rm -f      /tmp/b096394.txt");
     for(n=1,#v, write("/tmp/b096394.txt",n," ",v[n]));
     system("ls -l      /tmp/b096394.txt");
     quit);

  \\ decimal of binary 0.110..., being 2*.0110
  \\ .824908067...
  OEIS_check_vecfunc("A321071",
    len->interv_frac_digits(ThueMorse_constant_interv(1000,2)*2*10,,len));
  \\ contfrac of 2*
  \\ not in OEIS: 0, 1, 4, 1, 2, 2, 6, 2, 1, 2, 9, 1, 2, 2, 1, 1, 21, 1, 10
}


\\-----------------------------------------------------------------------------
\\ A365125  unit electrical point charges positioned

\\ where func returns interval
interv_solve_bisection2(t,func,steps) =
{
  \\ print("interv_solve_bisection2()");
  t=vecsort(t);
  my(ylo=func(t[1]));
  \\ print("  ylo ",ylo*1.0);
  if(interv_contains_0(ylo), return('none));
  my(yhi=func(t[2]));
  if(interv_contains_0(yhi), return('none));
  \\ print("  yhi ",yhi*1.0);

  if(interv_sign(ylo)==interv_sign(yhi),
     error("interv_solve_bisection2() func does not change sign"));

  \\ proportion 2,1,3
  my(prop=[2,1,3]);
  for(i=1,steps,
     for(p=1,3,
        \\ print("  i="i" p="p);
        my(tmid = round((t[1]*prop[p] + t[2]*(4-prop[p]))/4),
           ymid=func(tmid));
        if(tmid==t[1] || tmid==t[2], next);
        if(interv_contains_0(ymid), next);
        if(interv_sign(ymid)==interv_sign(ylo),
           t[1]=tmid, t[2]=tmid);
        next(2));
     break);
  t;
}

interv_between(x,y) =
{
  \\ print("interv_between()");
  \\ print("  x = "x*1.0);
  \\ print("  y = "y*1.0);
  x=vecsort(x); y=vecsort(y);
  if(x[2] <= y[1], [x[2],y[1]],
     y[2] <= x[1], [y[2],x[1]],
     'none);
}

{
  if(0,
  my(bits=512,
     scale=2^bits,
     scale2=scale^2);
  my(make_potential_func(v) =
     (x) -> fold(interv_add,
            vector(#v,i,
               interv_widen_integer
                (-scale2*interv_recip
                         (interv_sub_constant(v[i],x))))));
  my(next_term(v) =
     print("next_term()");
     print("  ",v[#v-1]/scale*1.0);
     print("  ",v[#v]/scale*1.0);
     my(t=interv_between(v[#v-1],v[#v]));
     if(t=='none, return('none));
     t[1]++; t[2]--;
     if(t[1]>=t[2],return('none));
     interv_solve_bisection2(t,make_potential_func(v),bits));
  if(0,
     my(v = [[0,0],[1,1],[1/2,1/2]] * scale);
     for(n=4,9, v=concat(v,[next_term(v)]));
     v=apply(t->vecsum(t)/2, v);
     print(v);
     my(lo=(2*v[#v]+v[#v-1])/3,
        hi=(v[#v]+2*v[#v-1])/3);
     my(f(x) = sum(i=1,#v, 1/(x-v[i])));
     my(df(x) = sum(i=1,#v, -1/(x-v[i])^2));
     check_equal(lo < hi, 1);
     ploth(x=lo,hi, f(x));
     ploth(x=lo,hi, df(x));
     quit);

  my(want=[6, 8, 7, 8, 4, 1, 8, 1, 0, 3, 2, 8, 3, 8, 9, 2, 6, 3, 2, 7, 1, 3, 4, 4, 0, 4, 4, 0, 9, 8, 8, 3, 3, 4, 8, 6, 1, 1, 5, 8, 3, 9, 7, 9, 4, 8, 7, 6, 6, 8, 9, 5, 4, 1, 1, 7, 4, 7, 5, 8, 6, 6, 9, 4, 4, 1, 0, 7, 8, 5, 2, 8, 1, 7, 2, 1, 2, 4, 7, 5, 3, 8, 9, 1, 0, 8, 7, 9, 1, 2, 6, 5, 7, 8, 9, 7, 8, 5, 3, 6]);
  check_equal(want[1..39],
              [6, 8, 7, 8, 4, 1, 8, 1, 0, 3, 2, 8, 3, 8, 9, 2, 6, 3, 2, 7, 1, 3, 4, 4, 0, 4, 4, 0, 9, 8, 8, 3, 3, 4, 8, 6, 1, 1, 5]);

  my(v = [[0,0],[1,1],[1/2,1/2]] * scale);
  for(n=4,oo,
     my(x=next_term(v));
     if(x=='none,break);
     print("n="n"  ",x/scale*1.0);
     v=concat(v,[x]);

     my(t=interv_union(v[#v-1],v[#v]),
        d=interv_frac_digits(t/scale*10),
        len=min(#d,#want));
     check_equal(d[1..len], want[1..len], Str("len ",len));
     print("  t width ",(t[2]-t[1])/scale*1.0);
     print("  v width ",(v[#v][2]-v[#v][1])/scale*1.0);
     if(#d >= #want,
        print(d);
        break));
  quit);
}
{
  if(0,
  my(bits=512,
     scale=2^bits,
     scale2=scale^2);
  my(potential(pos,x,chopfunc) =
     sum(j=1,#pos, chopfunc(scale2/(x-pos[j]))));
  my(mean(x,y) = (x+y)/2);

  \\ pot_lo>0 down to pot_hi<0
  \\ p(x) = pot_lo - (pot_lo-pot_hi)*(x-lo)/(hi-lo)
  \\ p(x) == (x*(pot_hi - pot_lo) + pot_lo*hi - pot_hi*lo )/(hi-lo)
  \\ pot_lo=300; pot_hi=-100; lo=1; hi=4;
  \\ p(lo)
  \\ p(hi)
  \\ x = (pot_lo*hi - pot_hi*lo)/(pot_lo - pot_hi)
  my(find_zero(pos,lo,hi,chopfunc) =
     my(pot_lo=potential(pos,lo,chopfunc),
        pot_hi=potential(pos,hi,chopfunc));
     for(i=1,bits,
        if(! (pot_lo > 0 && pot_hi < 0),
           print("  bisect i="i);
           print("    lo ",lo/scale*1.0);
           print("    hi ",hi/scale*1.0);
           print("    lo pot ",pot_lo/scale*1.0);
           print("    hi pot ",pot_hi/scale*1.0);
           error());
        my(mid);
        if(1 || i%2,
           mid=floor(mean(lo,hi));
           ,
           mid = floor((pot_lo*hi - pot_hi*lo)/(pot_lo - pot_hi));
           (lo <= mid && mid <= hi) || error());
        my(pot_mid = potential(pos,mid,chopfunc));
        \\ print("  mid ",mid/scale*1.0);
        \\ print("    pot ",pot_mid/scale*1.0);
        if(pot_mid==0,break);
        if(pot_mid > 0, lo=mid; pot_lo=pot_mid,
                        hi=mid; pot_hi=pot_mid));
     if(chopfunc==floor, lo, hi));

  my(los=[0,1,1/2]*scale, his=los,
     xlo=(1/2+1/64)*scale,
     xhi=(1-1/64)*scale);
  for(i=1,bits,
     print("i="i);
     print("  range ",xlo/scale*1.0);
     print("        ",xhi/scale*1.0);
     my(lo=find_zero(los,xlo,xhi,floor),
        hi=find_zero(his,xlo,xhi,ceil));
     print("new lo "lo/scale*1.0);
     print("new hi "hi/scale*1.0);
     print("  ",potential(los,lo,floor)/scale*1.0);
     print("  ",potential(his,hi,ceil)/scale*1.0);
     los = concat(los,lo);
     his = concat(his,hi);
     [xlo,xhi] = if(i%2, [xlo,lo-1], [hi+1,xhi]);
  );
  my(t=[xlo,xhi]/scale);
  my(v=interv_frac_digits(t*10));
  print(concat(apply(d->Str(d),v)));
  quit);
}
{
  if(0,
  \\ 3 charges [0, 1/2], [1]
  my(p(x) = 1/(x-0) + 1/(x-1/2) - 1/(1-x));
  my(sqrt3=quadgen(12));
  check_equal(sqrt3^2 == 3, 1);
  my(sqrt12 = 2*sqrt3);
  check_equal(sqrt12^2 == 12, 1);
  check_equal(p(sqrt3/6+3/6) == 0, 1, "p solution");
  check_equal(p( (sqrt3+1)/(2*sqrt3)) == 0, 1, "p solution");
  check_equal(p(1/sqrt12 + 1/2) == 0, 1, "p solution");

  \\ plot(x=1.1,1.5,iferr(p(x),e,0));
  \\ \\ print(1/(2*sqrt3)+1/2.);
  \\ my(q(x) = 1/(x-0) + 1/(x-1/2)
  \\          - 1/(1-x) - 1/(1/sqrt12 + 1/2 - x));
  \\ print(solve(x=.55,.65, q(x)));

  \\ 1/(x-v[i]) big v small x-v big 1/(x-v)
  \\ -1/(v[i]-x) = +1/(x-v[i]) same
  my(potential(pos,x,i) = sum(j=1,#pos, 1/(x-pos[j][i])));
  check_equal(potential([[0,0],[1,1]], 1/2, 1),
              0, "potential 2");
  check_equal(potential([[0,0],[1,1],[1/2,1/2]], 1/sqrt12+1/2, 1) == 0,
              1, "potential 3");

  my(within(s,t) =
     interv_intersect(s,t) == 'none || error();
     s=vecsort(s);
     t=vecsort(t);
     if(s[2] < t[1], [s[2], t[1]], [s[1], t[2]]));
  my(shrink(t,f) =
     t=vecsort(t);
     my(w=t[2]-t[1]);
     [t[1]+w*f, t[2]-w*f]);
  my(bits=1024);
  my(next_pos(pos) =
     my(r=shrink(within(pos[#pos-1],pos[#pos]), 1/1024),
        dir=sign(potential(pos,r[1],1)));
     \\ print("dir ",dir);
     dir*sign(potential(pos,r[2],1)) == -1 || error();
     for(i=1,bits,
        \\ print("i="i" r = "r*1.0);
        sign(potential(pos,r[1],1)) == dir || error();
        sign(potential(pos,r[1],2)) == dir || error();
        sign(potential(pos,r[2],1)) == -dir || error();
        sign(potential(pos,r[2],2)) == -dir || error();
        my(rmid = vecsum(r)/2,
           lo=potential(pos,rmid,1),
           hi=potential(pos,rmid,2));
        \\ print("rmid ",rmid*1.0);
        \\ print("  lo "lo*1.0);
        \\ print("  hi "hi*1.0);
        if(sign(lo)*sign(hi)<0, break);
        if(sign(lo)*dir > 0,
           r=[rmid,r[2]],
           r=[r[1],rmid]);
     );
     r);
  my(pos=[[0,0],[1,1],[1/2,1/2]]);
  for(i=1,100,
     my(p=next_pos(pos));
     p=interv_widen_bits(p,bits);
     print("p ",p[1]*1.0);
     print("  ",p[2]*1.0);
     pos=concat(pos,[p]));
  my(v=interv_frac_digits(p));
  print(concat(apply(d->Str(d),v)));

  \\                        v
  \\ E 0.68784181032838926327134404409883348611583979...
  \\   0.6878418103283892632825647869
  \\ p 0.6878418103283892632652240576
  quit);
}


\\-----------------------------------------------------------------------------
\\ A019778  sqrt(e)

{
  if(0,
     my(t=interv_frac_fromdigits(OEIS_data("A019778"))/10);
     print(t*1.0);
     my(c=interv_contfrac(t));
     print(c);
     \\ not in OEIS: 3, 30, 1, 1, 1, 1, 1, 69, 1, 1, 3, 9
     quit);
}


\\-----------------------------------------------------------------------------
\\ A365350  1/(pi-3)

{
  my(want=[7, 0, 6, 2, 5, 1, 3, 3, 0, 5, 9, 3, 1, 0, 4, 5, 7, 6, 9, 7, 9, 3, 0, 0, 5, 1, 5, 2, 5, 7, 0, 5, 5, 8, 0, 4, 2, 7, 3, 4, 3, 1, 0, 0, 2, 5, 1, 4, 5, 5, 3, 1, 3, 3, 3, 9, 9, 8, 3, 1, 6, 8, 7, 3, 5, 5, 5, 9, 0, 3, 3, 3, 7, 5, 8, 0, 0, 5, 6, 0, 8, 3, 5, 0, 3, 9, 7, 7, 4, 7]);
  my(Pi_digits=OEIS_data("A000796",'bfile)[1..1000],
     t_Pi=interv_frac_fromdigits(Pi_digits),
     t = interv_recip(interv_sub_constant(t_Pi,3)));
  check_equal(interv_frac_digits(t,, #want), want,
              "1/(pi-3) vs DATA");
}


\\-----------------------------------------------------------------------------
\\ A365307 1/(2*e-5)
\\   2 + 1/(3 + 2/(4 + 3/(5 + 4/(6 + 5/( ... /(n+1 + n/(n+2 ... )))))))
\\   1/(2*exp(1)-5)
\\   (2*exp(1)-5)
\\   A019762 Decimal expansion of 2*e
{
  \\ (1/1)*(9 + 2/(1))
  check_equal(contfracpnqn([1,2; 9,1]),  [11,9; 1,1]);

  my(frac_matrix(n) = matrix(2,n,r,c, max(1,c+(-1)^r)));
  check_equal(frac_matrix(4), [1,1,2,3; 2,3,4,5]);
  my(by_frac(n) = interv_from_contfrac(frac_matrix(n)));
  check_equal(by_frac(1), [2,3]);
  check_equal(by_frac(2), [2+1/4, 2+1/3]);

  my(want=[2, 2, 9, 0, 6, 1, 6, 6, 9, 2, 7, 8, 5, 3, 6, 2, 4, 2, 2, 1, 0, 7, 5, 3, 3, 4, 1, 4, 5, 6, 1, 8, 4, 5, 0, 2, 5, 7, 8, 2, 0, 6, 8, 7, 3, 8, 6, 9, 0, 7, 3, 4, 6, 6, 5, 0, 5, 7, 1, 3, 1, 4, 9, 5, 0, 9, 9, 4, 1, 8, 8, 0, 3, 0, 4, 8, 7, 0, 1, 0, 8, 2, 5, 0, 1, 1, 9, 3, 9, 9]);
  my(got=digits(2 2906166927853624221));
  check_equal(got, want[1..#got], "example");
  check_equal(interv_frac_digits(by_frac(100),, #want), want);
  my(t=interv_recip(interv_add_constant(2*interv_exp([1,1],100), -5)));
  check_equal(interv_frac_digits(t,, #want), want);

  \\ 3, 2(1+floor(n/6)), 3, 1, 2(1+floor(n/6)), 1
  my(cf(n) = if(n==0, 2,
                n%6==0 || n%6==4, 1,
                n%6==1 || n%6==3, 3,
                2*(1+floor(n/6))));
  check_equal(vector(19,n,n--; cf(n)),
              [2, 3,2,3,1,2,1,3,4,3,1,4,1,3,6,3,1,6,1]);
  my(got=interv_contfrac(t));
  check_equal(#got,125);
  check_equal(got,vector(#got,n,n--; cf(n)));

  \\ A081750 Simple continued fraction of 2*e.
  \\ 5,2,3,2,3,1,2,1,3,4
  \\ 1 2 3 4 5 0 1 2 3 4 5
  my(A081750(n) = if(n==1, 5,
                     n==2, 2,
                     n%6==0 || n%6==2, 1,
                     n%6==3 || n%6==5, 3,
                     ((n+2)\6)*2));
  OEIS_check_func("A081750",A081750,['offset,1]);
  check_equal(vector(100,n,n--; cf(n)),
              vector(100,n, A081750(n+1)));
  if(0,
     recurrence_guess(vector(100,n, A081750(n)));
     quit);
}


\\-----------------------------------------------------------------------------
\\ Champernowne Constants

{
  my(champ(num_terms,base=10) =
     my(v=concat(apply(t->digits(t,base),[1..num_terms])),
        c=fromdigits(v,base),
        p=base^#v);
     [c/p, (c+1)/p]);
  check_equal(champ(2), [12/100, 13/100]);

  \\ A066716 decimal of binary Champernowne
  OEIS_check_vecfunc("A066716",
    len->interv_frac_digits(10*champ(100,2),,len));

  \\ A365238 decimal of 1/(binary Champernowne)
  my(want=[1, 1, 5, 9, 7, 6, 9, 7, 3, 2, 3, 5, 0, 6, 6, 8, 0, 7, 1, 5, 8, 6, 9, 5, 8, 1, 2, 0, 3, 3, 0, 1, 4, 8, 2, 8, 3, 3, 9, 0, 7, 4, 1, 6, 3, 1, 7, 7, 1, 5, 7, 1, 5, 9, 3, 2, 9, 7, 8, 5, 8, 2, 8, 3, 7, 5, 2, 7, 3, 7, 4, 6, 8, 9, 3, 7, 0, 1, 6, 6, 4, 7, 7, 8, 3, 2, 5, 6, 2, 1, 4, 5, 9, 5, 9, 6, 3, 1, 8, 8, 0, 1, 1, 5, 2, 1, 1]);
  check_equal(interv_frac_digits(interv_recip(champ(100,2)),, #want), want);
}


\\-----------------------------------------------------------------------------
\\ A364931  1 /(1 + exp(-x)) = x

{
  my(want=[6, 5, 9, 0, 4, 6, 0, 6, 8, 4, 0, 7, 4, 0, 6, 6, 6, 0, 9, 8, 4, 3, 8, 6, 4, 9, 5, 9, 2, 8, 8, 6, 7, 5, 5, 1, 6, 9, 8, 0, 9, 0, 3, 3, 0, 3, 5, 7, 1, 1, 5, 1, 8, 8, 4, 8, 9, 2, 9, 4, 6, 3, 0, 8, 0, 4, 2, 2, 4, 1, 1, 3, 0, 4, 2, 4, 2, 9, 7, 5, 7, 5, 7, 9, 0, 3, 4, 7, 3, 5, 6, 4, 8, 2, 8, 2, 0, 1, 0, 5]);
  my(t = interv_frac_fromdigits(want) / 10);
  print(t*1.0);
  my(e = interv_recip(interv_add_constant(interv_exp(-t,100), 1)));
  print(e*1.0);
  check_equal(interv_intersect(e,t) != 'none, 1);
  check_equal(interv_contains(t,e), 1);
}




\\-----------------------------------------------------------------------------
\\ A112330  greedy Egyptian fraction length of harmonic H(n)

{
  my(H(n) = sum(i=1,n,1/i));
  my(a(n) = my(h=H(n)); #interv_Egyptian_terms([h,h]));
  OEIS_check_func("A112330",a,['offset,1]);

  \\ greedy num terms ignoring integer part
  \\ not in OEIS: 0, 1, 2, 1, 2, 3, 4, 3, 5, 6, 4, 3, 6, 4, 5, 5, 6, 5, 5, 6, 5, 8, 8, 8, 8, 6
}



\\-----------------------------------------------------------------------------
\\ A113014  1/(2+3/(4+5/(6+7/(...))))

{
  my(make_frac(n) =
     my(f=if(n%2,n,0));
     forstep(i=n-n%2,2,-2,
        f = (i-1)/(i + f));
     f);
  my(make_interv(n) = [make_frac(2*n+2), make_frac(2*n)]);
  my(t=make_interv(1));
  for(n=2,10,
     my(u=make_interv(n));
     interv_contains(t,u) || error();
     t = u);

  my(t=make_interv(10000));
  \\ A113014 decimal
  OEIS_check_vecfunc("A113014",
    len->interv_frac_digits(10*t,,len));

  my(want=[0, 2, 1, 1, 1, 2, 1, 2, 10, 2, 2, 66, 1, 1, 13, 66, 9, 5, 8, 1, 9, 1, 1, 1, 5, 2, 2, 3, 1, 1, 1, 16, 99, 6, 1, 5, 1, 2, 1, 55, 2, 2, 1, 1, 6, 4, 1, 1, 1, 40, 1, 1, 1, 6, 14, 7, 9, 1, 1, 2, 3, 2, 2, 2, 1, 1, 2, 7, 12, 1, 2, 2, 1, 4, 2, 4, 2, 1, 3, 2, 1, 10, 7, 1, 4, 1, 119, 1, 1, 1, 3, 5, 2, 12, 1]);
  check_equal(interv_contfrac(t,#want), want,
              "contfrac");

  if(0,
     want=interv_contfrac(t);
     \\ print(want);
     default(realprecision, 100);
     my(A365105(N) = contfrac(sqrt(2*exp(1)/Pi)/
                              imag(1-erfc(I/sqrt(2)))-1, N));
     for(N=3,200,
        my(got=A365105(N),
           w=want[1..N]);
        if(w[#w]==1, w[#w-1]++;w=w[^#w]);
        if(got!= w,
           print(N);
           print(got);
           print(w)));
     quit);

  \\ A365116 Egyptian fracitons
  my(want=[3, 22, 1060, 1471180, 4470565318951, 21387196871513452925199541, 508406155285302398938678134812723800438323137635884, 293206664431782985993415302840424364324000216460330294129310314248712028637333711113413230006858255456]);
  check_equal(#want,8);
  check_equal(interv_Egyptian_terms(t,#want), want,
              "Egyptian");

  want=interv_Egyptian_terms(t);
  if(0,
     print("want length ",#want);
     my(A365116(N) = my(a, v=vector(N));
        a=sqrt(2*exp(1)/Pi)/imag(1-erfc(I/sqrt(2)))-1;
        for(i=1, N, v[i]=ceil(1/a); a=a-1/ceil(1/a)); v);
     \\ some unfortunate precision and length combinations
     \\ result in silent wrong results
     \\ (very inadequate precision may provoke underflow exception)
     my(len=8);
     forstep(p=1000,0,-1,
        default(realprecision, p);
        my(got=A365116(len));
        if(got!=want[1..len],
           print(p);
           print(got);
           print(want[1..len]);
           quit)));

  if(0,
     print("want length ",#want);
     default(realprecision, 10000);
     check_equal(vector(5,N,#A365116(N)),
                 vector(5,N,N),
                 "lengths");
     my(len=13);
     check_equal(A365116(len), want[1..len], Str(len" terms"));
     quit);

  if(0,
     my(incr2(n) = make_frac(2*n+2)-make_frac(2*n));
     print("incr2 ",vector(10,n,incr2(n)));
     print(vector(10,n,numerator(incr2(n))));
     print(vector(10,n,denominator(incr2(n))));
     my(incr(n) = make_frac(n+1)-make_frac(n));
     print("incr ",vector(10,n,incr(n)));
     print(vector(10,n,numerator(incr(n))));
     print(vector(10,n,denominator(incr(n))));
     print(vector(10,n,make_frac(n)));
     print(vector(10,n,numerator(make_frac(n))));
     print(vector(10,n,denominator(make_frac(n))));
     print(vector(10,n,numerator(make_frac(2*n))));
     print(vector(10,n,denominator(make_frac(2*n))));
     \\ A354298 Numerator of Sum_{k=1..n} (-1)^(k+1) / (2*k-1)!!
     \\ A354299 Denominators Sum_{k=1..n} (-1)^(k+1) / (2*k-1)!!
     quit);
}


\\-----------------------------------------------------------------------------
\\ A363679  sum 2^(n-1) / n!*(n-1)!    is 1/A006472(n)

{
  my(A006472(n) = n!*(n-1)!/2^(n-1));
  OEIS_check_func("A006472",A006472,['offset,1]);

  my(A363679_interv(n) =
     my(s=sum(i=1,n, 1/A006472(i)));
     [s, s + 1/A006472(n)]);

  \\ A363679 decimals
  my(want=[2, 3, 9, 4, 8, 3, 3, 0, 9, 9, 2, 7, 3, 4, 0, 4, 7, 1, 6, 5, 2, 2, 6, 3, 2, 6, 3, 6, 4, 3, 6, 3, 7, 3, 1, 5, 1, 9, 6, 8, 6, 3, 7, 0, 0, 7, 0, 9, 1, 3, 6, 2, 4, 4, 4, 7, 2, 6, 7, 9, 7, 5, 6, 3, 8, 5, 7, 2, 9, 5, 5, 5, 9, 4, 8, 3, 3, 3, 3, 4, 6, 4, 0, 5, 2, 4, 5, 5, 9, 4, 9, 9, 7, 1, 2, 3, 1, 5, 0, 7, 8, 9, 2, 5, 2, 2, 3, 6, 9, 3, 1, 1, 0, 3, 7, 8, 4, 9, 6, 1]);
  my(t=A363679_interv(80));
  check_equal(interv_frac_digits(t,,#want), want);

  my(want=[2, 2, 1, 1, 7, 7, 15, 1, 8, 1, 2, 10, 1, 1, 2, 35, 1, 1, 2, 1, 1, 4, 6, 2, 3, 2, 3, 7, 3, 2, 4, 3, 2, 3, 1, 1, 8, 1, 2, 2, 4, 1, 2, 3, 1, 1, 1, 1, 5, 1, 38, 10, 35, 4, 4, 5, 1, 4, 1, 3, 2, 1, 67, 1, 1, 1, 1, 1, 12, 1, 8, 8, 5, 1, 3, 1, 2, 2, 6, 15, 9, 9, 1, 5, 2, 38, 5, 4, 1, 2, 1, 1, 3, 1, 123, 1, 1, 1, 1, 8]);
  check_equal(interv_contfrac(t,#want), want);
}

\\-----------------------------------------------------------------------------
\\ Prime Constant = sum 1/2^prime

prime_constant_interv(n) = my(s=sum(i=1,n, 1/2^prime(i))); \
  [s, s+1/2^prime(n)];
{
  \\ A051006 decimal
  OEIS_check_vecfunc("A051006",
    len->my(t=prime_constant_interv(10*(len+1)));
         interv_frac_digits(10*t,,len));

  \\ A036680 Egyptian fractions
  OEIS_check_vecfunc("A036680",
    len->my(t=prime_constant_interv(20*(len+1)));
         interv_Egyptian_terms(t,len));

  \\ A051007 continued fraction
  OEIS_check_vecfunc("A051007",
    len->my(t=prime_constant_interv(len+1));
         interv_contfrac(t,len));

  if(0,
     \\ continued fraction convergents
     my(t=prime_constant_interv(100),
        v=interv_contfrac(t));
     print(vector(10,n,numerator(contfrac_to_rational(v[1..n]))));
     print(vector(10,n,denominator(contfrac_to_rational(v[1..n]))));
     \\ not in OEIS: 0, 1, 2, 5, 17, 209, 27396, 27605, 220631, 248236
     \\ not in OEIS: 1, 2, 5, 12, 41, 504, 66065, 66569, 532048, 598617
     quit);

  if(0,
     \\ Engel expansion
     my(t=prime_constant_interv(100));
     print(interv_Engel_terms(t));
     \\ not in OEIS: 3, 5, 5, 10, 85, 100, 3325, 3541, 4202, 4756, 8025, 22975
     quit);
}


\\-----------------------------------------------------------------------------
\\ A365052  contfrac squares

\\ 1, 4, 37, 596, 14937, 538328
\\ 1, 5, 46, 741, 18571, 669297
A036245(n) = n>=1 || error(); contfracpnqn(vector(n,i, i^2))[2,1];
A036246(n) = n>=1 || error(); contfracpnqn(vector(n,i, i^2))[1,1];
OEIS_check_func("A036245",,['offset,1]);
OEIS_check_func("A036246",,['offset,1]);
{
  check_equal(vector(10,n,n+=2; A036245(n)),
              vector(10,n,n+=2; n^2*A036245(n-1) + A036245(n-2)));
  check_equal(vector(10,n,n+=2; A036246(n)),
              vector(10,n,n+=2; n^2*A036246(n-1) + A036246(n-2)));

  \\ x(n)/y(n) - x(n-1)/y(n-1) = 1/d(n)
  \\ d(n) = 1/( x(n)/y(n) - x(n-1)/y(n-1))
  \\      = y(n)*y(n-1) / ( x(n)*y(n-1) - x(n-1)*y(n) )
}

A365052_interv(len) = interv_from_contfrac(vector(len,i,sqr(i)));
{
  my(want=[1, 2, 4, 3, 2, 8, 8, 4, 7, 8, 3, 9, 9, 7, 1, 5, 6, 4, 4, 0, 8, 2, 4, 9, 6, 5, 4, 5, 3, 9, 4, 4, 2, 9, 4, 9, 9, 2, 3, 1, 2, 0, 0, 2, 6, 1, 1, 9, 7, 4, 4, 6, 8, 8, 5, 0, 6, 6, 4, 9, 7, 4, 5, 9, 8, 8, 1, 6, 3, 0, 3, 2, 2, 3, 3, 8, 2, 5, 3, 4, 2, 1, 4, 5, 9, 6, 4, 9, 8, 1, 5, 6, 1, 2, 1, 8, 5, 5, 9, 5]);
  \\ code by Rok Cestnik
  my(p(N) = my(m=contfracpnqn(vector(N, i, i^2))); m[1, 1]/m[2,1]);
  my(A365052(N) = my(t=2); while(floor(10^N*p(t)) != floor(10^N*p(t+1)), t++); digits(floor(10^(N-1)*p(t))));
  check_equal(vector(10,N, #A365052(N)),
              vector(10,N, N),
              "with N");
 check_equal(vector(50,N, A365052(N)),
             vector(50,N, want[1..N]),
             "with N");

  my(t=A365052_interv(100));
  check_equal(interv_frac_digits(t,,#want), want);

  my(make(n) = vector(n,i, i^2));
  check_equal(contfracpnqn(make(20),20)[1,],
              vector(20,n, A036246(n)),
              "convergents numerators");
  check_equal(contfracpnqn(make(20),20)[2,],
              vector(20,n, A036245(n)),
              "convergents denominators");

  check_equal(test_contfrac_to_rational([w,x,y,z]) - test_contfrac_to_rational([w,x,y]) ==
              1/(x^2*y^2*z + x^2*y + 2*x*y*z + x + z),  1);
}
{

  if(0,
     my(t=A365052_interv(100));
     print(interv_Engel_terms(t,20));
     print(interv_Egyptian_terms(t,20));
     quit);

  if(0,
     my(make(n) = vector(n,i,i--;i));
     my(make(n) = vector(n,i,i--; 1));
     print(vector(10,n, 1/(test_contfrac_to_rational(make(n+1))
                         - test_contfrac_to_rational(make(n)))));
     \\ 1, -3, 30, -430, 9675, -313425, 13896568, -810061176, 60152482785, -5547776645035

     print(vector(10,n, numerator(test_contfrac_to_rational(make(n)))));
     \\ a(n+1) = Sum_{k = 0..floor((n-1)/2)}
     \\    (n-2*k-1)!*binomial(n-k-1,k) * binomial(n-k,k+1)

     print(t[1]*1.0);
     my(make(n) = vector(n, i,i--; i^2));
     print(test_contfrac_to_rational([w,x,y,z]) - test_contfrac_to_rational([w,x,y]));
     my(conv(n) = test_contfrac_to_rational(make(n)));
     my(incr(n) = conv(n+1) - conv(n));
     print("incr ", vector(10,n, 1/incr(n)));
     \\ not in OEIS: 1, 5, 230, 34086, 13761111, 12429514587, 21962394750828
     print(vector(10,n, conv(n)));
     print(vector(10,n, numerator(test_contfrac_to_rational(vector(n,i,i^2)))));
     print(vector(10,n, denominator(test_contfrac_to_rational(vector(n,i,i^2)))));
     \\ A036246 CONTINUANT transform of squares 1, 4, 9, ...
     \\   a(n) = n^2 * a(n-1) + a(n-2)
     \\ A036245
     \\   a(n) = n^2 * a(n-1) + a(n-2)
     );
}

A073824_interv(len) = interv_from_contfrac(vector(len,i,i--; sqr(i)));
{
  my(want=OEIS_data("A073824"));
  my(t=A073824_interv(100));
  check_equal(interv_frac_digits(10*t,,#want), want);
  print(1.0/t[1]);
}

\\-----------------------------------------------------------------------------
\\ sum reciprocals A006472(n) = n!*(n-1)!/2^(n-1)

{
  my(want=[2, 3, 9, 4, 8, 3, 3, 0, 9, 9, 2, 7, 3, 4, 0, 4, 7, 1, 6, 5, 2, 2, 6, 3, 2, 6, 3, 6, 4, 3, 6, 3, 7, 3, 1, 5, 1, 9, 6, 8, 6, 3, 7, 0, 0, 7, 0, 9, 1, 3, 6, 2, 4, 4, 4, 7, 2, 6, 7, 9, 7, 5, 6, 3, 8, 5, 7, 2, 9, 5, 5, 5, 9, 4, 8, 3, 3, 3, 3, 4, 6, 4, 0, 5, 2, 4, 5, 5, 9, 4, 9, 9, 7, 1, 2, 3, 1, 5, 0, 7, 8, 9, 2, 5, 2, 2, 3, 6, 9, 3, 1, 1, 0, 3, 7, 8, 4, 9, 6, 1]);

  my(A006472(n) = n!*(n-1)!/2^(n-1));
  my(make(n) = my(s=sum(i=1,n, 1/A006472(i)));
     [s, s + 1/A006472(n)]);
  my(t=make(100));
  check_equal(interv_frac_digits(t,,#want), want);

  my(e=digits(2 3948330992734047165226326364363731519686370070913624447267975638));
  check_equal(e, want[1..#e]);
}


\\-----------------------------------------------------------------------------
\\ A364872  lex smallest contfrac equal sum reciprocals

sum_reciprocals(v) = sum(i=1,#v, 1/v[i]);
sqrtint_ceil(x) = my(r,s=sqrtint(ceil(x),&r)); s+(r!=0);
{
  my(v=[2, 1, 2, 2, 5, 95, x]);
  check_equal(test_contfrac_to_rational(v),
              (9804*x + 103)/(3617*x + 38));
  check_equal(sum_reciprocals(v),
              (103*x + 38)/(38*x));
  check_equal(contfracpnqn(v[^#v]), [9804, 103; 3617, 38]);
  check_equal(test_contfrac_to_rational(v[^#v]), 9804/3617);
  check_equal(sum_reciprocals(v[^#v]),      103/38);
  check_equal(9804/3617 - 103/38, 1/137446);

  my(p=test_contfrac_to_rational(v) - sum_reciprocals(v));
  check_equal(p*x*(3617*x + 38),
              1/38*x^2 - 3617*x - 38);
  check_equal(p, (9804*x + 103)/(3617*x + 38) - 1/x - 103/38);
  check_equal((9804*x + 103)*x - (3617*x + 38) - 103/38*x*(3617*x + 38),
              1/38*x^2 - 3617*x - 38);

  check_equal(p, (x^2 - 137446*x - 1444)/(137446*x^2 + 1444*x));
  p = (x^2 - 137446*x - 1444);
  check_equal(poldisc(p), 18891408692);
  \\ 137446.01050...
  p /= 2;
  my(a,b,c); [a,b,c] = Vec(p);
  check_equal(a, 1/2);
  check_equal(ceil(-b + sqrtint_ceil(b^2-4*a*c)),
              137447);
  print(polroots(p));
}
A364872_next_term(v) =
{
  my(m=contfracpnqn(v),
     num=Pol(m[1,]),
     den=Pol(m[2,]),
     p = num*x - den*(1 + sum_reciprocals(v)*x));
  p *= denominator(Vec(p));
  my(a,b,c); [a,b,c] = Vec(p);
  denominator(a)==1 || error();
  denominator(b)==1 || error();
  denominator(c)==1 || error();
  ceil((-b + sqrtint_ceil(b^2-4*a*c)) / (2*a));
}

A364872_vector(len) =
{
  my(v=vector(len,i,if(i==2,1,2)));
  for(i=5,len,
     v[i] = A364872_next_term(v[1..i-1]));
  v;
}
{
  my(want=[2, 1, 2, 2, 5, 95, 137447, 19092121105, 1456654254113777258001, 8728918703159963392919895262580124849062181]);
  check_equal(A364872_vector(#want), want);

  check_equal(vector(#want,n, my(v=want[1..n]);
    test_contfrac_to_rational(v) > sum_reciprocals(v)),
              vector(#want,n, 1),
              "contfrac above");

  check_equal(vector(#want-1,n,n++;
                     lex(test_contfrac_to_rational(want[1..n]),
                         test_contfrac_to_rational(want[1..n-1]))),
              vector(#want-1,n, -(-1)^n),
              "contfrac decreasing");

  my(v=[]);
  for(n=0,8,
     my(t = if(n<=3, want[n+1], A364872_next_term(v)));
     v=concat(v,t);
     \\ print("n="n" t="t"  "v);
     my(diff = test_contfrac_to_rational(v) - sum_reciprocals(v));
     diff > 0 || error("n="n" t="t" diff="diff);
     if(n>=2,
        my(u=v); u[#u] = t-1;
        my(diff = test_contfrac_to_rational(u) - sum_reciprocals(u));
        diff <= 0 || error("n="n" other diff="diff));
     );

  if(0, foreach(A364872_vector(13),t,print(t)); quit);
  if(0,
     my(v=[2, 1, 2, 2]);
     my(estimate(v) = ceil(1/(test_contfrac_to_rational(v) - sum_reciprocals(v))));
     for(n=4,30,
        my(t=A364872_next_term(v),
           e=estimate(v));
        print("n="n"  ",t-e,"   ",logint(t,10));
        v=concat(v,t));
     quit);
}

\\--------
\\ A364873 decimals
{
  my(want=[2, 7, 1, 0, 5, 3, 3, 5, 9, 1, 3, 7, 3, 5, 1, 0, 7, 8, 7, 3, 3, 8, 6, 4, 5, 6, 6, 2, 0, 4, 8, 1, 7, 0, 1, 1, 1, 5, 1, 8, 3, 3, 4, 9, 9, 3, 0, 7, 0, 4, 4, 7, 6, 3, 7, 9, 4, 3, 4, 3, 9, 0, 9, 5, 0, 8, 3, 0, 4, 7, 0, 0, 0, 8, 2, 0, 7, 6, 8, 6, 1, 8, 7, 3, 1, 3, 1, 8, 2, 2, 1, 9, 6, 8, 7, 2, 2]);
  my(e=digits(2 71053359137351078733864566));
  check_equal(e,want[1..#e], "example digits");
  my(c=A364872_vector(13),
     t=interv_from_contfrac(c));
  check_equal(interv_frac_digits(t,,#want), want,
              "digits");

  \\ code by Rok Cestnik
  my(cf(a) = my(m=contfracpnqn(a)); m[1, 1]/m[2, 1]);
  my(uf(a) = sum(i=1, #a, 1/a[i]));
  my(A364872(N) = my(a=[2]); for(i=2, N, a=concat(a, if(cf(a)==uf(a), a[i-1], ceil(1/(cf(a)-uf(a))))); while(cf(a)<=uf(a), a[i]++)); a);
  my(A364873(N) = my(t=2); while(floor(10^N*cf(A364872(t))) != floor(10^N*cf(A364872(t+1))), t++); digits(floor(10^(N-1)*cf(A364872(t)))));

  want=interv_frac_digits(t,,600);
  check_equal(#want,600);
  for(N=1,#want, A364873(N)==want[1..N] || error(N));
}


\\-----------------------------------------------------------------------------
\\ A364837  initial decimal digit of 2^(2^n)

{
  my(a(n) =
     my(t=[2,2],
        p=n+10,  \\ precision digits
        L=10^p);
     for(i=1,n,
        t=interv_pow_constant(t,2);
        t /= 10^(logint(t[1],10)-p);
        t=interv_widen_integer(t));
     t/=10^logint(t[1],10);
     interv_floor(t));
  my(want=[2, 4, 1, 2, 6, 4, 1, 3, 1, 1, 1, 3, 1, 1, 1, 1, 2, 4, 1, 2, 6, 4, 2, 4, 1, 3, 1, 1, 1, 2, 4, 1, 3, 9, 9, 8, 7, 5, 2, 8, 8, 6, 4, 1, 3, 9, 9, 9, 9, 9, 8, 7, 5, 2, 8, 7, 6, 3, 1, 2, 5, 3, 1, 1, 1, 3, 1, 1, 3, 9, 8, 7, 5, 3, 1, 1, 1, 3, 1, 2, 4, 2, 5, 2, 6, 4, 1, 2]);
  check_equal(vector(#want,n,n--; a(n)), want,
              "A364837 2^2^n vs DATA");
}

\\-----------------------------------------------------------------------------
\\ A364855  initial decimal digit of 3^(3^n)

{
  my(A010705(n) = if(n%2==0,3,7));
  OEIS_check_func("A010705",A010705,['offset,0]);
  my(final(n) = lift(Mod(3,10)^(3^n)));
  check_equal(vector(20,n,n--; final(n)),
              vector(20,n,n--; A010705(n)));

  my(cubings(n) = my(t=3); for(i=1,n,t=t^3); t);
  check_equal(vector(8,n,n--; cubings(n)),
              vector(8,n,n--; 3^(3^n)));

  \\ Power of 10 to reduce each time doesn't have to be exact.
  \\ Could decide from the bit length of t[1].
  \\
  my(a(n) =
     my(t=[3,3],
        p=n+10,  \\ precision digits
        L=10^p);
     for(i=1,n,
        t=interv_pow_constant(t,3);
        t /= 10^(logint(t[1],10)-p);
        t=interv_widen_integer(t));
     t/=10^logint(t[1],10);
     interv_floor(t));
  my(want=[3, 2, 1, 7, 4, 8, 6, 2, 2, 1, 3, 3, 6, 2, 1, 3, 3, 4, 6, 2, 2, 1, 1, 1, 5, 1, 2, 1, 1, 7, 4, 1, 1, 4, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 2, 1, 1, 2, 2, 1, 1, 1, 7, 4, 8, 6, 2, 1, 2, 1, 3, 4, 1, 1, 1, 4, 8, 6, 2, 2, 1, 2, 2, 1, 5, 1, 6, 3, 3, 4, 1, 1, 2, 1, 5, 1, 4, 1]);
  check_equal(a(0), 3);
  check_equal(vector(#want,n,n--; a(n)), want,
              "A364855 3^3^n vs DATA");
  \\ for(i=1,#want,my(n=i-1);
  \\    check_equal(a(n),want[i], Str("n="n)));
}


\\-----------------------------------------------------------------------------
\\ A363746  initial decimal digit of tetration n^^n

{
  \\ 4^^4 = 4^(2^512) so start from 4 and square 512 times
  \\ A241293 = digits
  \\ need e=180 at least or the interval hi explodes
  \\ e=300 enough for data section of A241293
  \\
  \\ Power of 10 to reduce each time doesn't have to be exact.
  \\ Could decide from the bit length of t[1].
  \\
  my(t=[4,4],
     e=300,
     L=10^e);
  for(i=1,512,
     \\ print(i" "t);
     t=interv_mul(t,t);
     t /= 10^(logint(t[1],10)-e);
     t=interv_widen_integer(t));
  t/=10^logint(t[1],10);
  check_equal(floor(t), [2,2]);
  OEIS_check_vecfunc("A241293",
    (len) -> interv_frac_digits(t,,len));

  if(0,
     print(interv_frac_digits(t,,20));
     quit);
}


\\-----------------------------------------------------------------------------
\\ A363841 contfrac Sum_{k>=0} 1/(k!)!^2

{
  my(make_t(len) =
     my(v= vector(len,k,k--; 1/(k!)!^2));
     [vecsum(v), vecsum(v) + v[#v]]);
  my(t = make_t(8),
     got = interv_contfrac(t));
  \\ print(t);
  \\ print(got);
  my(want=[2, 3, 1, 32399, 4, 1432456210278611587930429493084159999, 1, 3, 32399, 1, 3]);
  check_equal(got[1..#want], want);
  my(a(n) = got[n+1]);
  check_equal(#digits(a(11)), 303);

  if(0,
     \\ peaks (((k+1)!)!  / ((k!)!)^2)^2 - 1
     my(vector_search(v,target) =
        for(i=1,#v, if(v[i]==target,return(i))); 0);
     for(k=0,10,
        my(x=(((k+1)!)!  / ((k!)!)^2)^2 - 1);
        print(k"  "vector_search(got,x));
        );
     quit);

  if(0,
     print("want "want);
     forstep(p=32,10000,32,
        default(realbitprecision,p);
        my(c=contfrac(suminf(k=0, 1/k!!^2)));
        if(c!=got[1..#c],
           print("p="p);
           print("c    ",c);
           print("got  ",got[1..#c]);
          ));
     quit);

  \\----------
  \\ A363842  decimals
  my(want=[2, 2, 5, 0, 0, 0, 1, 9, 2, 9, 0, 1, 2, 3, 4, 5, 6, 7, 9, 0, 1, 2, 3, 4, 5, 6, 7, 9, 0, 1, 2, 3, 4, 5, 6, 7, 9, 0, 1, 2, 3, 4, 5, 6, 7, 9, 0, 1, 4, 9, 4, 3, 3, 7, 7, 0, 1, 0, 4, 2, 8, 4, 9, 4, 1, 6, 2, 2, 0, 3, 5, 2, 4, 4, 3, 4, 4, 8, 0, 7, 7, 5, 8, 5, 2, 2, 5]);
  check_equal(interv_frac_digits(t,,#want), want,
             "decimals");
  check_equal(make_t(4)[1], 1166401/518400);
  check_equal(digits(2 25000192901234567901234567901234567901234567901),
              want[1..48]);

  check_equal(2 25000192901234567901234567901234567901234567901,
              floor(1166401/518400 * 10^47));

  if(0,
     print(bestappr(t[1],518400));
     my(v=want[1..48]);
     my(f=fromdigits(v)/10^47);
     print(f*1.0);
     for(den=1,518400,
        my(num=ceil(f*den));
        if(digits(floor(num/den*10^47))==v,
           print(num" / "den);
           break));
     quit);
}

\\-----------------------------------------------------------------------------
\\ A005371 Lucas(Lucas(n)) big terms initial decimals

A005371(n) = Lucas(Lucas(n));
reduce_exp = 100;
reduce_10s(t,e) =
{
  \\ print("reduce_10s() "t" e="e);
  my(d = if(t[2] >= 10, logint(floor(t[2]),10))
         - reduce_exp);
  \\ print("  d="d);
  if(d>0,
     t/=10^d; e+=d;
     t = interv_widen_integer(t));
  [t,e];
}
{
  \\ check_equal(reduce_10s([2*10^30,3*10^30],4),
  \\             [[2*10^20,3*10^20],14],
  \\             "reduce_10s()");
}

\\ interval t and integer n
pow_and_exponent10(t,n) =
{
  \\ print("pow_and_exponent10() "t*1.0" n="n);
  my(p=[1,1],e=0);
  forstep(i=logint(n,2),0,-1,
     \\ print("i="i" p="p);
     p = interv_mul(p,p);
     e *= 2;
     \\ print("  square p="p);
     if(bittest(n,i), p = interv_mul(p,t));
     [p,e] = reduce_10s(p,e);
     \\ print("  reduce p="p*1.0" e="e);
  );
  [p,e];
}
{
  check_equal(pow_and_exponent10([2,2],6),
              [[64, 64], 0]);
}
A005371_interv_and_exponent10(n) =
{
  my(t_phi = interv_widen_bits([phi,phi],1000));
  n=Lucas(n);
  n>=0 || error();
  my(xt,xe); [xt,xe] = pow_and_exponent10(t_phi,n);
  xt = interv_add(xt,[-1,1]);
  [xt,xe];
}
A005371_digits_and_exponent10(n) =
{
  if(n<10,
     my(v=digits(A005371(n)));
     [v,#v-1];
     ,
     my(t,e); [t,e] = A005371_interv_and_exponent10(n);
     my(d=logint(floor(t[2]),10));
     t/=10^d; e+=d;
     [interv_frac_digits(t), e]);
}

{
  check_equal(vector(10,n, Lucas(n)),
              vector(10,n, simplify(phi^n + (-1/phi)^n)));
  check_equal(Lucas(55),  312119004989);
  check_equal(A005371(5), 199);
  check_equal(A005371(10), 50755107359004694554823204);

  if(0,
     for(n=1,10,
        my(v,e); [v,e] = A005371_digits_and_exponent10(n);
        print("e="e"  "v);
     );
     \\ approximations by Andrea Pinos in A005371
     my(want=[[0,  3,   0 ],
              [1,  1,   0 ],
              [2,  4,   0 ],
              [3,  7,   0 ],
              [4,  2 9,   1 ],
              [5,  1 99,   2 ],
              [6,  5 778,   3 ],
              [7,  1 149851,   6 ],
              [8,  6 643838879,   9 ],
              [9,  7 639424778862807,   15 ],
              [10,  5 07551073590046,   25 ],
              [11,  3 87739824812222,   41 ],
              [12,  1 96797764357060,   67 ],
              [13,  7 63063306752435,   108 ],
              [14,  1 50169152831785,   176 ],
              [15,  1 14588570332033,   285 ],
              [16,  1 72076685309669,   461 ],
              [17,  1 97180213571102,   746 ],
              [18,  3 39301175599680,   1207 ],
              [19,  6 69034782696712,   1953 ],
              [20,  2 27004288286071,   3161 ],
              [21,  1 51873764684693,   5115          ],
              [22,  3 44759958615750,   8276          ],
              [23,  5 23599928275132,   13391         ],
              [24,  1 80516289603344,   21668         ],
              [25,  9 45183162888043,   35059         ],
              [26,  1 70620957560103,   56728         ],
              [27,  1 61268056321645,   91788         ],
              [28,  2 75157101934557,   148516        ],
              [29,  4 43740510120829,   240304        ],
              [30,  1 22098352775809,   388821        ],
              [31,  5 41799853456507,   629125        ],
              [32,  6 61528696412147,   1017946       ],
              [33,  3 58416150773375,   1647072       ],
              [34,  2 37102568994170,   2665019       ],
              [35,  8 49813901173695,   4312091       ],
              [36,  2 01493059135241,   6977111       ],
              [37,  1 71231602643141,   11289203      ],
              [38,  3 45019794371967,   18266314      ],
              [39,  5 90782923339192,   29555517      ],
              [40,  2 03831802728958,   47821832      ],
              [41,  1 20420348285711,   77377350      ],
              [42,  2 45454966763256,   125199182     ],
              [43,  2 95577725860891,   202576532     ],
              [44,  7 25510208771439,   327775714     ],
              [45,  2 14444657597522,   530352247     ],
              [46,  1 55581788303498,   858127962     ],
              [47,  3 33636833211540,   1388480209    ],
              [48,  5 19078151549675,   2246608171    ],
              [49,  1 73183590672333,   3635088381    ],
              [50,  8 98958181249306,   5881696552    ],
              [51,  1 55684805693025,   9516784934    ],
              [52,  1 39954129773953,   15398481487   ],
              [53,  2 17887314997944,   24915266421   ],
              [54,  3 04942295593206,   40313747908   ],
              [55,  6 64430580161134,   65229014329   ] ]);
     for(i=1,#want,
        my(n=want[i][1],
           want_v=digits(want[i][2]),
           want_e=want[i][3]);
        my(v,e); [v,e] = A005371_digits_and_exponent10(n);
        v=v[1..#want_v];
        check_equal([v,e], [want_v,want_e],
                    Str("decimals n="n));
     );
     quit);
}


\\-----------------------------------------------------------------------------
\\ A359698  initial decimals of 2^k and 3^k

\\ high n decimal digits of b^p
pow_high_digits(b,p,n) =
{
  my(t,e); [t,e] = pow_and_exponent10([b,b],p);
  my(d=logint(floor(t[2]),10)+1);
  t/=10^d; e+=d;
  \\ print("div to t="t*1.0);
  (1/10 <= t[1] && t[2] < 1) || error();
  t*=10^n;
  \\ print("scale "n" to t="t*1.0);
  interv_floor_to_constant(t);
}
{
  my(want=[1, 17, 193, 619, 2016, 91958, 91958, 8186278, 45392361, 977982331, 26450915298]);
  for(i=1,#want,
     my(n=i-1);
     my(two   = pow_high_digits(2,want[i],n));
     my(three = pow_high_digits(3,want[i],n));
     print("2 and 3  n="n" "two," ",three);
     check_equal(two,three, Str("high digits ",n));
  );
}

\\ A088995 initial decimals of 2^k and 5^k
A088995(n) =
{
  for(k=1,oo,
     my(u=digits(2^k), v=digits(5^k));
     if(#u>=n && u[1..n]==v[1..n], return(k)));
}
{
  my(want=OEIS_data("A088995"));
\\   check_equal(#want, 17);
\\   check_equal(want[9],  961700165, "existing a(9)");
\\   check_equal(want[10], 961700165, "existing a(10)");
\\   check_equal(want[11], 248045160416, "existing a(11)");
\\   want[11]             = 83322853582;
\\
\\   check_equal(want[12],   1404948108914, "existing a(12)");
\\   check_equal(want[13],  13801435928724, "existing a(13)");
\\   want[13]             =  7603192018819;
\\   check_equal(want[14], 243632550916041, "existing a(14)");
\\   want[14]            = 167022179253602;
\\   check_equal(want[15], 5729166542536373, "existing a(15)");
\\   want[15]            = 3550275020220728;
\\   check_equal(want[16], 5729166542536373, "existing a(16)");
\\   check_equal(want[17], 331484151442699072, "existing a(17)");
\\   want[17]            = 106675272785875442;
\\
\\   want=Vec(want,22);
\\   want[18] = 331484151442699072;
\\   want[19] = 2330288635428230258177;
\\   want[20] = 2330288635428230258177;
\\   want[21] = 2330288635428230258177;
\\   want[22] = 14491551451088705995591;
\\   want[22] = 614215972715036565237000;

  for(n=1,#want,
     my(two  = pow_high_digits(2,want[n],n));
     my(five = pow_high_digits(5,want[n],n));
     print("2 and 5  n="n" k="want[n]"   "two," ",five);
     check_equal(two,five,
                 Str("high digits two and five n=",n));

     if(want[n] < 100000,
        check_equal(A088995(n),want[n]));
  );
}


\\-----------------------------------------------------------------------------
\\ A361873  contfrac of 3n+1
\\   equals besseli(4/3,2/3) / besseli(1/3,2/3) + 1

{
  my(want=[1, 2, 4, 1, 4, 9, 5, 7, 1, 9, 5, 7, 9, 3, 0, 3, 1, 1, 3, 0, 1, 9, 9, 6, 6, 3, 7, 6, 3, 0, 6, 4, 5, 0, 3, 2, 3, 4, 8, 0, 8, 5, 8, 6, 7, 1, 2, 5, 3, 6, 1, 3, 4, 8, 6, 4, 5, 4, 5, 9, 6, 2, 3, 3, 5, 6, 7, 5, 5, 9, 2, 4, 2, 7, 5, 6, 7, 2, 9, 7, 4, 4, 0, 6, 3, 9, 2, 6, 1, 7, 6, 9, 8, 7, 3, 2, 4, 5, 9, 7, 9, 4, 5, 7, 4, 9, 9, 7, 4, 1, 5, 5, 7, 1, 2, 0, 7, 6, 6, 7]);
  my(t_want = interv_frac_fromdigits(want));
  my(t=interv_from_contfrac(vector(100,i,3*i-2)));
  print(t*1.0);
  check_equal(interv_contains(t_want,t), 1,
              "A361873 contfrac 3n+1 vs DATA");
}


\\-----------------------------------------------------------------------------
\\ A362006
\\   minimum integer m such that
\\   floor(e^n) = floor(Sum_{k=0..m} (n^k)/(k!))
\\ cf
\\ A000149 a(n) = floor(e^n).

{
  my(e = interv_exp([1,1],150));  \\ e = exp(1)
  my(A000149(n) =
     interv_floor_to_constant(interv_pow_constant(e,n)));
  my(a(n) =
     my(target=A000149(n),f=0);
    for(m=0,oo,
       f += (n^m)/(m!);
       if(floor(f)==target, return(m))));

  my(want=[1, 4, 8, 9, 12, 15, 17, 19, 24, 25, 29, 30, 34, 37, 39, 41, 44, 48, 49, 52, 55, 59, 61, 62, 66, 68, 70, 74, 79, 79, 82, 84, 89, 89, 92, 96, 98, 102, 103, 106, 110, 112, 114, 116, 122, 124, 126, 128, 132, 133, 137, 138, 141, 144, 147, 151, 152, 154, 158, 161, 164, 168, 169, 170, 174]);
  check_equal(#want, 65);
  check_equal(vector(#want,n,a(n)), want);

  if(0,
  my(a_by_float(n) = my(m=0, x=floor(exp(n)), y=1);
     print("x="x);
     print("  ",A000149(n));
     while(floor(y) != x,
           if(floor(y) > x, error("past x"));
           my(old_y=y);
           m++; y += n^m/m!;
           \\ if(m>3 && floor(y) > floor(old_y)+1, error("m="m));
          );
     m);
  check_equal(vector(66,n,a(n)),
              vector(66,n,a_by_float(n)));


  \\ default(realbitprecision,96);
  \\ default(realbitprecision,256);
  check_equal(a(88),  236);
  check_equal(a(177), 478);
  \\ check_equal(a_by_float(88), 236);
  for(n=1,oo,
     my(got=a_by_float(n));
     print(n" "got);
     if(got>3*n,error(n));
  );
     quit);
}
\\-----------------------------------------------------------------------------
\\ A361972  log series thing

{
  my(want=[7, 9, 4, 6, 7, 8, 6, 4, 5, 4, 5, 2, 8, 9, 9, 4, 0, 2, 2, 0, 3, 8, 9, 7, 9, 6, 2, 0, 6, 5, 1, 4, 9, 5, 1, 4, 0, 6, 4, 9, 9, 9, 5, 9, 0, 8, 8, 2, 8, 0, 4, 9, 6, 8, 9, 0, 1, 5, 1, 2, 0, 9, 5, 0, 1, 4, 8, 1, 7, 8, 5, 8, 9, 6, 0, 6, 8, 7, 5, 6, 6, 6, 9, 6, 6, 1, 4, 7, 7, 7, 6, 2, 7, 3, 3]);
  my(t_want = interv_frac_fromdigits(want)/10);
  my(t_A241005 = interv_frac_fromdigits(OEIS_data("A241005"))/10);
  my(t_A074785 = interv_frac_fromdigits(OEIS_data("A074785"))/10);
  \\ print(t_want*1.0);
  \\ print(t_A241005*1.0);
  \\ print(t_A074785*1.0);
  my(t_sum = interv_add(t_A241005, t_A074785));
  check_equal(interv_contains(t_want,t_sum), 1);
}

\\-----------------------------------------------------------------------------
\\ A050210  Egyptian fraction largest from x/y

Egyptian_fraction_largest_of_rational(f) = \
  vecmax(interv_Egyptian_terms([f,f]));

A050210_T(n,k) = Egyptian_fraction_largest_of_rational(k/n);
A050210_row(n) = vector(n-1,k,A050210_T(n,k));
OEIS_check_rowfunc("A050210_row",,['offset,2]);


\\-----------------------------------------------------------------------------
\\ A362289  Egyptian fraction 1/n + 1/(n+1) largest denominator


{
  my(want=[2, 3, 12, 180, 30, 1428, 56, 2520, 90, 2310, 132, 100292556, 182, 9240, 240, 119952, 306, 614444040, 380, 23100, 462, 42190274940, 552, 77390453400, 650, 201474, 756, 23370247110, 870, 200880, 992, 14523137084239067683872, 1122, 2206260, 1260, 104845560637757648698080]);
  my(a(n) = my(f=1/n + 1/(n+1));
     vecmax(interv_Egyptian_terms([f,f])));
  check_equal(vector(#want,n,a(n)), want,
              "A362289 vs DATA");

  \\ formula
  check_equal(vector(50,n, a(n)),
              vector(50,n, A050210_T(n*(n+1), 2*n+1)),
              "A362289 vs A050210 table");

  \\ example
  check_equal(1/16 + 1/17, 33/272);
  check_equal(1/16 + 1/17, 1/9 + 1/98 + 1/119952);
  check_equal(a(16), 119952);

  check_equal(a(58),   3327895);
  check_equal(a(58)%3, 1);
  if(0,
     print(vector(100,n,a(n)%3));
     for(n=1,1000,
        if(n%6==1,next);
        a(n)%3 == 0 || error(n));
     quit);
}


\\-----------------------------------------------------------------------------
\\ A060997  contfrac of 1,2,3,...

{
  my(want=OEIS_data("A060997"));
  my(t_want = interv_frac_fromdigits(want));
  my(t=interv_from_contfrac([1..100]));
  \\ print(t*1.0);
  check_equal(interv_contains(t_want,t), 1,
              "A060997 vs DATA");
}

\\-----------------------------------------------------------------------------
\\ A359698  k frac of (3/2)^k record near frac Pi
\\

{
  my(want=[1, 2, 4, 7, 24, 25, 27, 49, 189, 698, 5780, 9407, 13230, 17823, 19517, 23443, 177689, 240673]);

  my(Pi_bits=OEIS_data("A004601"),
     t_Pifrac = interv_sub_constant(
                  2*interv_frac_fromdigits(Pi_bits,2),
                  3),
     t_pow = [1,1],
     t_best = interv_sub(t_pow,t_Pifrac),
     n=1);
  if(0,
     print(t_Pifrac*1.0);
     interv_contains([0,1],t_Pifrac) || error();
     for(k=1,oo,
        t_pow = interv_widen_bits(interv_mul(t_pow,3/2),
                                  50000);
        my(t_frac = interv_frac(t_pow),
           t_delta = interv_sub(t_frac,t_Pifrac));
        if(t_delta[2] < 0,
           t_delta = - t_delta);
        if(interv_lex(t_delta,t_best) < 0,
           t_best = t_delta;
           print1(k",");
           want[n] == k || error();
           n++));
     quit);
}

\\-----------------------------------------------------------------------------
\\ A049384  exponential factorial

\\ code by Charles R G in A049384
A049384(n)=if(n>1, n^A049384(n-1), 1);
OEIS_check_func("A049384",,['offset,0]);

{
  \\ a(5)
  my(t=A049384(5),
     v=digits(t));
  check_equal(t, 5^262144);
  check_equal(#v, 183231);
  my(want=digits(6 2060698786608744707483205572846793));
  check_equal(v[1..#want], want);
}

{
  \\ a(6)
  \\ decimals length
  \\ a(6) = 6^5^262144
  \\ log10(6^5^262144)
  \\ log(6^5^262144) / log(10)
  \\ 5^262144 * log(6) / log(10) + 1

  my(t_log6=interv_frac_fromdigits(OEIS_data("A016629")),
     t_log10=interv_frac_fromdigits(OEIS_data("A002392")));
  \\ print(t_log6*1.0);
  \\ print(t_log10*1.0);

  my(t = 5^262144 / 10^183230 * interv_div(t_log6, t_log10));

  my(want_digits = digits(4 829261036048226));
  want_digits = want_digits[^#want_digits];
  my(t_want = interv_frac_fromdigits(want_digits));
  \\ print("want ",want_digits);
  \\ print("got  ",interv_frac_digits(t,,#want_digits));
  \\ print(t*1.0);
  \\ print(t_want*1.0);

  check_equal(interv_contains(t_want, t), 1);
}



\\-----------------------------------------------------------------------------
\\ A355554  base 60 1 radian = 180/pi

{
  my(Pi_digits=OEIS_data("A000796",'bfile)[1..1000],
     t_Pi=interv_frac_fromdigits(Pi_digits));

  \\ A060707 = pi in base 60
  OEIS_check_vecfunc("A060707",(len)->interv_frac_digits(t_Pi,60,len));

  my(want=[57, 17, 44, 48, 22, 29, 22, 22, 7, 32, 46, 14, 58, 15, 20, 17, 32, 7, 4, 43, 35, 36, 12, 35, 9, 17, 4, 12, 9, 40, 27, 27, 26, 48, 25, 12, 52, 48, 52, 18, 21, 42, 13, 53, 32, 25, 44, 46, 54, 25, 56, 34, 21, 51, 6, 35, 33, 34, 49, 6, 43, 10, 36, 31, 50, 20, 31]);
  my(t_radian=interv_div([180,180],t_Pi));
  check_equal(interv_frac_digits(t_radian,60,#want), want);

  \\ A072097 = 180/pi = 1 radian decimal
  OEIS_check_vecfunc("A072097",
    (len)->interv_frac_digits(t_radian/10,,len));
}


\\-----------------------------------------------------------------------------
\\ A359295   hydrogen ionization eV
\\   Rh*h*c/e

{
  my(want=[1, 3, 6, 0, 5, 6, 9, 3, 1, 2, 2]);

  my(rH_vector=OEIS_data("A081821"),  \\ 10973731.568...
     h_vector=OEIS_data("A003676"),   \\ 6.62607015 * 10^{-34)
     c_vector=OEIS_data("A003678"),   \\ 299792458 exactly
     e_vector=OEIS_data("A081823"));  \\ 1.602176634*10^-19 exactly
  rH_vector=rH_vector[^#rH_vector];
  check_equal(logint(299792458,2), 28);

  my(t_rH=interv_frac_fromdigits(rH_vector) * 10^7,
     t_h=interv_frac_fromdigits(h_vector) * 10^-34,
     c=fromdigits(c_vector),
     e=fromdigits(e_vector) * 10^-(19+#e_vector-1));

  if(0,
     print(t_rH*1.0);
     print(t_h*10^34.);
     print(c);
     print(e*10^19.);

     my(t=interv_mul(t_rH,t_h) * c / e);
     print(t*1.0);
     my(v=interv_frac_digits(t/10));
     print(v);
     print(want);
     quit);
}

\\-----------------------------------------------------------------------------
\\ A317335 yet another paperfolding  turn 1=left, -2=right
\\ a(4*n-3) = 1, a(4*n-1) = -2, a(2*n) = a(n)

A317335(n) = n>=1 || error(); \
  if(bittest(n,valuation(n,2)+1), -2, 1);
OEIS_check_func("A317335",,['offset,1]);

\\ A317336 yet another paperfolding  next turn -7=left, -10=right
\\ but exception at n=1
A317336(n) =
{
  \\ n=1 which decrements to 0 is treated as the "1 bit" case
  n>=1 || error();
  if(!(n--) || bittest(n,valuation(n,2)+1), -7, -10);
}
OEIS_check_func("A317336",,['offset,1]);

\\ which is where Liouville contfrac term 1
A317331(n) = n>=1 || error(); \
  n<<2 - bittest(n,valuation(n,2)+1);
OEIS_check_func("A317331",,['offset,1]);

\\ which is where Liouville contfrac term 8
\\ 8n and +1 or -2
\\ 9, 17, 22, 33, 41
A317332(n) = n>=1 || error(); 8*n + A317335(n);
OEIS_check_func("A317332",,['offset,1]);
is_A317332(n) =
{
  (n+=2)>=8 &&
  bitand(n,7) == 3*!bittest(n,valuation(n>>3,2)+4);
}
OEIS_check_predfunc("is_A317332");

\\ which is where Liouville contfrac term 9
\\ 8n and -7 or -10, exception n=1
\\   1, 6, 14, 25, 30
A317333(n) = n>=1 || error(); 8*n + A317336(n);
OEIS_check_func("A317336",,['offset,1]);
is_A317333(n) =
{
   n==1 || ((n+=2)>=8
           && bitand(n,7) == 3*bittest(n,valuation(n>>3,2)+4));
}
OEIS_check_predfunc("is_A317333");


\\-----------------------------------------------------------------------------
\\ Liouville Contfrac
\\ A012245 decimal digits, charfunc of factorials
\\ A058304 contfrac terms
\\ 0, 9, 11, 99,  1, 10, 9, 999999999999,    1,
\\    8, 10,  1, 99, 11, 9, 999...999,       1,
\\    8, 11, 99,  1, 10, 8, 1,    999999999999,
\\    9, 10,  1, 99, 11, 9

{
  my(term(n) = 1/10^(n!));
  my(limit=7,
     lo=sum(i=1,limit, term(i)),
     hi=lo + term(limit),
     t=[lo,hi]);
  print(interv_contfrac(t,12));
  if(0,
     \\ Engel is some powers of 10
     print(interv_Engel_terms(t));
     quit);
  if(0,
     \\ Egyptian is simply the 1/10^(n!)
     print(interv_Egyptian_terms(t));
     quit);

  \\ decimal digits
  OEIS_check_vecfunc("A012245",
                     (len)->interv_frac_digits(t,,len+1)[^1]);
  \\ continued fraction terms
  \\ FIXME:
  \\ OEIS_check_vecfunc("A058304",
  \\                    (len)->interv_frac_digits(t,,len+1)[^1]);

  \\ a(n) = 1 iff n in A317331     4n - BitAbove
  \\ a(n) = 8 iff n in A317332     8n +1 or -2
  \\ a(n) = 9 iff n in A317333     8n -2 or +1
  \\ a(n) = 10 iff 8n + 2 + 3*(n mod 2)   b010
  \\ a(n) = 11 iff 8n + 5 - 3*(n mod 2)   b101
  \\ a(n) = 10^((m-1)*m!)-1 iff n in
  \\ {2^m* (4*k+1) - 1 | k >= 0}    01 00..00
  \\ {2^m* (4*k+3)     | k >= 0}    11 00..00
}




\\-----------------------------------------------------------------------------
\\ A359456 charfunc, Fiboville
\\   1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0,
\\
\\ vector(6,n,n++; (n-1)*n!)
\\   [2, 12, 72, 480, 3600, 30240
\\ m=3; 10^((m-1)*m!)-1
\\ logint(999999999999999999+1,10)
\\ 999999999999999999 == 10^18-1

\\ A359458 =  0, 2, 18, 180, 2640, 59280, 2096640,
\\ eset         [2, 18, 180, 2640, 59280]

{
  \\ A359838 continued fraction expansion of interpreted as binary
  my(want=[0, 1, 3, 3, 1, 2, 1, 262143, 3, 1, 3, 3, 1,
           1532495540865888858358347027150309183618739122183602175,
           4, 3, 1, 3, 262143, 1, 2, 1, 3, 3, 1]);
  my(e=1, s=0, width=1);
  for(n=2,9, e*=fibonacci(n); s += (width=1/2^e));
  print(s*1.0);
  my(t=[s,s+width]);
  my(got=interv_contfrac(t,#want));
  check_equal(got,want);
  if(0,
     print(interv_contfrac(t));
     quit);
}

{
  if(0,
  my(want=[0, 9, 11, 99, 1, 10, 9, 999999999999999999, 1, 8, 10, 1, 99, 11, 9]);

  my(e=1, s=0, width=1);
  for(n=2,9,
     e*=fibonacci(n); s += (width=1/10^e));
  print(s*1.0);
  my(t=[s,s+width]);
  my(got=interv_contfrac(t,#want));
  check_equal(got,want);

  my(got=[0, 9, 11, 99, 1, 10, 9, 999999999999999999, 1, 8, 10, 1, 99, 11, 9]);
  check_equal(got,want[1..#got]);

  my(epow10(n) = my(e=logint(n,10));
     if(n==10^e,e,0));
  print(got);
  got=interv_contfrac(t);
  my(eset=[]);
  for(n=1,#got,
     my(e=epow10(got[n]+1));
     if(e>1,
        print(n" 10^"e" - 1"); eset=setunion(eset,[e]);
        ,
        print(n" "got[n])));
  print("eset ",eset);
  quit);
}

\\-----------------------------------------------------------------------------
\\ A052119  contfrac 0; 1,2,3,...
\\   0.697774657964007982

\\ A060997  contfrac 1; 2,3,4,...
\\   1.433127426722311758

\\ A021085  decimal expansion of 1/81
\\   0.1234567901234567901234
\\   1.234567891011121314151
{
  if(0,
     my(t=interv_frac_fromdigits(concat(vector(120,i,digits(i-1)))));
     print(t*1.0);
     quit);
}
{
  my(t=interv_from_contfrac(vector(20,i,i-1)));
  print(t*1.0);
  quit;
}

\\-----------------------------------------------------------------------------
\\ A329356  Kolakoski bits

{
  my(k2=OEIS_data("A000002"),
     kbits=apply(t->2-t,k2));
  my(a(n) = fromdigits(kbits[1..n],2));
  OEIS_check_func("A329356",a,['offset,0]);

  \\ as a constant
  if(0,
     \\ kbits [1,0,0,1,1,0,1,0,0,1,0,0,1,1,0,1,1,0,0,1]
     \\   A000002  1, 2, 2, 1, 1, 2, 1, 2, 2, 1, 2, 2, 1, 1,
     \\            1221121221
     \\   A118270  0.110010110
     \\ .60274640361026036187981879218197

     \\ 1 - 0.7945071927794792762 / 2

     my(t=interv_frac_fromdigits(kbits,2,0));
     \\ my(t=interv_frac_fromdigits(apply(b->1-b,kbits),2,0));
     print(kbits[1..20]);
     print(t*1.0);
     print(interv_frac_digits(t));
     print(fromdigits(interv_frac_digits(t)));
     quit);
}


\\-----------------------------------------------------------------------------
\\ Newton  1/4pi = 4 arctan(1/5) - arctan(1/239)

{
  my(width_digits(t) = logint(floor(1/abs(t[1]-t[2])),10));
  my(t5=interv_atan_Taylor([1/5,1/5], 120));
  my(t239=interv_atan_Taylor([1/239,1/239], 36));
  my(t = 4*interv_add(t5*4, -t239));
  \\ print("t5   ",width_digits(t5));
  \\ print("t239 ",width_digits(t239));
  \\ print("t ",width_digits(t)"  ",t*1.0);
  \\ print(t*1.0);

  my(Pi_digits=OEIS_data("A000796",'bfile)[1..1000],
     t_Pi=interv_frac_fromdigits(Pi_digits));
  \\ print("t_Pi ",width_digits(t_Pi)"  ",t_Pi*1.0);
  check_equal(interv_contains(t, t_Pi), 1);
}

\\-----------------------------------------------------------------------------
\\ A072398, A072399  num/den of best contfrac den <= 10^n
\\    3, 22, 22, 355, 355, 312689, 1146408,
\\   ~/OEIS/b072398.txt

{
  \\ A001203 pi contfrac
  my(v=OEIS_data("A001203",'bfile));
  my(m=contfracpnqn(v,2000),
     len=#m);

  \\ A002486 pi contfrac convergents denominators
  \\ print(m[2,][1..20]); quit;

  my(want_nums=OEIS_data("A072398"));
  my(want_dens=OEIS_data("A072399"));
  \\ my(want_nums=OEIS_data("A072398",'bfile));
  \\ my(want_dens=OEIS_data("A072399",'bfile));

  my(den_to_n(d) = if(d==1,0, logint(d-1,10)+1));
  check_equal(den_to_n(10^5), 5);
  check_equal(den_to_n(10^5-1), 5);
  check_equal(den_to_n(10^4+1), 5);
  check_equal(den_to_n(10^4), 4);
  my(hi=den_to_n(m[2,len]));
  print("hi = "hi);

  my(nums=vector(hi),
     dens=vector(hi));
  for(i=1,len,
     \\ print(m[1,i]," / ",m[2,i]);
     my(n=den_to_n(m[2,i]));
     n>=1 || next;
     nums[n] = m[1,i];
     dens[n] = m[2,i]);
  for(n=1,hi,
     if(nums[n]==0,
        nums[n]=nums[n-1];
        dens[n]=dens[n-1]));
  nums=nums[^hi];
  dens=dens[^hi];
  nums=concat(3,nums);
  dens=concat(1,dens);
  print("nums len ",#nums," dens len ",#dens);

  \\ nums=nums[1..10];
  \\ want_nums=want_nums[1..10];
  \\ dens=dens[1..10];
  \\ want_dens=want_dens[1..10];
  check_equal(want_nums, nums[1..#want_nums], "nums");
  check_equal(want_dens, dens[1..#want_dens], "dens");

  \\ default(realprecision,3*32*16);
  \\ want_nums = [3,22,22,355,355,312689,1146408,5419351,245850922,2549491779,21053343141,21053343141,1783366216531,8958937768937,139755218526789,1564184217101490,1564184217101490,43229178266965627,2629158953199395797,23145244623808076139,156758394460257741379,1320873730600286763652,23618968756344904004357,217854213729505283093821,992153446160946225924373,992153446160946225924373,79759425309091418545365116,1990017018942641678730430408,7482503677362179149875455309,229490032580876524402362417848,2251995320763534810803101877034,24312968463237129870029395811678,43110693640855860975252218139695,2678378249018681779230444098144751,25379927325216193111226340605165592,196365028326003268372222240705032706,196365028326003268372222240705032706,19943740338751939836336170601863402455,158960827625037508885572698092792121522,576208454512220219301654502806283311429,18220074860778079692526735102143388669547];
  \\ check_equal(want_nums - nums[1..#want_nums], "nums from bestappr()");
}


\\-----------------------------------------------------------------------------
\\ A357715  sqrt(16 + 32/sqrt(5))
\\   x = sqrt(16 + 32/sqrt(5))
\\   x^2 = 16 + 32/sqrt(5)
\\   x^2 - 16 = 32/sqrt(5)
\\   (x^2 - 16)^2 - 32^2/5 = 0
\\
\\ W = sqrt(2 - 2 / sqrt(5))
\\ H = sqrt(2 + 2 / sqrt(5))
\\ (W/2)^2 + (H/2)^2   \\ 1
\\ H/W   \\ phi
\\ 2*W+2*H = 5.5055...
\\ (4 / sqrt(5)) * sqrt(5 + 2 * sqrt(5))
\\ sqrt(5 + 2 * sqrt(5)) / (sqrt(5) / 4)
\\ 4 * sqrt(1 + 2 / sqrt(5))
\\ from A019952 = sqrt(25 + 10*sqrt(5))/5

my(c=1600+640*quadgen(20)); \
A357715_vector(len) = digits(sqrtint(floor(c*100^(len-2))));
{
  my(p(x) =  (x^2 - 16)^2 - 32^2/5);
  my(want=[5, 5, 0, 5, 5, 2, 7, 6, 8, 1, 8, 8, 4, 6, 9, 4, 1, 5, 2, 8, 2, 8, 8, 3, 8, 3, 2, 7, 6, 4, 3, 5, 5, 0, 7, 1, 8, 1, 0, 3, 5, 9, 7, 3, 4, 4, 0, 3, 2, 6, 3, 4, 6, 5, 3, 4, 6, 2, 7, 0, 3, 0, 6, 2, 4, 7, 6, 3, 8, 0, 7, 7, 5, 0, 6, 8, 6, 9, 1, 9, 4, 0, 2, 6, 3, 8, 1, 1, 9, 7, 2, 4, 4, 0, 2, 8, 0]);
  my(t=interv_frac_fromdigits(want));
  check_equal(p(t[1]) < 0, 1);
  check_equal(p(t[2]) > 0, 1);
  check_equal(A357715_vector(#want), want,
              "A357715_vector() vs DATA");
  check_equal(vector(100,len,len--; #A357715_vector(len)),
              vector(100,len,len--; len));
  check_equal(vector(500,len,len--; A357715_vector(len)),
              vector(500,len,len--; A357715_vector(len+1)[1..len]));

  my(t=interv_frac_fromdigits(A357715_vector(1000)));
  check_equal(p(t[1]) < 0, 1);
  check_equal(p(t[2]) > 0, 1);

  my(v=digits(5 5055276818846941));
  check_equal(v, want[1..#v]);

  my(c = 1600 + 640*quadgen(4*5));
  my(a_vector(len) = digits(sqrtint(floor(c*100^(len-2)))));
  check_equal(vector(40,len,len--; #a_vector(len)),
              vector(40,len,len--; len));
  check_equal(vector(40,len,len--; a_vector(len)),
              vector(40,len,len--; want[1..len]));
  \\ print(c);
  \\ print(a_vector(0));
  \\ print(a_vector(1));
  \\ print(a_vector(10));
  \\ quit;

  \\ for(p=1,500,
  \\    default(realprecision, p+8); x = sqrt
  \\              (16 + 32 / sqrt(5));
  \\
  \\              \\ Display the constant as an
  \\              example:
  \\
  \\              print(concat(vecextract(Vec(Str(x)),
  \\              Str("1.."p+1))));

  if(0,
     \\ sqrt(16 + 32/sqrt(5))
     \\ 5.505527
     my(make_vec(e) =
        default(realprecision,e);
        digits(floor(sqrt(16 + 32/sqrt(5)) * 10^e)));
     my(want=a_vector(1000));
     check_equal(make_vec(6),
                 [5, 5,0,5,5,2,7]);
     for(e=10,#want-1,
        my(got);
        iferr(got=make_vec(e),err,
              print("e="e" error");
              next);
        if(got!=want[1..e+1],
           print("bad at e="e);
           print(got[max(1,#got-5)..#got]);
           print(want[#got-5..#got]);
           print("prec ",default(realprecision),
                 " words ",default(realbitprecision)/32);
          ));
     quit);
  if(0,
     \\ sqrt(16 + 32/sqrt(5))
     my(p=5);
     my(scale=1000000,
        t=[5,5]*scale^2);
     print("5 = ",t*1.0);
     t = interv_sqrtint(t) / scale;
     print("sqrt(5) = ",t*1.0);
     t = interv_recip(t);
     print("1/sqrt(5) = ",t*1.0);
     t *= 32;
     print("32/sqrt(5) = ",t*1.0);
     t = interv_add_constant(t, 16);
     t = interv_sqrtint(t*scale^2) / scale;
     print(t[1]*1.0);
     print(t[2]*1.0);
     quit);
}


\\-----------------------------------------------------------------------------
\\ Pi
\\ A006784  Engel expansion pi
{
  my(digs=OEIS_data("A000796",'bfile)[1..300],
     t=interv_frac_fromdigits(digs));

  \\ Engel
  OEIS_check_vecfunc("A006784",
    len->interv_Engel_terms(t,len));
}


\\-----------------------------------------------------------------------------
\\ A165420  2^(2^(n-2-(n>2)))
\\   Engel expansion of KM + 1

A165420(n) = 1<<(1<<(n-2-(n>2)));
{
  my(a=A165420);
  OEIS_check_func("A165420",,['offset,1]);

  \\ code by G. C. Greubel in A165420
  my(b(n) = if(n==1, 1, if(n==2, 2,
               prod(j=1, n-1, self()(j)))));
  check_equal(vector(8,n,b(n)),
              vector(8,n,a(n)));
}

\\-----------------------------------------------------------------------------
\\ Kempner-Mahler

{
  my(L = 10,
     lo = sum(i=0,L, 1/2^(2^i)),
     hi = lo + 2/2^(2^(L+1)),
     t=[lo,hi]);
  check_equal(1/2^(2^0), 1/2, "KM first term");
  check_equal(1/2^(2^1), 1/4, "KM second term");
  \\ print(t*1.0);

  \\ decimal 0.81642150902189314370....
  OEIS_check_vecfunc("A007404",
    (len)->interv_frac_digits(10*t,,len));

  \\ contfrac
  OEIS_check_vecfunc("A007400",
    (len)->interv_contfrac(t,len));

  \\ Engel
  \\ A165420 = a(1) = 1, a(2) = 2, then product of previous
  \\   1,2,2,4,16,256,65536,4294967296
  my(got=interv_Engel_terms(interv_add_constant(t,1)));
  check_equal(#got,12);
  check_equal(vector(#got,n, A165420(n)), got);

  my(got=interv_Engel_terms(t));
  check_equal(vector(#got,n,n++; A165420(n)), got);
}

\\-----------------------------------------------------------------------------
\\ Primes Constant, primes charfunc as binary

{
  my(v=vector(1000,n, isprime(n)),
     t=interv_frac_fromdigits(v,2,0));

  \\ decimal 0.414682509851111660
  my(want=OEIS_data("A051006"));
  my(got=interv_frac_digits(t,,#want+1)[^1]);
  check_equal(got,want, "A051006 prime constant decimal");

  \\ Egyptian
  my(want=OEIS_data("A036680"));
  my(got=interv_Egyptian_terms(t,#want));
  check_equal(got,want, "A051006 prime constant Egyptian");
  \\ example in A051006
  my(got=digits(011010100010100010100010000010100));
  check_equal(v[1..#got+1], concat(0,got));
  \\ 1/2^3 + 1/2^13 + 1/2^226.
  \\ 1/3 + 1/13 + 1/226 + 1/757098 + 1/1493980747140.
  \\ 0.41468250

  if(0,
     print(interv_Engel_terms(t,12));
     \\ not in OEIS: 3, 5, 5, 10, 85, 100, 3325
     quit);
}


\\-----------------------------------------------------------------------------
\\ A357469
\\   real root of x^3 - x^2 + x - 2
{
  my(f(x) = x^3 - x^2 + x - 2);
  my(want=OEIS_data("A357469"),
     t=interv_solve_bisection([1,2],f,1000),
     tf=interv_subst(f(x),x,t));
  check_equal(interv_contains_0(tf), 1,
              "A357469 tf crosses 0");
  my(got = interv_frac_digits(t,,#want));
  check_equal(got, want);

  check_equal(f(x-1), x^3 - 4*x^2 + 6*x - 5);
}

\\-----------------------------------------------------------------------------
\\ A197032 Philo line 2,1
\\   real root of x^3 - 4*x^2 + 6*x - 5
\\   data fixed Nov 2022

{
  my(f(x) = x^3 - 4*x ^2 + 6*x - 5);
  my(want=[2, 3, 5, 3, 2, 0, 9, 9, 6, 4, 1, 9, 9, 3, 2, 4, 4, 2, 9, 4, 8, 3, 1, 0, 1, 3, 3, 2, 5, 7, 7, 3, 8, 8, 4, 5, 7, 2, 7, 0, 7, 0, 5, 6, 1, 3, 8, 5, 6, 8, 4, 6, 8, 2, 6, 8, 0, 6, 6, 9, 3, 0, 4, 2, 6, 5, 1, 5, 1, 8, 9, 7, 2, 3, 2, 2, 0, 9, 2, 0, 8, 5, 9, 1, 6, 5, 8, 0, 3],
     t=interv_solve_bisection([2,3],f,1000),
     tf=interv_subst(f(x),x,t));
  check_equal(interv_contains_0(tf), 1,
              "tf crosses 0");
  my(got = interv_frac_digits(t,,#want));
  check_equal(got, want);
}

\\-----------------------------------------------------------------------------
\\ A357969  sum j!/primorial(j)
\\ 2.240053652689050110257

{
  my(term(n) = n!/vecprod(primes(n)));
  my(limit=200,
     s=sum(i=0,limit,term(i)),
     t=[s,s+term(limit)]);
  my(want=OEIS_data("A357969"));
  my(got=interv_frac_digits(t,,#want));
  check_equal(got,want);

  my(e=2 240053652689050110257);
  check_equal(digits(e), want[1..#digits(e)]);
}


\\-----------------------------------------------------------------------------
\\ A001466 Pi-3 Egyptian Fraction
\\    8, 61, 5020, 128541455, 162924332716605980, 28783052231699298507846309644849796
\\ vs 8, 61, 5020, 128541457, 7862753935607729
\\
\\ A182257 Pi  Egyptian Fractions
\\   1, 1, 1, 8, 61, 5020, 128541455,
\\   using run of 1s for the integer part
\\
\\ A224230 Pi  Egyptian Fractions
\\   3, 8, 61, 5020, 128541455,
\\   using 3 for integer part

{
  my(Pi_digits=OEIS_data("A000796",'bfile)[1..1000],
     t_Pi=interv_frac_fromdigits(Pi_digits),
     t_Pi_sub3 = interv_sub_constant(t_Pi,3));

  \\ Pi-3
  my(want=OEIS_data("A001466"));
  my(got=interv_Egyptian_terms(t_Pi_sub3,#want));
  check_equal(got,want, "A001466 Pi-3 Egyptian");

  \\ 7, 790, 749896, 1270073831726
  my(want=OEIS_data("A014013"));
  my(got=interv_Egyptian_terms_alternating(t_Pi_sub3,#want));
  check_equal(got,want, "A014013 Pi-3 Egyptian alternating");



  \\ Pi
  my(want=OEIS_data("A182257"));
  my(got=interv_Egyptian_terms(t_Pi,#want));
  check_equal(got,want, "Pi Egyptian");
}

\\-----------------------------------------------------------------------------
\\ A336810  contfrac sum 1/(k!)!
\\

A336810_sum_interval(k_max) =
{
  my(s=0,t);
  for(k=0,k_max,
     t = 1/(k!)!;
     s += t);
  [s, s+t];
}
{
  my(t=A336810_sum_interval(6));
  my(want=OEIS_data("A336810"));
  check_equal(interv_contfrac(t,#want), want,
              Str("A336810 new data length ",#want));

  my(want=OEIS_data("A336810"));
  check_equal(interv_contfrac(t,#want), want,
              Str("A336810 data length ",#want));

  if(0,
     my(want=OEIS_data("A336810",'bfile));
     my(a(n) = want[n+1]);
     check_equal(#digits(a(11)), 152);
     check_equal(#digits(a(21)), 1349);
     check_equal(#digits(a(41)), 12981);

     check_equal(interv_contfrac(t,#want), want,
              Str("A336810 bfile length ",#want)));

  if(0,
     my(m=contfracpnqn(want,10));
     print(m[1,]);
     print(m[2,]);
     \\ not in OEIS: 2, 3, 5, 898, 1801, 2155531580867214949497
     \\ not in OEIS: 1, 1, 2, 359, 720, 861733891296165887639
     quit);
}



\\-----------------------------------------------------------------------------
\\ A004601 Pi in Binary
\\ A180433 bitwise complement
\\ suminf(k=1,1/2^prime(k))

{
  my(Pi_digits=OEIS_data("A000796",'bfile)[1..3100],
     t_Pi=interv_frac_fromdigits(Pi_digits));

  my(A004601_vector(len) =
     interv_frac_digits(t_Pi/2,2,len));
  OEIS_check_vecfunc("A004601_vector",A004601_vector);

  my(A180433_vecctor(len) =
     apply(t->1-t, interv_frac_digits(t_Pi/2,2,len)));
  OEIS_check_vecfunc("A180433_vecctor",A180433_vecctor);
  if(0, OEIS_check_vecfunc("A180433_vecctor",A180433_vecctor,'bfile));
  check_equal(#A180433_vecctor(10),10);
  check_equal(#A180433_vecctor(10000),10000);
  if(0,
     my(v=A180433_vecctor(10000));
     system("rm -f      /tmp/b.txt");
     for(n=1,#v, write("/tmp/b.txt",n," ",v[n]));
     system("ls -l      /tmp/b.txt");
     quit);
}



\\-----------------------------------------------------------------------------
\\ sqrt(27)
\\   A248254  decimal
\\   A248254  Egyptian
\\   Engel only A028257 of sqrt(3)

{
  my(sqrt_digits=OEIS_data("A010482"),
     t_sqrt=interv_frac_fromdigits(sqrt_digits),
     t_sqrt_exact=vector(2,i, 3*quadgen(4*3)));
  check_equal(interv_contains(t_sqrt, t_sqrt_exact), 1);

  \\ Egyptian fractions = 5, 6, 34, 13516
  \\   so 5 + 1/6 + 1/24 + ...
  my(want=OEIS_data("A248254"),
     got=interv_Egyptian_terms(interv_sub_constant(t_sqrt_exact,4), #want));
  got[1] += 4;
  check_equal(got,want, "sqrt(27) Egyptian");
}


\\-----------------------------------------------------------------------------
\\ Engel expansion sqrt(157)

\\ Engel
\\ not in OEIS: 1, 4, 84, 148, 185, 2503, 6978, 23592
\\ Egyptian
\\ not in OEIS: 1, 4, 334, 417252, 203698861781

{
  \\ my(sqrt_digits=OEIS_data("A178310"),
  \\    t_sqrt=interv_frac_fromdigits(sqrt_digits),
  \\    want=OEIS_data(A006784),
  \\    got=interv_Egyptian_terms(t_sqrt,#want));
  \\ check_equal(got,want);
}


\\-----------------------------------------------------------------------------
\\ Pi Continued Fraction

\\ return number of starting elements u[i]==v[i]
vector_match_length(u,v) =
{
  my(len=min(#u,#v));
  for(i=1,len, if(u[i]!=v[i],return(i-1)));
  len;
}

{
  my(Pi_digits=OEIS_data("A000796",'bfile)[1..1100],
     t_Pi=interv_frac_fromdigits(Pi_digits));
  my(l=interv_contfrac(t_Pi));
  print("contfrac length ",#l);

  \\ matrix rows nums and dens
  my(m_convs=contfracpnqn(l,#l+1));
  if(0,
     for(i=1,min(3,#m_convs),
        print(m_convs[1,i]" / "m_convs[2,i])));

  my(A002485_convergent_num(n) =
     n>=0 || error();
     if(n<=1,n, m_convs[1,n-1]));
  my(A002486_convergent_den(n) =
     n>=0 || error();
     if(n<=1,1-n, m_convs[2,n-1]));
  OEIS_check_func("A002485_convergent_num",A002485_convergent_num);
  OEIS_check_func("A002486_convergent_den",A002486_convergent_den);

  OEIS_check_vecfunc("A002485",
    len->concat([0,1],m_convs[1,][1..len-2]));
  OEIS_check_vecfunc("A002486",
    len->concat([1,0],m_convs[2,][1..len-2]));

  my(convergent(n) =
     A002485_convergent_num(n) / A002486_convergent_den(n));
  check_equal(convergent(5), 355/113);

  \\ num_match() returns how many digits of pi are correct in the initial
  \\ digits of fraction f.
  \\
  \\ "estimate" is an estimated number of digits which is used for where to
  \\ look for a digit of f disagreeing with pi.  A good estimate is a
  \\ speedup but the result is the same no matter what estimate.
  \\
  my(num_match(f,estimate=32) =
     while(1,
           my(v=digits(floor(f*10^estimate)));
           #v < #Pi_digits || error();
           my(got=vector_match_length(Pi_digits,v));
           if(got<#v, return(got));
           estimate=ceil(estimate*1.1+30)));
  check_equal(num_match(22/7), 3,
             "22/7 = 3.142");

   \\ A356665 num digits of convergents
  my(want=[1, 3, 5, 7, 10, 10, 10, 10, 12, 11, 13, 13, 15, 16, 16, 17, 18, 18, 19, 20, 22, 24, 25, 25, 26, 28, 30, 31, 31, 33, 34, 35, 38, 40, 41, 41, 42, 43, 45, 46, 46, 47, 48, 50, 51, 52, 52, 54, 55, 56, 56, 57, 57, 59, 60, 60, 61, 61, 62, 61, 63, 65, 64]);
  check_equal(#want,63);

  my(A356665(n) =
     n>=2 || error();
     num_match(convergent(n), ceil(1.05*n+20)));
  check_equal(vector(#want,n,n++; A356665(n)), \\ OFFSET=2
              want,
              "A356665 num digits vs DATA");
  \\ print(vector(#want,n,A356665(n)));

  if(0,
        \\ while(f>=1, f/=10);
        \\ for(i=1,#Pi_digits,
        \\    \\ print("at f="f" "f*1.0);
        \\    f*=10;
        \\    my(d=floor(f));
        \\    f-=d;
        \\    if(d!=Pi_digits[i],return(i-1));
        \\ error("match more than Pi_digits"));

     print("A356665 terms");
     my(record=-oo,
        rlo=oo);
     for(n=1,#l,
        my(got=A356665(n));
        my(numlen=logint(convergent_num(n),10)+1);
        my(denlen=logint(convergent_den(n),10)+1);
        my(tot=numlen+denlen,
           diff = got - tot);
        print("  "n" "got" diff ",diff,
              if(diff>record,
                 record=diff;"  HIGH",
                 ""),
              if(diff<rlo,
                 rlo=diff;"  LOW",
                 "")));
     quit);
}



\\-----------------------------------------------------------------------------
\\ https://oeis.org/draft/A265296
\\ Sum_{n>=1} (c(2*n) - c(2*n-1))
\\   convergents to sqrt(3)
{
  if(0,
     my(want=OEIS_data("A265296"),
        bits=1000,
        lo=sqrtint(3<<(2*bits)) / 2^bits,
        hi=lo+1/2^bits);
     check_equal(lo^2<3, 1);
     check_equal(hi^2>3, 1);
     my(t=[lo,hi],
        cfrac=interv_contfrac(t),
        convmat=contfracpnqn(cfrac,#cfrac),
        convs=vector(#convmat,i, convmat[1,i]/convmat[2,i]));
  \\ for(i=1,8,print(convs[2*i]*1.0));
  \\ for(i=1,8,print((convs[2*i]-convs[2*i-1])*1.0));
     my(diffs=vector(#convs\2,i, convs[2*i]-convs[2*i-1]));
     my(altsum=vecsum(diffs));
     print("final diff ",diffs[#diffs]*1.0);
     print(floor(altsum*10^100));
     quit);
}

\\-----------------------------------------------------------------------------
\\ A356032  decimals x^4 + x - 1

{
  my(want=[7, 2, 4, 4, 9, 1, 9, 5, 9, 0, 0, 0, 5, 1, 5, 6, 1, 1, 5, 8, 8, 3, 7, 2, 2, 8, 2, 1, 8, 7, 0, 3, 6, 5, 6, 5, 7, 8, 6, 4, 9, 4, 4, 8, 1, 3, 5, 0, 0, 1, 1, 0, 1, 7, 2, 7, 0, 3, 9, 8, 0, 2, 8, 4, 3, 7, 4, 5, 2, 9, 0, 6, 4, 7, 5, 1]);
  my(f(x) = x^4 + x - 1);
  my(t = interv_solve_bisection([1/2,1], f, 1000),
     lo= fromdigits(want)   /10^#want,
     hi=(fromdigits(want)+1)/10^#want);
  check_equal(interv_frac_digits(t,,#want+1),
              concat(0,want),
              "interv_frac_digits");
  check_equal(f(lo) < 0, 1);
  check_equal(f(hi) > 0, 1);

  my(e=digits(724491959000515611588372282187036565786494481350011017270));
  check_equal(e,want[1..#e]);

  \\ A060007 other real root (negative)
  my(t = - interv_solve_bisection([-2,-1], f, 1000),
     want=OEIS_data("A060007"));
  check_equal(interv_frac_digits(t,,#want),
              want,
              "A060007 interv_frac_digits");

  localprec(1000);
  my(rr=polroots(f(x)),
     z=rr[4]);
  check_equal(floor(real(z)*10^9),
              248126062);
  check_equal(floor(imag(z)*10^9),
              1 033982060);

  my(Ap = (3/16)*(9 + sqrt(3*283)),
     Am = (3/16)*(9 - sqrt(3*283)),
     ep = (-1 + sqrt(3) *I)/2,
     u = (Ap^(1/3) + ep*Am^(1/3))/3,
     r = (-sqrt(2)*u + sqrt(sqrt(2*u) - 2*u^2))
        / (2*sqrt(u)));
  check_equal(abs(r-rr[2]) < 1e-500, 1);

  u = (2/3)*sqrt(3)*sinh((1/3)*asinh((3/16)*sqrt(3)));
  r = (-sqrt(2)*u + sqrt(sqrt(2*u) - 2*u^2))
     / (2*sqrt(u));
  check_equal(abs(r-rr[2]) < 1e-500, 1);

  r = (-sqrt(2)*u + sqrt(-sqrt(2*u) - 2*u^2))
     / (2*sqrt(u));
  printf("real r  %8.f\n",real(r));
  printf("real r4 %8.f\n",real(rr[4]));
  printf("%8.f\n",imag(r));
  printf("%8.f\n",imag(rr[4]));
  check_equal(abs(real(r) - (-real(rr[4]))) < 1e-500, 1);
  check_equal(abs(imag(r) - imag(rr[4])) < 1e-500, 1);
}


\\-----------------------------------------------------------------------------
\\ A331692  continued fraction of sqrt(phi)
\\   1, 3, 1, 2, 11, 3, 5
\\ also a331692-contfrac-sqrt-phi--test.gp


\\-----------------------------------------------------------------------------
\\ A355575  pi Engel variant  (rejected)

make_pi_digits(len) =
{
  localprec(len+1000);
  digits(floor(Pi * 10^(len-1)));
}

\\ Pi - 3 = 3/(22)
\\        + 1/(22*9)
\\        + 4/(22*9*114)
\\        + 1/(22*9*114*35) + ...
{
  \\ my(Pi_digits=OEIS_data("A000796",'bfile));
  my(bits=1000,
     Pi_digits=make_pi_digits(bits));
 \\ Pi_digits = Pi_digits[1..300];
  print("Pi_digits length ",#Pi_digits);
  my(t_Pi=interv_frac_fromdigits(Pi_digits),
     want=[22, 9, 114, 35, 178, 511, 127, 570, 1050, 956, 2323, 8125, 29166, 41549, 76365, 231887, 1159745, 9508561, 48906603, 129211492, 366532653, 137182041, 413412878, 375119962, 989323988, 1656855777, 413670705959, 259516290562, 2392983002184, 14857501231366]);

  \\ 0 term means denominator factor 1
  if(0,
     my(nums=[1,1,1,1,1,1,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1],
        r=vector(21),
        a=vector(20));
     r[1]=Pi;
     for(n=1,#a,
        a[n] = floor((1 / r[n])*nums[n] + 1);
        r[n+1] = r[n]*a[n] - nums[n]);
     print(a);
     quit);

  my(debug=0);
  nums=Pi_digits;
  if(debug,
     for(i=1,#nums,
        if(nums[i]==0,
           print("nums["i"] = "nums[i]);
           break)));

  my(t=interv_sub_constant(t_Pi,3),
     limit=max(#want,40),
     ret=List([]));
  limit = oo;
  while(t[1]>0 && #ret<limit,
        my(pidig=nums[#ret+1]);

        my(d=floor(interv_div([pidig,pidig],t)));
        d=interv_add_constant(d,1);
        \\ my(d=ceil(interv_div([pidig,pidig],t)));

        if(debug,
           print("at "#ret" t="t*1.0);
           print("  pidig="pidig" d="d);
           print());
        if(d[1]!=d[2],
           if(debug, print("stopp for disagree"));
           break);
        listput(ret,d[1]);
        t=interv_sub_constant(t*d[1], pidig));
  if(debug, print("final t=",t*1.0));
  ret=Vec(ret);

  print("pseudo-Engel length ",#ret);
  check_equal(ret[1..#want], want,
              "A355575 pseudo-Engel expansion of pi");

  my(pidigit(n) = floor(Pi*10^(n-1)) % 10);
  check_equal(vector(6,n,pidigit(n)),
              [3, 1, 4, 1, 5, 9]);

  if(0,
     my(r=vector(10),
        a=vector(10));
     r[1]=Pi-3;
     for(n=1,9,
        a[n] = floor( (1/r[n])*pidigit(n) + 1);
        print("n="n" a[n] = "a[n]);
        r[n+1] = r[n]*a[n] - Pi_digits[n]);
     print(a);
     quit);
}

\\-----------------------------------------------------------------------------
\\ A356030  x^3 - x - 2
{
  my(want=[1, 5, 2, 1, 3, 7, 9, 7, 0, 6, 8, 0, 4, 5, 6, 7, 5, 6, 9, 6, 0, 4, 0, 8, 0, 8, 3, 2, 2, 5, 4, 4, 3, 8, 5, 1, 4, 4, 2, 8, 3, 8, 9, 8, 2, 8, 4, 2, 7, 9, 0, 3, 9, 0, 9, 0, 9, 0, 4, 9, 8, 0, 1, 5, 4, 2, 8, 1, 5, 6, 4, 0, 3, 4, 3, 0]);

  my(v=digits(15213797068045675696040808322544385144283898284279039090904980154281564));
  check_equal(want[1..#v],v);

  my(t = interv_solve_bisection
        ([1,2], x-> x^3 - x - 2, 1000));
  check_equal(interv_frac_digits(t,,#want),
              want);
}


\\-----------------------------------------------------------------------------
\\
{
  my(want=[5, 8, 9, 7, 5, 4, 5, 1, 2, 3, 0, 1, 4, 5, 8, 3, 8, 4, 2, 7, 8, 8, 0, 1, 7, 4, 7, 0, 9, 6, 0, 7, 1, 3, 6, 2, 4, 5, 1, 4, 3, 5, 1, 7, 3, 2, 2, 8, 3, 6, 2, 0, 7, 2, 3, 8, 8, 0, 7, 9, 3, 4, 1, 2, 3, 5, 0, 4, 8, 0, 2, 4, 1, 5, 7, 7]);

  my(v=digits(5897545123014583842788017470960713624514351732283620723880793412350480241));
  check_equal(want[1..#v],v);

  my(t = interv_solve_bisection
        ([0,1], x-> 2*x^3 + x - 1, 1000));
  check_equal(interv_frac_digits(t,,#want+1),
              concat(0,want));
}


\\-----------------------------------------------------------------------------
\\ A356585  num digits Gosper hyperfactorial n,n

\\ 1^(1^m)*2^(2^m)*3^(3^m)*...*n^(n^m)
Gosper_hyperfactorial(n,m) = prod(i=1,n, i^(i^m));

A330716(n) = Gosper_hyperfactorial(n,n);
OEIS_check_func("A330716",,['offset,0])

{
  my(want=[1, 1, 2, 16, 198, 2927, 50060, 979361, 21645853, 534381060, 14590180163, 436814197446, 14235563000269, 501817445873045, 19029286646922723, 772532087068933899, 33434018751249535666, 1536767964161539414904, 74769012084248550773909]);
  check_equal(vector(6,n,n--; #digits(A330716(n))),
              want[1..6]);

  \\ FIXME: This is not exact.
  my(log10_prec = 2^100);
  my(log10_interval(n) =
     localprec(1000);
     my(L=log(n)/log(10));
     L*=log10_prec;
     [floor(L),ceil(L)]/log10_prec);
  \\ print("log10 of 2 = ",log10_interval(2));

  my(Gosper_hyperfactorial_log10(n,m) =
     my(t=[0,0]);
     for(i=2,n,
        t = interv_add(t, log10_interval(i)*(i^m)));
     t);
  my(a(n) =
     if(n<=1, return(1));
     my(t=Gosper_hyperfactorial_log10(n,n),
        c=interv_ceil(t));
     if(c=='none,error());
     c);

  check_equal(Gosper_hyperfactorial(2,2), 16);
  check_equal(interv_ceil(Gosper_hyperfactorial_log10(2,2)), 2);
  \\ print(Gosper_hyperfactorial_log10(2,2)*1.0);

  check_equal(Gosper_hyperfactorial(3,3),
              1952152956156672, "3,3");
  check_equal(logint(Gosper_hyperfactorial(3,3),10),
              15, "logint");
  check_equal(interv_ceil(Gosper_hyperfactorial_log10(3,3)), 16);
  \\ print(Gosper_hyperfactorial_log10(2,2)*1.0);

  check_equal(vector(5,n,n--; a(n)),
              want[1..5],
              "five terms");
  check_equal(vector(#want,n,n--; a(n)),
              want,
              "DATA");
}

{
  my(want=[1, 1, 5, 51, 657, 9722, 166296, 3253365, 71905965, 1775175455, 48467529392, 1451065354742, 47289516677131, 1667001471950287, 63213921938077523, 2566296044236261518, 111065406214766719510, 5105032675471072965466, 248377281869637961805657]);
  check_equal(vector(6,n,n--; #binary(A330716(n))),
              want[1..6]);

  my(log2_prec = 2^200);
  my(log2_interval(n) =
     localprec(1000);
     my(L=log(n)/log(2));
     L*=log2_prec;
     [floor(L),ceil(L)]/log2_prec);
  \\ print("log2 of 3 = ",log2_interval(3)*1.0);

  my(Gosper_hyperfactorial_log2(n,m) =
     my(t=[0,0]);
     for(i=2,n,
        t = interv_add(t, log2_interval(i)*(i^m)));
     t);

  check_equal(#binary(16), 5);
  check_equal(Gosper_hyperfactorial(2,2),
              16, "2,2");

  check_equal(Gosper_hyperfactorial(3,3),
              1952152956156672, "3,3");
  check_equal(logint(Gosper_hyperfactorial(3,3),2),
              50, "logint");
  check_equal(interv_ceil(Gosper_hyperfactorial_log2(3,3)), 51);
  \\ print(Gosper_hyperfactorial_log2(2,2)*1.0);

  my(a(n) =
     if(n<=1, return(1));
     if(n==2, return(5));  \\ tricked by power of 2
     my(t=Gosper_hyperfactorial_log2(n,n),
        c=interv_ceil(t));
     if(c=='none,error());
     c);

  check_equal(vector(5,n,n--; a(n)),
              want[1..5],
              "binary five terms");
  check_equal(vector(#want,n,n--; a(n)),
              want,
              "binary DATA");
}





\\-----------------------------------------------------------------------------
\\ A105165 e base phi

\\ code by Chittaranjan Pardeshi in A105165
alist(len) =
{
  my(phi = quadgen(5), n=1, f=1, r=1/f, e=1+1, hi=e+r, t=0, w=phi^3);
  vector(len, i,
         w/=phi;
         while ( t+w < hi && t+w > e,
                n += 1;
                f *= n;
                r = 1/f;
                e += r;
                hi = e+r );
         if (t+w <= e, t += w; 1, 0));
}

e_interv(k) =
{
  my(lo=sum(i=0,k, 1/i!));
  [lo, lo+1/k!];
}

{
}

{
  my(t=e_interv(100),
     phi=quadgen(5));
  \\ print(t*1.0);
  foreach([["A001113",10, 1],
           ["A004593",2,  1/2],
           ["A004594",3,  1],
           ["A004595",4,  1],
           ["A004596",5,  1],
           ["A004597",6,  1],
           ["A004598",7,  1],
           ["A004599",8,  1],
           ["A004600",9,  1],
           ["A105165",phi, 1/phi^2]
          ], elem,
     my(anum,base,f); [anum,base,f] = elem;
     my(want=OEIS_data(anum));
     check_equal(interv_frac_digits(t*f,base,#want),
                 want,
                 Str("digits e base="base)));

  my(want=OEIS_data("A105165"));
  for(len=0,#want,
     alist(len) == want[1..len] || error());

  my(want=interv_frac_digits(t/phi^2,phi,1000));
  print("e base phi len ",#want);
  for(len=0,#want,
     alist(len) == want[1..len] || error());
}


\\-----------------------------------------------------------------------------
\\ Sqrt 21

{
  \\ sqrt21 as quad
  my(sqrt21=quadgen(4*21),
     t=[sqrt21,sqrt21]);
  check_equal(t[1]^2 == 21, 1, "sqrt21 quad");

  \\ Engel expansion of sqrt(21)
  if(0,
     print(interv_Engel_terms(t,10));
     \\ not in OEIS: 2,7,7,11,61,80,256,626,39928
     quit);

  \\ A248249 greedy Egyptian fractions sqrt(21)
  \\   4, 2, 13, 177, 344766
  my(want=OEIS_data("A248249"));
  my(got=interv_Egyptian_terms(interv_sub_constant(t,3),
                               #want));
  got[1] += 3;
  check_equal(got,want,
              "interv_Egyptian_terms() greedy sqrt21");

  foreach([["A010477",10]
          ], elem,
     my(anum,base); [anum,base] = elem;
     my(want=OEIS_data(anum));
     check_equal(interv_frac_digits(t,base,#want),
                 want,
                 Str("digits sqrt21 base="base)));
  \\ hexadecimal
  \\ not in OEIS: 9, 5, 2, 3, 10, 14, 4, 5, 4, 7, 10
}


\\-----------------------------------------------------------------------------
\\ A067614 Continued Fraction of sqrt(prime(n)) Second Term

A067614(n) = my(r); 2*sqrtint(prime(n),&r) \ r;
{
  OEIS_check_func("A067614",,['offset,1]);

  my(by_quad(n) = my(x=quadgen(4*prime(n)));
     x = frac(x);
     x = 1/x;
     floor(x));
  check_equal(vector(10000,n, A067614(n)),
              vector(10000,n, by_quad(n)),
              "by_quad");

  my(by_interv(n) = my(p=prime(n),
                       t=interv_solve_bisection
                         ([0,p], x->x^2 - p, 100));
     interv_contfrac(t,2)[2]);
  check_equal(vector(1000,n, A067614(n)),
              vector(1000,n, by_interv(n)),
              "by_interv");

  my(by_formula(n) = my(p=prime(n),
                        s=sqrtint(p),
                        r=p-s^2);
     floor(2*s/r));
  check_equal(vector(10000,n, A067614(n)),
              vector(10000,n, by_formula(n)),
              "by_interv");
}
A000006(n) = sqrtint(prime(n));
OEIS_check_func("A000006",,['offset,1]);

A056892(n) = my(r); sqrtint(prime(n),&r); r;
OEIS_check_func("A056892",,['offset,1]);

{
  my(by_anums(n) = my(s=A000006(n),r=A056892(n));
     floor(2*s/r));
  check_equal(vector(10000,n, A067614(n)),
              vector(10000,n, by_anums(n)),
              "by_anums");
}

\\ vector(20,n, my(r,s=sqrtint(prime(n),&r)); s\r)
\\ not in OEIS: 1, 0, 2, 0, 1, 0, 4, 1, 0, 1, 0, 6, 1, 0, 0, 1, 0, 0, 2, 1

\\ third term of contfrac
\\ vector(10,n, interv_contfrac(vector(2,i,quadgen(4*prime(n))),3)[3])
\\ not in OEIS: 2, 2, 4, 1, 6, 1, 8, 1, 3, 1


\\-----------------------------------------------------------------------------
\\ A352678 sqrt2 base phi

{
  my(want=OEIS_data("A352678"),
     t_sqrt2 = interv_solve_bisection
              ([1,2], x->x^2-2, 1000));
  my(phi=quadgen(5));
  my(got=interv_frac_digits(t_sqrt2,phi,#want));
  check_equal(got,want,
              "sqrt2 base phi");

  my(got=digits(1 01000001010010100100000001010000000001010101010101001));
  check_equal(got,want[1..#got], "example");

  \\ my code
  my(a_vector(len) = my(t=0,w=1);
     vector(len,i,
            my(tw=t+w); w/=phi;
            if(tw^2 <= 2, t=tw;1, 0));
    );
  check_equal(a_vector(#want),want);

  \\ code by Chittaranjan Pardeshi in A352679
  my(alist(len) =
     my(phi = quadgen(5), w=phi, t = 0);
     vector(len, i, w /= phi;
            if ( ( t + w )^2 <= 2, t += w ; 1, 0)));
  my(by_interv=interv_frac_digits(t_sqrt2,phi));
  print("sqrt2 by_interv ldnegth ",#by_interv);
  check_equal(by_interv,alist(#by_interv));
}

\\-----------------------------------------------------------------------------
\\ A352679 sqrt3 base phi

{
  my(want=OEIS_data("A352679"),
     prec_bits = 1000,
     t_sqrt3 = interv_solve_bisection
              ([1,2], x->x^2 - 3, prec_bits),
     t_phi = interv_solve_bisection
              ([1,2], x-> x^2-x-1, prec_bits),
     t_sqrt3_over_phi = interv_div(t_sqrt3,t_phi));
  \\ check_equal(interv_contains(t_sqrt3, sqrt(3)), 1);
  my(phi=quadgen(5));
  my(got=interv_frac_digits(t_sqrt3_over_phi,phi,#want));
  check_equal(got,want,
              "sqrt3 base phi");

  my(a_vector(len) = my(t=0,w=phi);
     vector(len,i,
            my(tw=t+w); w/=phi;
            if(tw^2 <= 3, t=tw;1, 0));
    );
  check_equal(a_vector(#want),want,
              "sqrt3 base phi a_vector()");

  \\ code by Chittaranjan Pardeshi in A352679
  my(alist(len) =
     my(phi = quadgen(5), w=phi^2, t =0);
     vector(len, i,
            w /= phi;
            if ( ( t + w )^2 <= 3,
                t += w; 1,  0)));
  my(by_interv=interv_frac_digits(t_sqrt3_over_phi,phi));
  print("sqrt3 by_interv ldnegth ",#by_interv);
  check_equal(by_interv,alist(#by_interv));
}

\\-----------------------------------------------------------------------------
\\ A352594 sqrt(phi) base phi

{
  my(want=OEIS_data("A352594"));

  \\ 1.2360679
  \\ 1.2720196
  \\ sqrt(phi)
  my(a(n)=want[n]);
  \\ print(sum(i=1,#want, 0. + a(i)/phi^(i-1))); quit;
  \\ print(0. + a(1) + a(2)/phi + a(3)/phi^2 + a(4)/phi^3 + a(5)/phi^4); quit;

  my(got=digits(1 001000100000010010001000001000100010101010100010101));
  check_equal(got,want[1..#got], "example");

  \\ my code
  my(a_vector(len) = my(t=0,w=1);
     vector(len,i,
            my(tw=t+w); w/=phi;
            if(tw^2 <= phi, t=tw;1, 0));
    );
  check_equal(a_vector(#want),want);

  my(t_sqrt_phi = interv_solve_bisection
                 ([1,2], x->x^4-x^2-1, 1000));
  \\ print(t_sqrt_phi*1.0);

  \\ code by Chittaranjan Pardeshi in A352594
  my(alist(len) =
     my(phi = quadgen(5), w=phi, t = 0);
     vector(len, i,
            w = w / phi;
            if ( ( t + w )^2 <= phi,
                t = t + w ;
                1,
                0)));
  my(by_interv=interv_frac_digits(t_sqrt_phi,phi));
  print("sqrt(phi) by_interv ldnegth ",#by_interv);
  check_equal(by_interv,alist(#by_interv));
}

\\-----------------------------------------------------------------------------
\\ A173995 Contfrac of Sum 1/Fermat primes A019434
\\ 0, 1, 1, 2, 9, 1, 3, 5, 1, 2, 1, 1,
\\ 1, 1, 3, 1, 7, 1, 31, 1, 2, 4, 5

\\ A019434 = 3, 5, 17, 257, 65537

{
  \\ next at k=32, but that overflows in 32-bits

  \\ 0, 1, 1, 2, 9, 1, 3
  my(v=OEIS_data("A019434"),
     f=sum(i=1,#v,1/v[i]),
     k=23,
     e=1/( 2^(2^k) + 1 ),
     t=[f, f+e],
     c=interv_contfrac(t),
     want=OEIS_data("A173995"));
  print(#c);
  print(#want);
  check_equal(want[1..#c], c);

  \\ [0, 1, 1, 2, 9, 1, 3, 5, 1, 2, 1, 1, 1, 1, 3, 1, 7, 1, 31, 1, 2, 4, 5]
}



\\-----------------------------------------------------------------------------
\\ A352677 largest same value binary and base phi
\\
\\ digits 11.xxx  is (2-phi) + (1-1) above
\\ radix point

my(phi=quadgen(5)); \
A352677_vector(len) =
{
  my(r=2-phi,b=1,p=1);
  vector(len,i,
         if(i<3,1, b/=2;p/=phi;
            my(new=r-p+b); if(new>=0,r=new;1,0)));
}
{
  my(want=[1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0]);
  check_equal(A352677_vector(#want), want);
  check_equal(vecsum(A352677_vector(1000)), 276);
}
\\ print(A352677_vector(137));quit


\\-----------------------------------------------------------------------------
\\ sqrt(phi)
\\
\\ A352501  lower
\\    11, 386, 2441, 25748423, 637519684,
\\ A352502  higher
\\    14, 491, 3105, 32752500, 810937565,

\\ digits of sqrt(s) in base phi
sqrt_base_phi_digits(s,len) =
{
  \\ t^2 <= s
  my(t=0, k=0);
  while(phi^k<s,k++);
  vector(len,i, k--;
         if((t+phi^k)^2 <= s,
            t += phi^k; 1,
            0));
}

sqrt_phi_base_phi_digits(len) =
{
  \\ t^2 <= s
  \\ (T/phi^k)^2 <= S/phi^k
  \\ my(r=0,s=1/phi);
  \\ vector(len,i,
  \\        t*=phi; s*=phi^2;
  \\        if((t+1)^2 <= s, t++;1, 0));

  \\ my(t=0,s=phi);
  \\ vector(len,i, my(d=(t+1)^2<=s);
  \\        t+=d;t*=phi; s*=phi^2; d);

  my(t=0,s=phi);
  vector(len,i, my(d=(t+1)^2<=s);
         t=(t+d)*phi; s*=phi^2; d);
}
\\ sqrt_phi_base_phi_digits(15)

{
  my(t=interv_solve_bisection([1,2], x->x^4-x^2-1, 1000));
  \\ sqrt(phi)
  \\ polroots(x^4-x^2-1)
  \\ print(t*1.0);

  \\ continued fraction terms
  my(c=interv_contfrac(t));
  my(want=OEIS_data("A331692"));
  \\ print(c);
  \\ print(want);
  check_equal(c[1..#want],want,
              "A331692 sqrt(phi) continued fraction");

  my(want_lower=[
    11, 386, 2441, 25748423, 637519684,
    2799936925, 3934324789543,
    127501370029150, 21274660147684109,
    644571595359295797,
    15845190736671957299,
    995980378496501932493,
    47375682236837399943653,
    213688560255016550712685,
    28372206851301867342910959 ]);

  \\ A225204 convergents numreators
  \\ 1, 4, 5, 14, 159, 491, 2614, 3105, 33664
  my(A225204(n) =
     numerator(test_contfrac_to_rational(c[1..n])));
  my(A225205(n) =
     denominator(test_contfrac_to_rational(c[1..n])));

  OEIS_check_func("A225204",A225204,['offset,1]);
  OEIS_check_func("A225205",A225205,['offset,1]);

  \\ print(select(n->issquare(A225204(n)) && issquare(A225205(n)), [1..100])); quit;

  if(0,
     read("various-Fibbinary.gp");
     my(got_indices=select(n->
       A225204(n)^2 ==
       Zeckendorf_shift_up((A225205(n)^2)),
               [1..#c]));
     print(got_indices);
     \\ not in OEIS: 4, 6, 8, 16, 20, 21, 26, 28, 35, 38, 41, 45, 48

     my(got_lower=apply(A225205,got_indices));
     check_equal(got_lower[1..#want_lower],
                 want_lower);

     for(n=1,20,
        my(f=test_contfrac_to_rational(c[1..n]),
           num=numerator(f),
           den=denominator(f));
        print(f);
        if(setsearch(want_lower,denominator(f)),
           print("  ***");
           print("  "to_Zeckendorf(num^2));
           print("  "to_Zeckendorf(den^2));
          );
        print();
     );
     quit);

  \\ print(interv_Engel_terms(t,15));quit
  \\ not in OEIS: 1, 4, 12, 18, 41, 42, 94, 162, 252

  \\ A139339 decimal digits sqrt(phi)
  my(want=OEIS_data("A139339"),
     got=interv_frac_digits(t,,#want));
  check_equal(got,want);

  \\ A352594 sqrt(phi) greedy base phi
  my(want=OEIS_data("A352594"),
     got=interv_frac_digits(t,phi,#want));
  check_equal(got,want);

  check_equal(sqrt_base_phi_digits(phi,#want),
              want);
  check_equal(sqrt_phi_base_phi_digits(#want),
              want);

  my(phi=quadgen(5));
  my(a_vector(len) = my(t=0,s=phi); vector(len,i, my(d=(t+1)^2<=s); t=(t+d)*phi; s*=phi^2; d);
     );
  check_equal(a_vector(#want), want);
}

\\ r = c + 1/r'
\\ r' = 1/(r-c) = 1/frac(r)

\\ phi = (sqrt5+1)/2     x^2-x-1
\\ s = sqrt(phi)         x^4-x^2-1
\\ floor(s) = sqrt( (sqrt5+1)/2 )
\\          = sqrt( 2*sqrt5 + 2 ) / 2
\\ phi = (c^2+1)/2
\\ r = (f*sqrt(sqrt5 + 1) + x)/d

\\ r = f/g*sqrt(phi) + x/y
\\   = sqrt( (sqrt(5*f^4/g^4) + f^2/g^2*)/2 ) + x/y
\\   = sqrt( 2*(sqrt(5*f^4/g^4) + f^2/g^2*) )/2 + x/y
\\   = (sqrt( 2*y^2*(sqrt(5*f^4/g^4) + f^2/g^2*) ) + x)/(2*y)

\\   =
\\ floor(r)
\\ 1/r = f/g*sqrt(phi)-x/y

{
  my(sqrt5=quadgen(20),
     phi=(1+sqrt5)/2);
  check_equal(x + y*phi, x+y/2 + y/2*sqrt5);
  check_equal(1/(x + y*phi) ==
              (x+y/2 - y/2*sqrt5)
              / ((x+y/2)^2 - y^2/4*5),  1);

  check_equal(1/(x + y*phi) ==
              (4*x+2*y - 2*y*sqrt5)
              / ((2*x+y)^2 - 5*y^2),  1);

  check_equal(sqrt5, 2*phi-1);
  check_equal(1/(x + y*phi) ==
              (4*x + 4*y - 4*y*phi)
              / ((2*x+y)^2 - 5*y^2),  1);
}
\\ phi=quadgen(5)
\\ 1/(x + y*phi)


\\-----------------------------------------------------------------------------
\\ Sqrt 6

{
  \\ sqrt6 as quad
  my(t=[quadgen(24),quadgen(24)]);
  check_equal(t[1]^2 == 6, 1, "sqrt6 quad");

  \\ decimal digits
  my(want=OEIS_data("A010464"),
     got=interv_frac_digits(t,,#want));
  check_equal(got,want);

  \\ continued fration
  my(want=OEIS_data("A040003"),
     got=interv_contfrac(t,#want));
  check_equal(got,want);

  \\ A?????? Engel expansion of sqrt(6)
  \\ not in OEIS: 3, 23, 23, 49, 80, 121, 124
  \\ my(want=OEIS_data("A123456"));
  \\ check_equal(interv_Engel_terms(t,#want),
  \\             want,
  \\             "interv_Engel_terms() of sqrt6");

}

\\-----------------------------------------------------------------------------
\\ Sqrt 2

{
  \\ sqrt2 as quad

  my(sqrt2=quadgen(8),
     t=[sqrt2,sqrt2]);
  check_equal(sqrt2^2 == 2, 1, "sqrt2 quad");

  \\ A028254 Engel expansion of sqrt(2)
  my(want=OEIS_data("A028254"));
  check_equal(interv_Engel_terms(t,#want),
              want,
              "interv_Engel_terms() of sqrt2");

  \\ A006487 greedy Egyptian fractions sqrt(2)
  \\   1, 3, 13, 253, 218201, 61323543802,
  my(want=OEIS_data("A006487"));
  check_equal(interv_Egyptian_terms(t,#want),
              want,
              "interv_Egyptian_terms() sqrt2");
  my(t=interv_from_Egyptian(want));
  check_equal(t[1]^2 < 2, 1,
             "interv_from_Egyptian() sqrt2 surround");
  check_equal(t[2]^2 > 2, 1);

  foreach([["A004539",2],
           ["A004540",3],
           ["A004541",4],
           ["A004542",5],
           ["A004543",6],
           ["A004544",7],
           ["A004545",8],
           ["A004546",9],
           ["A002193",10]], elem,
     my(anum,base); [anum,base] = elem;
     my(want=OEIS_data(anum));
     check_equal(interv_frac_digits(t,base,#want),
                 want,
                 Str("digits sqrt2 base="base)));
  \\ hexadecimal
  \\ not in OEIS: 0, 9, 14, 6, 6, 7, 15, 3

  \\-----------------
  \\ A069139 greedy Egyptian fractions 1/sqrt(2)
  \\   2, 5, 141, 68575, 32089377154,
  my(want=OEIS_data("A069139"));
  check_equal(interv_Egyptian_terms(t/2,#want),
              want,
              "interv_Egyptian_terms() greedy 1/sqrt2");
}

{
  my(t=interv_solve_bisection([1,2], x->x^2 - 2, 1000));

  \\ \\ sqrt2 in base phi
  \\ my(phi=quadgen(5));
  \\ my(want=OEIS_data("A123456"),
  \\    got=interv_frac_digits(t,phi,#want));
  \\ check_equal(got,want);
}


\\-----------------------------------------------------------------------------
\\ A336686 decimal Sum_{k>=0} 1/(k!)!
\\ A336810 continued fraction
\\ moderately wild terms in contrfrac

{
  my(want=OEIS_data("A336686"),
     limit = 6,
     x = sum(k=0,limit, 1/(k!)!),
     t=[x,x+10*1/(limit!)!]);
  check_equal(interv_frac_digits(t,,#want), want);

  my(want=OEIS_data("A336810"));
  check_equal(interv_contfrac(t,#want), want);
  my(want_t=interv_from_contfrac(want));
  check_equal(interv_contains(want_t,t), 1);
}


\\-----------------------------------------------------------------------------
\\ A249270 decimal sum 1/(product of primes <= k)
\\ A346801 Engel expansion

\\ Return an interval for sum 1/(prod primes < k).
\\ FIXME: The upper bound is not very good.
t_sum_prime_product_recips(k) =
{
  my(d=1,ret=0);
  for(i=0,k-1,
     if(isprime(i),d/=i);
     \\ print(d);
     ret+=d);
  [ret,ret+10*d];
}

{
  my(want = OEIS_data("A346801"),
     t = t_sum_prime_product_recips(500));
  check_equal(interv_Engel_terms(t,#want),
              want);

  \\ decimal digits
  my(want=OEIS_data("A249270"),
     want_t=interv_frac_fromdigits(want));
  check_equal(interv_contains(want_t,t), 1);
}

\\-----------------------------------------------------------------------------
\\ Champernowne Decimals

{
  \\ A077771 decimal of
  \\ A077772 contfrac of
  \\ ternary 1, 2, 1,0, 1,1, 1,2, etc all ternary numbers

  my(want=OEIS_data("A077771"),
     champ=concat(vector(500,n,digits(n,3))),
     t=interv_frac_fromdigits(champ,3,0),
     got=interv_frac_digits(t,,#want+1));
  check_equal(got,concat(0,want), "A077771 decimals");

  my(want=OEIS_data("A077772"),  \\ contfrac
     got=interv_contfrac(t,#want));
  check_equal(got,want, "A072117 contfrac");
  my(want_t=interv_from_contfrac(want));
  check_equal(interv_contains(want_t,t), 1);

  my(v=[1, 2, 1,0, 1,1, 1,2]);
  check_equal(fromdigits(v,3), 3929);
  \\ print(fromdigits(v,3)/3.^#v); quit;
  \\ 0.5988416399 per A077771
}


\\------------------------------------------------------------------------------
\\ plastic constant root of x^3-x-1
\\ A060006 decimal digits
\\ A072117 contfrac

{
  my(want=OEIS_data("A060006"),
     t=interv_solve_bisection([1,2], x->x^3-x-1, 500),
     got=interv_frac_digits(t,,#want));
  check_equal(got,want, "A060006 decimals");

  my(want=OEIS_data("A072117"),  \\ contfrac
     got=interv_contfrac(t,#want));
  check_equal(got,want, "A072117 contfrac");
  my(want_t=interv_from_contfrac(want));
  check_equal(interv_contains(want_t,t), 1);
}


\\------------------------------------------------------------------------------
\\ A181313 contfrac of sum 1/2^Fibonacci(k) for k>=1
{
  my(v=vector(1000));
  for(k=1,oo,
     my(f=fibonacci(k),
        i=f+1);
     if(i>#v,break);
     v[i]++);
  my(t=interv_frac_fromdigits(v,2),
     got=interv_contfrac(t),
     want=OEIS_data("A181313"));
  \\ print(t*1.0);
  got=got[1..#want];
  check_equal(got,want,
             "A181313 contfrac");
}

\\ A006518  contfrac for k >= 2
{
  my(v=vector(1000));
  for(k=2,oo,
     my(f=fibonacci(k),
        i=f+1);
     if(i>#v,break);
     v[i]++);
  my(t=interv_frac_fromdigits(v,2),
     got=interv_contfrac(t),
     want=OEIS_data("A006518"));
  got=got[1..#want];
  check_equal(got,want,
             "A006518 contfrac");

  t=interv_add_constant(t,1/2);
  \\ print(t*1.0);
  my(want=OEIS_data("A084119"));
  got=interv_frac_digits(t,,#want);
  check_equal(got,want,
             "A006518 decimals");
}


\\------------------------------------------------------------------------------
\\ A082665
progress("A082665");
{
  my(samples=OEIS_data("A082665"),
     t_samples = interv_frac_fromdigits(samples),
     n_max = 100,
     L,
     s = sum(n=3,n_max, L = 1/2^floor(n^log(log(n)))),
     t_sum = [s, s+L]);
  print(t_samples[1]*1.0);
  print(t_sum[1]*1.0);
  check_equal(interv_contains(t_samples,t_sum), 1);
}


\\-----------------------------------------------------------------------------
\\ Sum 1/2^Fibonacci(n)
progress("A084119");

sum_of_fib_to_k(k,start=0) =
{
  my(s=sum(i=start,k, 1/2^fibonacci(i)));
  [s, s + 1/2^fibonacci(k)];
}
sum_of_fib_k_of_bits(bits) = ceil(log(bits)/log(phi) + 5);

{
  my(samples=OEIS_data("A084119"),
     samples_bits = ceil(#samples * log2(10)),
     k = sum_of_fib_k_of_bits(samples_bits));
  print(k);
  my(t_samples = interv_frac_fromdigits(samples),
     t_sum = sum_of_fib_to_k(k,1));
  print(t_samples[1]*1.0);
  print(t_sum[1]*1.0);
  check_equal(interv_contains(t_samples,t_sum), 1);
}
{
  \\ A084119 decimal digits of sum 1/2^Fibonacci
  my(samples=OEIS_data("A084119",'bfile),
     samples_bits = ceil(#samples * log2(10)),
     k = sum_of_fib_k_of_bits(samples_bits),
     t_samples = interv_frac_fromdigits(samples),
     t_sum = sum_of_fib_to_k(k,1));
  print(samples_bits" "k);
  print(t_samples[1]*1.0);
  print(t_sum[1]*1.0);
  check_equal(interv_contains(t_samples,t_sum), 1,
             "A084119 1/2^Fibonacci(n) decimals");
}


\\-----------------------------------------------------------------------------
\\ A100322 Pi^k frac digits n

{
  my(Pi_digits=OEIS_data("A000796",'bfile)[1..300],
     t_Pi=interv_frac_fromdigits(Pi_digits));

  my(a_vector(len) =
     my(ret=vector(len),
        nlenmax=logint(len,10)+1,
        t_pow=[1,1],
        k=0);
     while(hammingweight(ret) < #ret,
           k++;
           t_pow = interv_mul(t_pow, t_Pi);
           my(int=floor(t_pow));
           int[1]==int[2] || error("k="k" not enough precision");
           my(t=interv_frac(t_pow));
           for(nlen=1,nlenmax,
              my(n=interv_floor_to_constant(t*10^nlen););
              if(n==0,break); \\ no initial 0 digit
              if(n<=#ret && !ret[n],
                 print("found k="k" has n="n);
                 ret[n]=k)));
     ret);

  my(want=OEIS_data("A100322"));
  if(2, \\ shorten
     want=want[1..5]);
  my(got=a_vector(#want));
  \\ my(got=a_vector(75));
  check_equal(got[1..#want], want);

  \\ Pi^k fraction part starts with k
  \\ 1, 4, 75, 9424, 12669, 331783
  if(#got >= 75,
     my(want=OEIS_data("A100323"));
     check_equal(select(n->got[n]==n,[1..#got]),
                 want[1..3]));
}
{
  \\ 1, 4, 75, 9424, 12669, 331783
  if(0,  \\ fairly slow
     my(Pi_digits=OEIS_data("A000796",'bfile)[1..5000],
        t_Pi=interv_frac_fromdigits(Pi_digits),
        t=interv_pow_constant(t_Pi,9424));
     t=interv_frac(t);
     check_equal(interv_floor(t*10000), 9424));
}


\\------------------------------------------------------------------------------
check_equal(a,'a);
check_equal(t,'t, "no global t");
check_equal(x,'x);
check_equal(want,'want);

progress("end");
